@@ -4,16 +4,16 @@
  * Input of Calma GDS-II stream format.
  * Processing of paint (paths, boxes, and boundaries) and text.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -98,7 +98,9 @@ calmaInputRescale(n, d)
 	{
 	    /* Scale the GDS planes in this cell's cd_client record */
 	    Plane **gdsplanes = (Plane **)def->cd_client;
-	    CIFScalePlanes(n, d, gdsplanes);
+	    /* Should not happen, but punt if client record is not set; */
+	    if (def->cd_client != (ClientData)0)
+		CIFScalePlanes(n, d, gdsplanes);
 	}
     }
 
@@ -182,7 +184,7 @@ calmaReadPoint(p, iscale)
     p->p_y /= calmaReadScale2;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -231,7 +233,7 @@ calmaElementBoundary()
 	plane = cifCurReadPlanes[ciftype];
 
     /* Read the path itself, building up a path structure */
-    if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1)) 
+    if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1))
     {
 	if (plane != NULL)
 	    CalmaReadError("Error while reading path for boundary/box; ignored.\n");
@@ -345,7 +347,7 @@ calmaElementBoundary()
 	DBPlaceCell(use, cifReadCellDef);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -500,10 +502,10 @@ calmaElementPath()
      * Allow zero-width paths; we will ignore them later.
      */
     width = 0;
-    PEEKRH(nbytes, rtype) 
+    PEEKRH(nbytes, rtype)
     if (nbytes > 0 && rtype == CALMA_WIDTH)
     {
-	if (!calmaReadI4Record(CALMA_WIDTH, &width)) 
+	if (!calmaReadI4Record(CALMA_WIDTH, &width))
 	{
 	    CalmaReadError("Error in reading WIDTH in calmaElementPath()\n") ;
 	    return;
@@ -556,7 +558,7 @@ calmaElementPath()
 
     /* Read the points in the path */
     savescale = calmaReadScale1;
-    if (!calmaReadPath(&pathheadp, 2)) 
+    if (!calmaReadPath(&pathheadp, 2))
     {
 	CalmaReadError("Improper path; ignored.\n");
 	return;
@@ -712,7 +714,7 @@ calmaElementText()
     cifnum = CIFCalmaLayerToCifLayer(layer, textt, cifCurReadStyle);
     if (cifnum < 0)
     {
-	if (cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS) 
+	if (cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS)
 	    type = -1;
 	else {
 	    calmaLayerError("Label on unknown layer/datatype", layer, textt);
@@ -779,7 +781,7 @@ calmaElementText()
 	int width;
 
 	/* Use WIDTH value to set the font size */
-	if (!calmaReadI4Record(CALMA_WIDTH, &width)) 
+	if (!calmaReadI4Record(CALMA_WIDTH, &width))
 	{
 	    CalmaReadError("Error in reading WIDTH in calmaElementText()\n") ;
 	    return;
@@ -863,7 +865,7 @@ calmaElementText()
      * a flag for that in the "cifoutput" section of the techfile.
      */
 
-#if 0 
+#if 0
 
     {
 	static bool algmsg = FALSE;
@@ -872,7 +874,7 @@ calmaElementText()
 	char *savstring;
 	for (cp = textbody; *cp; cp++)
 	{
-	    if (*cp <= ' ' | *cp > '~') 
+	    if (*cp <= ' ' | *cp > '~')
 	    {
 		if (!changed)
 		{
@@ -881,7 +883,7 @@ calmaElementText()
 		}
 		if (*cp == '\r' && *(cp+1) == '\0')
 		    *cp = '\0';
-		else if (*cp == '\r') 
+		else if (*cp == '\r')
 		    *cp = '_';
 		else if (*cp == ' ')
 		    *cp = '_';
@@ -920,10 +922,11 @@ calmaElementText()
     }
     else if (type < 0)
     {
-	CalmaReadError("Warning:  label \"%s\" at (%d, %d) is on unhandled"
-		" layer:purpose pair %d:%d and will be discarded.\n", textbody,
-		r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
-		r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor, layer, textt);
+	if (!(cifCurReadStyle->crs_flags & CRF_IGNORE_UNKNOWNLAYER_LABELS))
+	    CalmaReadError("Warning:  label \"%s\" at (%d, %d) is on unhandled"
+		    " layer:purpose pair %d:%d and will be discarded.\n", textbody,
+		    r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
+		    r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor, layer, textt);
     }
     else
     {
@@ -979,13 +982,19 @@ calmaElementText()
 
 	    /* No port information can be encoded in the GDS file, so	*/
 	    /* assume defaults, and assume that the port order is the	*/
-	    /* order in which labels arrive in the GDS stream.		*/
+	    /* order in which labels arrive in the GDS stream.  If	*/
+	    /* ports have the same text, then give them the same index.	*/
 
 	    i = -1;
 	    for (sl = cifReadCellDef->cd_labels; sl != NULL; sl = sl->lab_next)
 	    {
 		idx = sl->lab_flags & PORT_NUM_MASK;
 		if (idx > i) i = idx;
+		if ((idx > 0) && (sl != lab) && !strcmp(sl->lab_text, textbody))
+		{
+		    i = idx - 1;
+		    break;
+		}
 	    }
 	    i++;
 	    lab->lab_flags |= (PORT_NUM_MASK & i);
@@ -997,7 +1006,7 @@ calmaElementText()
     /* done with textbody */
     if (textbody != NULL) freeMagic(textbody);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1104,7 +1113,7 @@ calmaReadPath(pathheadpp, iscale)
     }
     return (*pathheadpp != NULL);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1135,6 +1144,15 @@ calmaLayerError(mesg, layer, dt)
     CalmaLayerType clt;
     HashEntry *he;
 
+    /* Ignore errors for cells that are marked as read-only, since  */
+    /* these are normally expected to have unhandled layer types,   */
+    /* since the purpose of read-only cells is to preserve exactly  */
+    /* layout in the cell which may not be represented in the tech  */
+    /* file.							    */
+
+    if ((cifReadCellDef->cd_flags & CDVENDORGDS) == CDVENDORGDS)
+	return;
+
     clt.clt_layer = layer;
     clt.clt_type = dt;
     he = HashFind(&calmaLayerHash, (char *) &clt);
