@@ -3,16 +3,16 @@
  *
  * Output of Calma GDS-II stream format.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -53,9 +53,12 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include "utils/stack.h"
 
     /* Exports */
+bool CalmaDoLibrary = FALSE;	 /* If TRUE, do not output the top level */
 bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
 bool CalmaDoLower = TRUE;	 /* If TRUE, allow lowercase labels. */
 bool CalmaFlattenArrays = FALSE; /* If TRUE, output arrays as individual uses */
+bool CalmaAddendum = FALSE;	 /* If TRUE, do not output readonly cell defs */
+bool CalmaNoDateStamp = FALSE;	 /* If TRUE, output zero for creation date stamp */
 
     /* Experimental stuff---not thoroughly tested (as of Sept. 2007)! */
 bool CalmaContactArrays = FALSE; /* If TRUE, output contacts as subcell arrays */
@@ -118,7 +121,7 @@ typedef struct BT1 {
     int bt_points;		/* Number of points in this list */
     struct BT1 *bt_next;	/* Next polygon record */
 } BoundaryTop;
-    
+
 /*--------------------------------------------------------------*/
 
 /* Number assigned to each cell */
@@ -146,7 +149,7 @@ HashTable calmaPrefixHash;
 /* Imports */
 extern time_t time();
 
-
+
 /* -------------------------------------------------------------------- */
 
 /*
@@ -238,7 +241,7 @@ static char calmaMapTablePermissive[] =
     'x',  'y',  'z',  '{',  '|',  '}',  '~',    0,	/* x   - DEL */
 };
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -328,7 +331,7 @@ CalmaWrite(rootDef, f)
      * to insure that each child cell is output before it is used.  The
      * root cell is output last.
      */
-    (void) calmaProcessDef(rootDef, f);
+    (void) calmaProcessDef(rootDef, f, CalmaDoLibrary);
 
     /* Finish up by outputting the end-of-library marker */
     calmaOutRH(4, CALMA_ENDLIB, CALMA_NODATA, f);
@@ -340,7 +343,7 @@ CalmaWrite(rootDef, f)
 	TxPrintf("%d problems occurred.  See feedback entries.\n", problems);
 
     /*
-     * Destroy all contact cell definitions 
+     * Destroy all contact cell definitions
      */
     if (CalmaContactArrays) calmaDelContacts();
 
@@ -348,7 +351,7 @@ CalmaWrite(rootDef, f)
     HashKill(&calmaPrefixHash);
     return (good);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -372,7 +375,7 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
     char *filename;
 {
     int nbytes, rtype;
-    char *strname = NULL, *newnameptr, newname[CALMANAMELENGTH*2];
+    char *strname = NULL, *newnameptr;
     HashEntry *he, *he2;
     CellDef *edef;
     char *prefix = NULL;
@@ -394,7 +397,10 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 
     /* Output structure begin */
     calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
-    calmaOutDate(def->cd_timestamp, outf);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), outf);
+    else
+    	calmaOutDate(def->cd_timestamp, outf);
     calmaOutDate(time((time_t *) 0), outf);
 
     /* Find the structure's unique prefix, in case structure calls subcells */
@@ -444,14 +450,16 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 	if (edef != NULL)
 	{
 	    bool isAbstract, isReadOnly;
-	    char *chklibname;
+	    char *chklibname, *dotptr;
 
 	    /* Is view abstract? */
 	    DBPropGet(edef, "LEFview", &isAbstract);
 	    chklibname = (char *)DBPropGet(edef, "GDS_FILE", &isReadOnly);
+	    dotptr = strrchr(filename, '.');
+	    if (dotptr) *dotptr = '\0';
 
-	    /* Is the library name the same? */
-	    if (isAbstract && !strcmp(filename, chklibname))
+	    /* Is the library name the same as the filename (less extension)? */
+	    if (isAbstract && isReadOnly && !strcmp(filename, chklibname))
 	    {
 		/* Same library, so keep the cellname and mark the cell */
 		/* as having been written to GDS.			*/
@@ -468,11 +476,12 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 		    newnameptr = strname;   /* Should never happen */
 		else
 		{
-		    newnameptr = mallocMagic(strlen(strname) + strlen(prefix) + 9);
-		    sprintf(newnameptr, "1%s_%s", prefix, strname);
+		    newnameptr = mallocMagic(strlen(strname) + strlen(prefix) + 8);
+		    sprintf(newnameptr, "1%s%s", prefix, strname);
 		    HashSetValue(he, (char *)newnameptr);
 		}
 	    }
+	    if (dotptr) *dotptr = '.';
 	}
 	else
 	{
@@ -482,8 +491,8 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 		newnameptr = strname;	    /* Should never happen */
 	    else
 	    {
-		newnameptr = mallocMagic(strlen(strname) + strlen(prefix) + 9);
-		sprintf(newnameptr, "1%s_%s", prefix, strname);
+		newnameptr = mallocMagic(strlen(strname) + strlen(prefix) + 8);
+		sprintf(newnameptr, "1%s%s", prefix, strname);
 		HashSetValue(he, (char *)newnameptr);
 	    }
 	}
@@ -495,7 +504,7 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
     /* for handling any AREF or SREF names, which need name	*/
     /* checks.							*/
 
-    while (1) 
+    while (1)
     {
 	int datatype;
 
@@ -535,12 +544,12 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 		    /* the same way used for structure definitions.	*/
 
 		    newnameptr = (char *)mallocMagic(strlen(strname) +
-				strlen(prefix) + 9);
-		    sprintf(newnameptr, "0%s_%s", prefix, strname);
+				strlen(prefix) + 8);
+		    sprintf(newnameptr, "0%s%s", prefix, strname);
 
 		    edef = DBCellLookDef(newnameptr + 1);
 		    if (edef != NULL)
-			sprintf(newnameptr, "0%s_%s[[0]]", prefix, strname);
+			sprintf(newnameptr, "0%s%s[[0]]", prefix, strname);
 		    HashSetValue(he, (char *)newnameptr);
 		    calmaOutStringRecord(CALMA_SNAME, newnameptr + 1, outf);
 		}
@@ -599,8 +608,9 @@ calmaFullDump(def, fi, outf, filename)
     char *filename;
 {
     int version, rval, i;
-    char *libname = NULL, uniqlibname[3];
-    char *sptr;
+    char *libname = NULL, uniqlibname[4];
+    char *sptr, *viewopts;
+    bool isAbstract;
     HashTable calmaDefHash;
     HashEntry *he;
 
@@ -630,32 +640,48 @@ calmaFullDump(def, fi, outf, filename)
     // Record the GDS library so it will not be processed again.
     he = HashFind(&calmaLibHash, filename);
 
-    /* Generate a SHORT name for this cell (else it is easy to run into the
-     * GDS 32-character cellname limit).  Save it in the hash record.  The
-     * chance of generating the same prefix for a library that has items
-     * with conflicting names is vanishingly small, but to be pedantic, store
-     * the prefix in a hash table and check to make sure that uniqueness is
-     * ensured.
+    /* If property LEFview is defined as "no_prefix" instead of "TRUE",
+     * then do not create a unique prefix for subcells.  This is generally
+     * ill-advised, but can be needed if a foundry runs specific DRC checks
+     * on specific cell names, in which case adding a prefix can cause DRC
+     * errors to appear.  It is then incumbent on the user to ensure that
+     * names in the GDS file do not shadow any names in the database.
      */
-    while (TRUE)
-    {
-	HashEntry *he2;
 
-	for (i = 0; i < 2; i++) {
-	    rval = random() % 62;
-	    rval = (rval < 26) ? ('A' + rval) : ((rval < 52) ? ('a' + rval - 26) :
-			('0' + rval - 52));
-	    uniqlibname[i] = (char)(rval & 127);
-	}
-	uniqlibname[2] = '\0';
-	he2 = HashLookOnly(&calmaPrefixHash, uniqlibname);
-	if (he2 == NULL)
+    viewopts = (char *)DBPropGet(def, "LEFview", &isAbstract);
+    if ((!isAbstract) || (strcasecmp(viewopts, "no_prefix")))
+    {
+
+	/* Generate a SHORT name for this cell (else it is easy to run into the
+	 * GDS 32-character cellname limit).  Save it in the hash record.  The
+	 * chance of generating the same prefix for a library that has items
+	 * with conflicting names is vanishingly small, but to be pedantic, store
+	 * the prefix in a hash table and check to make sure that uniqueness is
+	 * ensured.
+	 */
+	while (TRUE)
 	{
-	    he2 = HashFind(&calmaPrefixHash, uniqlibname);
-	    break;
+	    HashEntry *he2;
+
+	    for (i = 0; i < 2; i++) {
+		rval = random() % 62;
+		rval = (rval < 26) ? ('A' + rval) : ((rval < 52) ? ('a' + rval - 26) :
+			('0' + rval - 52));
+		uniqlibname[i] = (char)(rval & 127);
+	    }
+	    uniqlibname[2] = '_';
+	    uniqlibname[3] = '\0';
+	    he2 = HashLookOnly(&calmaPrefixHash, uniqlibname);
+	    if (he2 == NULL)
+	    {
+		he2 = HashFind(&calmaPrefixHash, uniqlibname);
+		break;
+	    }
 	}
+	HashSetValue(he, StrDup(NULL, uniqlibname));
     }
-    HashSetValue(he, StrDup(NULL, uniqlibname));
+    else
+	HashSetValue(he, StrDup(NULL, ""));
 
     while (calmaDumpStructure(def, outf, &calmaDefHash, filename))
 	if (SigInterruptPending)
@@ -721,16 +747,17 @@ calmaProcessUse(use, outf)
     CellUse *use;	/* Process use->cu_def */
     FILE *outf;		/* Stream file */
 {
-    return (calmaProcessDef(use->cu_def, outf));
+    return (calmaProcessDef(use->cu_def, outf, FALSE));
 }
 
 int
-calmaProcessDef(def, outf)
+calmaProcessDef(def, outf, do_library)
     CellDef *def;	/* Output this def's children, then the def itself */
     FILE *outf;		/* Stream file */
+    bool do_library;	/* If TRUE, output only children of def, but not def */
 {
     char *filename;
-    bool isReadOnly, oldStyle, hasContent, isAbstract;
+    bool isReadOnly, oldStyle, hasContent, isAbstract, hasGDSEnd;
     HashEntry *he;
 
     /* Skip if already output */
@@ -746,15 +773,11 @@ calmaProcessDef(def, outf)
 
     /* Read the cell in if it is not already available. */
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL))
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL))
 	    return (0);
-
-    /*
-     * Output the definitions for any of our descendants that have
-     * not already been output.  Numbers are assigned to the subcells
-     * as they are output.
-     */
-    (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
+    }
 
     /*
      * Check if this is a read-only file that is supposed to be copied
@@ -762,7 +785,7 @@ calmaProcessDef(def, outf)
      * not, or if there is any problem obtaining the original cell
      * definition, resort to writing out magic's version of the def,
      * and print a warning message.
-     * 
+     *
      * Treat the lack of a GDS_START property as an indication
      * that we should treat this cell like a reference-only
      * cell.  That is, the instance will be called but no
@@ -771,8 +794,25 @@ calmaProcessDef(def, outf)
 
     DBPropGet(def, "LEFview", &isAbstract);
     DBPropGet(def, "GDS_START", &hasContent);
+    DBPropGet(def, "GDS_END", &hasGDSEnd);
     filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
 
+    /* When used with "calma addendum true", don't output the read-only	*/
+    /* cells.  This makes the library incomplete and dependent on the	*/
+    /* vendor libraries, so use with caution.				*/
+
+    if (isReadOnly && hasContent && CalmaAddendum) return (0);
+
+    /* Give some feedback to the user */
+    TxPrintf("   Writing cell %s\n", def->cd_name);
+
+    /*
+     * Output the definitions for any of our descendants that have
+     * not already been output.  Numbers are assigned to the subcells
+     * as they are output.
+     */
+    (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
+
     if (isReadOnly && hasContent)
     {
 	char *buffer, *offptr, *retfilename;
@@ -790,7 +830,8 @@ calmaProcessDef(def, outf)
 	    /* This is a rare error, but if the subcell is inside	*/
 	    /* another vendor GDS, it would not normally be output.	*/
 
-	    DBPropGet(def->cd_parents->cu_parent, "GDS_FILE", &isReadOnly);
+	    DBPropGet((def->cd_parents->cu_parent == NULL) ? def :
+			def->cd_parents->cu_parent, "GDS_FILE", &isReadOnly);
 	    if (!isReadOnly || isAbstract)
 		TxError("Calma output error:  Can't find GDS file \"%s\" "
 				"for vendor cell \"%s\".  Using magic's "
@@ -799,7 +840,7 @@ calmaProcessDef(def, outf)
 	    if (isReadOnly)
 		def->cd_flags |= CDVENDORGDS;
 	}
-	else if (isAbstract)
+	else if (isAbstract || (!hasGDSEnd))
 	{
 	    /* This is the trickiest part.  If the cell view is abstract then	*/
 	    /* the cell view has no hierarchy, and there is no way to descend	*/
@@ -831,7 +872,10 @@ calmaProcessDef(def, outf)
 
 		/* Output structure header */
 		calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
-		calmaOutDate(def->cd_timestamp, outf);
+    		if (CalmaNoDateStamp)
+		    calmaOutDate(time((time_t *) 0), outf);
+		else
+		    calmaOutDate(def->cd_timestamp, outf);
 		calmaOutDate(time((time_t *) 0), outf);
 
 		/* Name structure the same as the magic cellname */
@@ -883,12 +927,13 @@ calmaProcessDef(def, outf)
 
     /* Output this cell definition from the Magic database */
     if (!isReadOnly)
-	calmaOutFunc(def, outf, &TiPlaneRect);
+	if (!do_library)
+	    calmaOutFunc(def, outf, &TiPlaneRect);
 
     return (0);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -927,7 +972,10 @@ calmaOutFunc(def, f, cliprect)
 
     /* Output structure begin */
     calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, f);
-    calmaOutDate(def->cd_timestamp, f);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), f);
+    else
+	calmaOutDate(def->cd_timestamp, f);
     calmaOutDate(time((time_t *) 0), f);
 
     /* Output structure name */
@@ -974,7 +1022,8 @@ calmaOutFunc(def, f, cliprect)
     /* Output all the tiles associated with this cell; skip temporary layers */
     GEO_EXPAND(&def->cd_bbox, CIFCurStyle->cs_radius, &bigArea);
     CIFErrorDef = def;
-    CIFGen(def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, (ClientData) f);
+    CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, FALSE,
+		(ClientData)f);
     if (!CIFHierWriteDisable)
 	CIFGenSubcells(def, &bigArea, CIFPlanes);
     if (!CIFArrayWriteDisable)
@@ -1024,7 +1073,7 @@ calmaOutFunc(def, f, cliprect)
 				((lab->lab_flags & PORT_NUM_MASK) == i))
 		    {
 			calmaWriteLabelFunc(lab, type, f);
-			break;	
+			/* break; */  /* Do not limit to unique labels! */
 		    }
 		}
     }
@@ -1291,7 +1340,7 @@ calmaWriteUseFunc(use, f)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1316,13 +1365,13 @@ calmaOutStructName(type, def, f)
     CellDef *def;
     FILE *f;
 {
-    char defname[CALMANAMELENGTH+1];
+    char *defname;
     unsigned char c;
     char *cp;
     int calmanum;
     char *table;
 
-    if (CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
+    if (CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS)
     {
 	table = calmaMapTablePermissive;
     } else {
@@ -1341,10 +1390,11 @@ calmaOutStructName(type, def, f)
 	}
 	/* We really should ensure that the new name is unique. . . */
     }
-    if (cp <= def->cd_name + CALMANAMELENGTH)
+    if ((!(CIFCurStyle->cs_flags & CWF_STRING_LIMIT)) ||
+	    (cp <= def->cd_name + CALMANAMELENGTH))
     {
 	/* Yes, it's legal: use it */
-	(void) strcpy(defname, def->cd_name);
+	defname = StrDup(NULL, def->cd_name);
     }
     else
     {
@@ -1352,12 +1402,14 @@ calmaOutStructName(type, def, f)
 bad:
 	calmanum = (int) def->cd_client;
 	if (calmanum < 0) calmanum = -calmanum;
+	defname = (char *)mallocMagic(32);
 	(void) sprintf(defname, "XXXXX%d", calmanum);
 	TxError("Warning: string in output unprintable; changed to \'%s\'\n",
 		 defname);
     }
 
     calmaOutStringRecord(type, defname, f);
+    freeMagic(defname);
 }
 
 /* Added by NP 8/21/2004 */
@@ -1410,7 +1462,7 @@ calmaGetContactCell(type, lookOnly)
     def = DBCellLookDef(contactCellName);
     if ((def == (CellDef *) NULL) && (lookOnly == FALSE))
     {
-	def = DBCellNewDef(contactCellName, (char *) NULL);
+	def = DBCellNewDef(contactCellName);
        	def->cd_flags &= ~(CDMODIFIED|CDGETNEWSTAMP);
         def->cd_flags |= CDAVAILABLE;
     }
@@ -1518,7 +1570,7 @@ calmaWriteContacts(f)
     /* the proper contact cut drawn.  It is turned on again at the end	*/
     /* of the routine.  Note that this routine is not called unless	*/
     /* CalmaContactArrays is TRUE.					*/
-    
+
     CalmaContactArrays = FALSE;
 
     DBEnumerateTypes(&tMask);
@@ -1622,7 +1674,7 @@ calmaDelContacts()
  *	this should be most efficient.
  *
  * Side effects:
- *	May allocate memory. 
+ *	May allocate memory.
  * ---------------------------------------------------------------------------
  */
 
@@ -1729,7 +1781,7 @@ calmaRemoveDegenerate(blist)
 
 		    freeMagic(curseg->lb_next);
 		    freeMagic(curseg);
-	    
+
 		    /* Make sure record doesn't point to a free'd segment */
 		    bounds->bt_first = lastseg;
 		    bounds->bt_points -= 2;
@@ -1779,7 +1831,7 @@ calmaRemoveColinear(blist)
 		(lastseg->lb_start.p_y == curseg->lb_next->lb_start.p_y)))
 	    {
 		lastseg->lb_next = curseg->lb_next;
-	    
+
 		/* Make sure record doesn't point to a free'd segment */
 		if (curseg == bounds->bt_first) bounds->bt_first = lastseg;
 
@@ -1845,7 +1897,7 @@ calmaRemoveColinear(blist)
  *    Still, each segment is never checked against more than 200 points,
  *    because when a boundary reaches this number (the maximum for GDS
  *    boundary records), the record will tend to be skipped (it should
- *    probably be output here. . .) 
+ *    probably be output here. . .)
  *
  * Results:
  *	None.
@@ -1858,7 +1910,7 @@ calmaRemoveColinear(blist)
 
 void
 calmaMergeSegments(edge, blist, num_points)
-    LinkedBoundary *edge;    
+    LinkedBoundary *edge;
     BoundaryTop **blist;
     int num_points;
 {
@@ -1906,7 +1958,7 @@ calmaMergeSegments(edge, blist, num_points)
 
 		    freeMagic(compseg);
 		    freeMagic(curseg);
-	
+
 		    /* Make sure the record doesn't point to the free'd segment */
 		    if (compseg == bounds->bt_first) bounds->bt_first = complast;
 		    bounds->bt_points += num_points - 2;
@@ -1917,7 +1969,7 @@ calmaMergeSegments(edge, blist, num_points)
     }
 
     /* If still no connecting edge was found, or if we overflowed the GDS max	*/
-    /* number of records for a boundary, then start a new entry.		*/ 
+    /* number of records for a boundary, then start a new entry.		*/
 
 make_new_bound:
 
@@ -2002,7 +2054,7 @@ calmaProcessBoundary(blist, cos)
 	{
 	    if (lbref != listtop)
 		TxPrintf("->");
-	    else 
+	    else
 		lbstop = listtop;
 
 	    switch(lbref->lb_type)
@@ -2316,7 +2368,7 @@ done_searches:
 	    /* tiles, we may have expanded it.  This could use a LOT of	*/
 	    /* optimizing.  1) remove colinear points in calmaAddSegment */
 	    /* when both subsegments are external paths, and 2) here,	*/
-	    /* take the shortest path to making "edge" exactly 4 points.*/	
+	    /* take the shortest path to making "edge" exactly 4 points.*/
 	    /* Note that in non-Manhattan mode, num_points may be 3.	*/
 
 	    if (num_points != 4)
@@ -2457,7 +2509,7 @@ calmaWritePaintFunc(tile, cos)
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2621,7 +2673,7 @@ calmaWriteLabelFunc(lab, type, f)
 	calmaOutRH(4, CALMA_ENDEL, CALMA_NODATA, f);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2652,7 +2704,10 @@ calmaOutHeader(rootDef, f)
 
     /* Beginning of library */
     calmaOutRH(28, CALMA_BGNLIB, CALMA_I2, f);
-    calmaOutDate(rootDef->cd_timestamp, f);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), f);
+    else
+    	calmaOutDate(rootDef->cd_timestamp, f);
     calmaOutDate(time((time_t *) 0), f);
 
     /* Library name (name of root cell) */
@@ -2677,7 +2732,7 @@ calmaOutHeader(rootDef, f)
     if (CIFCurStyle->cs_flags & CWF_ANGSTROMS) mum = 1e-10;
     calmaOutR8(mum, f);		/* Meters per database unit */
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2710,7 +2765,7 @@ calmaOutDate(t, f)
     calmaOutI2(datep->tm_min, f);
     calmaOutI2(datep->tm_sec, f);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2734,7 +2789,7 @@ calmaOutDate(t, f)
 void
 calmaOutStringRecord(type, str, f)
     int type;		/* Type of this record (data type is ASCII string) */
-    char *str;	/* String to be output (<= CALMANAMELENGTH chars) */
+    char *str;	/* String to be output */
     FILE *f;	/* Stream file */
 {
     int len;
@@ -2742,7 +2797,7 @@ calmaOutStringRecord(type, str, f)
     char *table, *locstr, *origstr = NULL;
     char *locstrprv; 	/* Added by BSI */
 
-    if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
+    if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS)
     {
 	table = calmaMapTablePermissive;
     } else {
@@ -2759,9 +2814,16 @@ calmaOutStringRecord(type, str, f)
      * last CALMANAMELENGTH characters (since cell names are more
      * likely to be unique in the last characters than in the first
      * characters).
+     *
+     * NOTE:  GDS format has not used CALMANAMELENGTH restrictions
+     * for ages.  Since this is a 2-byte record, then is it not
+     * worth checking the 65536 - 4 character limit.  The CALMANAMELENGTH
+     * restriction must be enabled in the cifoutput flags.
      */
+
+
     if (len & 01) len++;
-    if (len > CALMANAMELENGTH)
+    if ((CIFCurStyle->cs_flags & CWF_STRING_LIMIT) && (len > CALMANAMELENGTH))
     {
 	TxError("Warning:  Cellname %s truncated ", str);
 	TxError("to %s (GDS format limit)\n", str + len - CALMANAMELENGTH);
@@ -2883,7 +2945,7 @@ calmaOutR8(d, f)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
