@@ -8,16 +8,16 @@
  *	separately.  This module computes the extra CIF that may be
  *	needed.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -27,6 +27,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 #include <stdio.h>
 
+#include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -35,6 +36,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "cif/CIFint.h"
 #include "cif/cif.h"
 #include "drc/drc.h"
+#include "graphics/graphics.h"
 #include "textio/textio.h"
 #include "utils/undo.h"
 #include "utils/malloc.h"
@@ -82,7 +84,7 @@ static CIFLayer *CurCifLayer;
     (dst)->r_ybot = (src)->r_ybot * scale; \
     (dst)->r_xtop = (src)->r_xtop * scale; \
     (dst)->r_ytop = (src)->r_ytop * scale;
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -112,7 +114,7 @@ CIFInitCells()
     CIFTotalDef = DBCellLookDef("__CIF__");
     if (CIFTotalDef == (CellDef *) NULL)
     {
-	CIFTotalDef = DBCellNewDef ("__CIF__",(char *) NULL);
+	CIFTotalDef = DBCellNewDef("__CIF__");
 	ASSERT(CIFTotalDef != (CellDef *) NULL, "cifMakeCell");
 	DBCellSetAvail(CIFTotalDef);
 	CIFTotalDef->cd_flags |= CDINTERNAL;
@@ -124,7 +126,7 @@ CIFInitCells()
     CIFComponentDef = DBCellLookDef("__CIF2__");
     if (CIFComponentDef == (CellDef *) NULL)
     {
-	CIFComponentDef = DBCellNewDef ("__CIF2__",(char *) NULL);
+	CIFComponentDef = DBCellNewDef("__CIF2__");
 	ASSERT(CIFComponentDef != (CellDef *) NULL, "cifMakeCell");
 	DBCellSetAvail(CIFComponentDef);
 	CIFComponentDef->cd_flags |= CDINTERNAL;
@@ -148,7 +150,7 @@ CIFInitCells()
     CIFDummyUse = DBCellNewUse(CIFTotalDef, (char *) NULL);
     DBSetTrans (CIFDummyUse, &GeoIdentityTransform);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -198,7 +200,7 @@ cifHierCleanup()
 
     SigEnableInterrupts();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -243,9 +245,8 @@ cifHierCopyFunc(tile, cxp)
     if (cxp->tc_scx->scx_use->cu_def->cd_flags & CDVENDORGDS)
     {
 	if (!CIFCurStyle)
-	    return 0;
-	else if (!(CIFCurStyle->cs_flags & CWF_SEE_VENDOR))
-	    return 0;
+	    if (!(CIFCurStyle->cs_flags & CWF_SEE_NO_VENDOR))
+		return 0;
     }
 
     /* Ignore space tiles, since they won't do anything anyway. */
@@ -274,7 +275,7 @@ cifHierCopyFunc(tile, cxp)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -316,8 +317,8 @@ cifHierCellFunc(scx)
     newscx = *scx;
     GEO_EXPAND(&scx->scx_area, CIFCurStyle->cs_radius, &newscx.scx_area);
     (void) DBTreeSrTiles(&newscx, &CIFCurStyle->cs_yankLayers, 0,
-	cifHierCopyFunc, (ClientData) CIFComponentDef);
-    
+	    cifHierCopyFunc, (ClientData) CIFComponentDef);
+
     /* Set CIFErrorDef to NULL to ignore errors here... these will
      * get reported anyway when the cell is CIF'ed non-hierarchically,
      * so there's no point in making a second report here.
@@ -325,11 +326,12 @@ cifHierCellFunc(scx)
 
     CIFErrorDef = (CellDef *) NULL;
     GeoTransRect(&scx->scx_trans, &scx->scx_area, &rootArea);
-    CIFGen(CIFComponentDef, &rootArea, CIFComponentPlanes,
-	    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
+    CIFGen(CIFComponentDef, scx->scx_use->cu_def, &rootArea,
+	    CIFComponentPlanes, &CIFCurStyle->cs_hierLayers, FALSE,
+	    TRUE, TRUE, (ClientData)NULL);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -370,7 +372,7 @@ cifHierErrorFunc(tile, checkArea)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -418,6 +420,44 @@ cifHierCheckFunc(tile, plane)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * cifHierTempCheckFunc --
+ *
+ * 	This function is like cifHierCheckFunc() (see above), but is used
+ *	for "templayers", where any parent/child disagreement should be
+ *	considered a non-issue as far as output is concerned.  Only the
+ *	actual mask layer will report any problems.
+ *
+ * Results:
+ *	Always returns 0 to keep the search alive.
+ *
+ * Side effects:
+ *	Error messages may be output.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+cifHierTempCheckFunc(tile, plane)
+    Tile *tile;			/* Tile containing CIF. */
+    Plane *plane;		/* Plane to check against and modify. */
+{
+    Rect area;
+
+    TiToRect(tile, &area);
+
+    if (IsSplit(tile))
+	DBNMPaintPlane(plane, TiGetTypeExact(tile), &area, CIFEraseTable,
+		(PaintUndoInfo *) NULL);
+    else
+        DBPaintPlane(plane, &area, CIFEraseTable, (PaintUndoInfo *) NULL);
+
+    CIFTileOps++;
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -453,7 +493,7 @@ cifHierPaintFunc(tile, plane)
     CIFTileOps += 1;
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -482,17 +522,22 @@ cifCheckAndErase(style)
     int i;
 
     /* Process all of the bits of CIF in CIFComponentPlanes. */
-    
+
     for (i=0; i<style->cs_nLayers; i++)
     {
 	CIFErrorLayer = i;
 	if (CIFComponentPlanes[i] == NULL) continue;
-	(void) DBSrPaintArea((Tile *) NULL, CIFComponentPlanes[i],
-	    &TiPlaneRect, &CIFSolidBits, cifHierCheckFunc,
-	    (ClientData) CIFTotalPlanes[i]);
+	if (CIFCurStyle->cs_layers[i]->cl_flags & CIF_TEMP)
+	    (void) DBSrPaintArea((Tile *) NULL, CIFComponentPlanes[i],
+	    	&TiPlaneRect, &CIFSolidBits, cifHierTempCheckFunc,
+	    	(ClientData) CIFTotalPlanes[i]);
+	else
+	    (void) DBSrPaintArea((Tile *) NULL, CIFComponentPlanes[i],
+	    	&TiPlaneRect, &CIFSolidBits, cifHierCheckFunc,
+	    	(ClientData) CIFTotalPlanes[i]);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -523,6 +568,8 @@ CIFGenSubcells(def, area, output)
     int stepSize, x, y, i, radius, oldTileOps, oldTileOps2;
     Rect totalArea, square, interaction;
     SearchContext scx;
+    int cuts, totcuts;
+    float pdone, plast;
 
     UndoDisable();
     CIFInitCells();
@@ -537,6 +584,16 @@ CIFGenSubcells(def, area, output)
     scx.scx_use = CIFDummyUse;
     scx.scx_trans = GeoIdentityTransform;
 
+    /* This routine can take a long time, so use the display
+     * timer to force a 5-second progress check (like is done
+     * with extract)
+     */
+    GrDisplayStatus = DISPLAY_IN_PROGRESS;
+    SigSetTimer(5);			/* Print at 5-second intervals */
+    cuts = 0;
+    pdone = 0.0;
+    plast = 0.0;
+
     /* Any tile operations processed here get billed to hierarchy
      * in addition to being added to the total.
      */
@@ -548,9 +605,15 @@ CIFGenSubcells(def, area, output)
      * out if there are any subcell interactions within one
      * radius of the square.
      */
-    
+
     totalArea = *area;
     GeoClip(&totalArea, &def->cd_bbox);
+
+    totcuts = (totalArea.r_ytop - totalArea.r_ybot + stepSize - 1)
+			/ stepSize;
+    totcuts *= ((totalArea.r_xtop - totalArea.r_xbot + stepSize - 1)
+			/ stepSize);
+
     for (y = totalArea.r_ybot; y < totalArea.r_ytop; y += stepSize)
 	for (x = totalArea.r_xbot; x < totalArea.r_xtop; x += stepSize)
 	{
@@ -565,48 +628,50 @@ CIFGenSubcells(def, area, output)
 	    GEO_EXPAND(&square, radius, &square);
 	    if (!DRCFindInteractions(def, &square, radius,
 		&interaction)) continue;
-	    
+
 	    /* We've found an interaction.  Flatten it into CIFTotalUse, then
 	     * make CIF from what's flattened.  Yank extra material to
 	     * ensure that CIF is generated correctly.
 	     */
-	    
+
 	    GEO_EXPAND(&interaction, CIFCurStyle->cs_radius, &scx.scx_area);
 	    (void) DBTreeSrTiles(&scx, &CIFCurStyle->cs_yankLayers, 0,
 		cifHierCopyFunc, (ClientData) CIFTotalDef);
 	    CIFErrorDef = def;
-	    CIFGen(CIFTotalDef, &interaction, CIFTotalPlanes,
-		    &CIFCurStyle->cs_hierLayers, TRUE, TRUE);
+	    CIFGen(CIFTotalDef, def, &interaction, CIFTotalPlanes,
+		    &CIFCurStyle->cs_hierLayers, TRUE, TRUE, TRUE,
+		    (ClientData)NULL);
 
 	    /* Now go through all the subcells overlapping the area
 	     * and generate CIF for each subcell individually.  OR this
 	     * combined CIF together into CIFComponentPlanes.
 	     */
-	    
+
 	    scx.scx_area = interaction;
 	    (void) DBCellSrArea(&scx, cifHierCellFunc, (ClientData) NULL);
-	    
+
 	    /* Also generate CIF for the paint in the parent alone.  Ignore
 	     * CIF generation errors here, since they will already have been
 	     * recorded when the parent was CIF'ed by itself.
 	     */
 
 	    CIFErrorDef = (CellDef *) NULL;
-	    CIFGen(def, &interaction, CIFComponentPlanes,
-		    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
+	    CIFGen(def, def, &interaction, CIFComponentPlanes,
+		    &CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+		    (ClientData)NULL);
 
 	    /* Make sure everything in the pieces is also in the overall
 	     * CIF, then erase the piece stuff from the overall, and
 	     * throw away the pieces.
 	     */
-	    
+
 	    CIFErrorDef = def;
 	    cifCheckAndErase(CIFCurStyle);
 
 	    /* Lastly, paint everything from the pieces into the result
 	     * planes.
 	     */
-	    
+
 	    oldTileOps2 = CIFTileOps;
 	    for (i=0; i<CIFCurStyle->cs_nLayers; i++)
 	    {
@@ -618,12 +683,36 @@ CIFGenSubcells(def, area, output)
 	    CIFHierRects += CIFTileOps - oldTileOps2;
 
 	    cifHierCleanup();
+
+	    cuts++;
+	    pdone = 100.0 * ((float)cuts / (float)totcuts);
+	    if ((((pdone - plast) > 1.0) || (cuts == totcuts)) && (cuts > 1))
+	    {
+		/* Only print something if the 5-second timer has expired */
+		if (GrDisplayStatus == DISPLAY_BREAK_PENDING)
+		{
+		    TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
+		    plast = pdone;
+		    TxFlushOut();
+
+		    GrDisplayStatus = DISPLAY_IN_PROGRESS;
+		    SigSetTimer(5);
+		}
+#ifdef MAGIC_WRAPPER
+		/* We need to let Tk paint the console display */
+		while (Tcl_DoOneEvent(TCL_DONT_WAIT) != 0);
+#endif
+	    }
 	}
-    
+
     CIFHierTileOps += CIFTileOps - oldTileOps;
+
+    GrDisplayStatus = DISPLAY_IDLE;
+    SigRemoveTimer();
+
     UndoEnable();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -679,19 +768,20 @@ cifHierElementFunc(use, transform, x, y, checkArea)
 	cifHierCopyFunc, (ClientData) CIFComponentDef);
 
     CIFErrorDef = (CellDef *) NULL;
-    CIFGen(CIFComponentDef, checkArea, CIFComponentPlanes,
-	    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
+    CIFGen(CIFComponentDef, use->cu_def, checkArea, CIFComponentPlanes,
+	    &CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+	    (ClientData)NULL);
 
     return 0;
 }
-
+
 
 /*
  * ---------------------------------------------------------------------------
  * cifGrowSliver() --
  *
  * This function is passed the address of a "sliver" rectangle.  It then
- * grows the shortest dimension of the sliver so that it is at least minimum 
+ * grows the shortest dimension of the sliver so that it is at least minimum
  * width.  The minimum width is found through the variable CurCifLayer which
  * identifies the current layer being generated.  A "minwidth" command has been
  * added to the cifoutput section for which the minimum width of each layer
@@ -744,7 +834,7 @@ cifGrowSliver(tile, area)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -794,7 +884,7 @@ cifHierPaintArrayFunc(tile)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -852,7 +942,7 @@ cifHierArrayFunc(scx, output)
     radius = CIFCurStyle->cs_radius;
     if ((use->cu_xlo == use->cu_xhi) && (use->cu_ylo == use->cu_yhi))
 	return 2;
-    
+
     /* We only want interactions between neighboring cells, so reduce	*/
     /* the array size to at most 2x2, process, then restore the		*/
     /* original array count.						*/
@@ -869,7 +959,7 @@ cifHierArrayFunc(scx, output)
      * of the parent.  If the array is 1-dimensional, we set the
      * corresponding spacing to an impossibly large distance.
      */
-    
+
     childArea.r_xbot = 0;
     childArea.r_ybot = 0;
 
@@ -920,8 +1010,9 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &A);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &A, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &A, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+		(ClientData)NULL);
 	anyInteractions = TRUE;
     }
 
@@ -937,8 +1028,9 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &C);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &C, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &C, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+		(ClientData)NULL);
 	anyInteractions = TRUE;
     }
 
@@ -954,9 +1046,10 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &B);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &B, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
-	
+	CIFGen(CIFTotalDef, use->cu_def, &B, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+		(ClientData)NULL);
+
 	/* D */
 
 	D.r_xbot = use->cu_bbox.r_xtop - xsep + radius;
@@ -967,8 +1060,9 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &D);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &D, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &D, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE, TRUE,
+		(ClientData)NULL);
     }
 
     if (anyInteractions)
@@ -977,7 +1071,7 @@ cifHierArrayFunc(scx, output)
 	/* Remove redundant CIF that's already in the children, and
 	 * make sure everything in the kids is in the parent too.
 	 */
-    
+
 	CIFErrorDef = use->cu_parent;
 	cifCheckAndErase(CIFCurStyle);
 
@@ -986,7 +1080,7 @@ cifHierArrayFunc(scx, output)
 	 * be replicated many times over to cover each of the array
 	 * interaction areas.
 	 */
-    
+
 	oldTileOps = CIFTileOps;
 	for (i=0; i<CIFCurStyle->cs_nLayers; i++)
 	{
@@ -1037,7 +1131,7 @@ cifHierArrayFunc(scx, output)
 		(void) DBSrPaintArea((Tile *) NULL, CIFTotalPlanes[i],
 			&cifArea, &CIFSolidBits, cifHierPaintArrayFunc,
 			(ClientData) NULL);
-	    
+
 		/* The right edge of the array (from D). */
 
 		cifHierXSpacing = 0;
@@ -1048,7 +1142,7 @@ cifHierArrayFunc(scx, output)
 		(void) DBSrPaintArea((Tile *) NULL, CIFTotalPlanes[i],
 			&cifArea, &CIFSolidBits, cifHierPaintArrayFunc,
 			(ClientData) NULL);
-	    
+
 		/* The core of the array (from A and B).  This code is a bit
 		 * tricky in order to work correctly even for arrays where
 		 * radius < ysep.  The "if" statement handles this case.
@@ -1081,7 +1175,7 @@ cifHierArrayFunc(scx, output)
     return 2;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1122,7 +1216,7 @@ CIFGenArrays(def, area, output)
     /* Bill all tile operations here to hierarchical processing in
      * addition to adding to the total.
      */
-    
+
     oldTileOps = CIFTileOps;
 
     if (output == NULL)
