@@ -4,16 +4,16 @@
  * Defines things shared internally by the cif module of Magic,
  * but not generally needed outside the cif module.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFint.h,v 1.3 2008/12/04 17:10:29 tim Exp $"
@@ -46,11 +46,17 @@
 typedef struct bloat_data
 {
     int bl_plane;		/* Plane on which a bloat or squares
-				 * operation is valid.
+				 * operation is valid.  If -1, then the bloat
+				 * types are CIF types.
 				 */
     int bl_distance[TT_MAXTYPES];
 } BloatData;
 
+typedef struct bridge_data
+{
+    int br_width;		/* Minimum width rule for bridge */
+} BridgeData;
+
 typedef struct squares_data
 {
     int sq_border;
@@ -69,6 +75,7 @@ typedef struct slots_data
     int sl_lsize;
     int sl_lsep;
     int sl_offset;
+    int sl_start;
 } SlotsData;
 
 typedef struct cifop
@@ -79,8 +86,9 @@ typedef struct cifop
 				 * below for the legal ones.
 				 */
     int co_distance;		/* Grow or shrink distance (if needed). */
-    ClientData co_client;	/* Pointer to a BloatData, SquaresData, or
-				 * SlotsData structure, or NULL.
+    ClientData co_client;	/* Pointer to a BloatData, SquaresData,
+				 * SlotsData, or BridgeData structure,
+				 * or NULL.
 				 */
     struct cifop *co_next;	/* Next in list of operations to perform. */
 } CIFOp;
@@ -95,6 +103,7 @@ typedef struct cifop
  *			the masks.
  * CIFOP_GROW -		Grow the current results uniformly by co_distance.
  * CIFOP_GROW_G -	Grow the current results to snap to the indicated grid.
+ * CIFOP_GROWMIN -	Grow result such that no dimension is less than co_distance.
  * CIFOP_SHRINK -	Shrink the current results uniformly by co_distance.
  * CIFOP_BLOAT -	Find layers in paintMask, then bloat selectively
  *			according to bl_distance, and OR the results into
@@ -130,26 +139,36 @@ typedef struct cifop
  * CIFOP_COPYUP -	Added 5/5/16---make and keep a copy the resulting layer,
  *			which will be painted into parent cells instead of the
  *			current cell.  This replaces the "fault" method.
+ * CIFOP_CLOSE -	Added 11/25/19---close up areas smaller than indicated
+ * CIFOP_BRIDGE -	Added 6/11/20---Bridge across catecorner gaps
+ * CIFOP_BRIDGELIM -	Added 27/07/20---Bridge across catecorner gaps, but with limiting layers
+ * CIFOP_MASKHINTS -	Added 12/14/20---Add geometry from cell properties, if any.
  */
 
 #define CIFOP_AND	1
 #define CIFOP_OR	2
 #define CIFOP_GROW	3
-#define CIFOP_GROW_G	4
-#define CIFOP_SHRINK	5
-#define CIFOP_BLOAT	6
-#define CIFOP_SQUARES	7
-#define CIFOP_SLOTS	8
-#define CIFOP_BLOATMAX	9
-#define CIFOP_BLOATMIN	10
-#define CIFOP_BLOATALL	11
-#define CIFOP_ANDNOT	12
-#define CIFOP_SQUARES_G	13
-#define CIFOP_BBOX	14
-#define CIFOP_BOUNDARY	15
-#define CIFOP_NET	16
-#define CIFOP_MAXRECT	17
-#define CIFOP_COPYUP	18
+#define CIFOP_GROWMIN	4
+#define CIFOP_GROW_G	5
+#define CIFOP_SHRINK	6
+#define CIFOP_BLOAT	7
+#define CIFOP_SQUARES	8
+#define CIFOP_SLOTS	9
+#define CIFOP_BLOATMAX	10
+#define CIFOP_BLOATMIN	11
+#define CIFOP_BLOATALL	12
+#define CIFOP_ANDNOT	13
+#define CIFOP_SQUARES_G	14
+#define CIFOP_BBOX	15
+#define CIFOP_BOUNDARY	16
+#define CIFOP_NET	17
+#define CIFOP_MAXRECT	18
+#define CIFOP_COPYUP	19
+#define CIFOP_CLOSE	20
+#define CIFOP_BRIDGE	21
+#define CIFOP_BRIDGELIM 22
+#define CIFOP_MASKHINTS 23
+
 
 /* Added by Tim 10/21/2004 */
 /* The following structure is used to pass information on how to draw
@@ -193,7 +212,7 @@ typedef struct
 				 */
     int min_width;		/* the minimum width rule in centi-microns
 				 * for the layer. This is used by Grow Sliver
-				 * to generate drc correct parent slivers	
+				 * to generate drc correct parent slivers
 				 */
 #ifdef THREE_D
     int	   cl_renderStyle;	/* Style to render CIF layer with */
@@ -207,12 +226,9 @@ typedef struct
  *
  * CIF_TEMP:	Means that this is a temporary layer used to build
  *		up CIF information.  It isn't output in the CIF file.
- * CIF_BBOX_TOP:  Indicates that the bounding box rectangle should
- *		only be generated if the cell is a top-level cell.
  */
 
 #define CIF_TEMP 		1
-#define CIF_BBOX_TOP		2
 
 /* The following data structure describes a complete set of CIF
  * layers.  The number of CIF layers (MAXCIFLAYERS) must not be
@@ -253,7 +269,7 @@ typedef struct cifstyle
 				 * file more readable).  Default of 1.
 				 * Unused for GDS input/output.
 				 */
-    int cs_expander;		/* cs_scaleFactor / cs_expander = scale in 
+    int cs_expander;		/* cs_scaleFactor / cs_expander = scale in
 				 * centimicrons.  Default of 1.  Value 10
 				 * means cs_scaleFactor is measured in
 				 * nanometers (millimicrons)
@@ -290,8 +306,9 @@ typedef struct cifstyle
 #define CWF_GROW_SLIVERS	0x02
 #define CWF_ANGSTROMS 		0x04
 #define CWF_GROW_EUCLIDEAN	0x08
-#define CWF_SEE_VENDOR		0x10	/* Override vendor GDS flag in cells */
-#define CWF_NO_ERRORS		0x20	/* Do not generate error msgs and fdbk */
+#define CWF_SEE_NO_VENDOR	0x10	/* Hide magic's GDS from vendor cells	*/
+#define CWF_NO_ERRORS		0x20	/* Do not generate error msgs and fdbk	*/
+#define CWF_STRING_LIMIT	0x40	/* Use older Calma format character limit */
 
 /* procedures */
 
