@@ -5,16 +5,16 @@
  *	reading CIF files, plus a bunch of utility routines
  *	for skipping white space, parsing numbers and points, etc.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -60,6 +60,7 @@ FILE *cifErrorFile;
 int cifLineNumber;		/* Number of current line. */
 int cifTotalWarnings;		/* Number of warnings detected */
 int cifTotalErrors;		/* Number of errors detected */
+bool cifSeenSnapWarning;	/* Track this to prevent excessive messaging */
 
 /* The variables used below hold general information about what
  * we're currently working on.
@@ -92,7 +93,7 @@ Plane *cifReadPlane;			/* Plane into which to paint material
 					 * NULL means no layer command has
 					 * been seen for the current cell.
 					 */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -120,7 +121,10 @@ CIFReadError(char *format, ...)
     if (CIFWarningLevel == CIF_WARN_NONE) return;
     if ((cifTotalErrors < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
     {
-	TxError("Error at line %d of CIF file: ", cifLineNumber);
+	if (cifLineNumber > 0)
+	    TxError("Error at line %d of CIF file: ", cifLineNumber);
+	else
+	    TxError("CIF file read error: ", cifLineNumber);
 	va_start(args, format);
 	Vfprintf(stderr, format, args);
 	va_end(args);
@@ -130,7 +134,7 @@ CIFReadError(char *format, ...)
 	TxError("Error limit set:  Remaining errors will not be reported.\n");
     }
 }
-
+
 
 void
 CIFReadWarning(char *format, ...)
@@ -141,7 +145,10 @@ CIFReadWarning(char *format, ...)
     if (CIFWarningLevel == CIF_WARN_NONE) return;
     if ((cifTotalWarnings < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
     {
-	TxError("Warning at line %d of CIF file: ", cifLineNumber);
+	if (cifLineNumber > 0)
+	    TxError("Warning at line %d of CIF file: ", cifLineNumber);
+	else
+	    TxError("CIF file read warning: ");
 	va_start(args, format);
 	Vfprintf(stderr, format, args);
 	va_end(args);
@@ -212,8 +219,10 @@ CIFScaleCoord(cifCoord, snap_type)
 	switch (snap_type)
 	{
 	    case COORD_EXACT:
-		CIFReadWarning("Input off lambda grid by %d/%d; grid redefined.\n",
+		if (!cifSeenSnapWarning)
+		    CIFReadWarning("Input off lambda grid by %d/%d; grid redefined.\n",
 			remain, denom);
+		cifSeenSnapWarning = TRUE;
 
 		CIFTechInputScale(1, denom, FALSE);
 		CIFTechOutputScale(1, denom);
@@ -237,8 +246,10 @@ CIFScaleCoord(cifCoord, snap_type)
 	    case COORD_HALF_U: case COORD_HALF_L:
 		if (denom > 2)
 		{
-		    CIFReadWarning("Input off lambda grid by %d/%d; grid redefined.\n",
-				remain, denom);
+		    if (!cifSeenSnapWarning)
+			CIFReadWarning("Input off lambda grid by %d/%d; "
+				"grid redefined.\n", remain, denom);
+		    cifSeenSnapWarning = TRUE;
 
 		    /* scale to nearest half-lambda */
 		    if (!(denom & 0x1)) denom >>= 1;
@@ -271,8 +282,10 @@ CIFScaleCoord(cifCoord, snap_type)
 
 		break;
 	    case COORD_ANY:
-		CIFReadWarning("Input off lambda grid by %d/%d; snapped to grid.\n",
-			abs(remain), abs(denom));
+		if (!cifSeenSnapWarning)
+		    CIFReadWarning("Input off lambda grid by %d/%d; snapped to grid.\n",
+				abs(remain), abs(denom));
+		cifSeenSnapWarning = TRUE;
 
 		/* Careful:  must round down a bit more for negative numbers, in
 		 * order to ensure that a point exactly halfway between Magic units
@@ -294,7 +307,7 @@ CIFScaleCoord(cifCoord, snap_type)
 
     return result;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -327,7 +340,7 @@ cifIsBlank(ch)
     }
     else return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -349,7 +362,7 @@ cifIsBlank(ch)
 void
 CIFSkipBlanks()
 {
-    
+
     while (cifIsBlank(PEEK())) {
 	if (TAKE() == '\n')
 	{
@@ -357,7 +370,7 @@ CIFSkipBlanks()
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -387,7 +400,7 @@ CIFSkipSep()
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -409,7 +422,7 @@ void
 CIFSkipToSemi()
 {
     int	ch;
-    
+
     for (ch = PEEK() ; ((ch != ';') && (ch != EOF)) ; ch = PEEK()) {
 	if (TAKE() == '\n')
 	{
@@ -417,7 +430,7 @@ CIFSkipToSemi()
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -437,7 +450,7 @@ CIFSkipToSemi()
 void
 CIFSkipSemi()
 {
-    
+
     CIFSkipBlanks();
     if (PEEK() != ';') {
 	CIFReadError("`;\' expected.\n");
@@ -446,7 +459,7 @@ CIFSkipSemi()
     TAKE();
     CIFSkipBlanks();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -492,7 +505,7 @@ CIFParseSInteger(valuep)
 	*valuep = -(*valuep);
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -610,7 +623,7 @@ CIFParsePoint(pointp, iscale)
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -707,7 +720,7 @@ CIFParsePath(pathheadpp, iscale)
  *	first three points in the given CIF path.
  *
  * Results:
- *	TRUE if point is inside, FALSE if outside or on the border 	
+ *	TRUE if point is inside, FALSE if outside or on the border
  *
  * Side effects:
  *	None.
@@ -748,7 +761,7 @@ test_insideness(start, tpoint)
  * Side effects:
  *	value of respt contains point to which segment will be
  *	truncated.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -850,7 +863,7 @@ path_intersect(pathHead, start, respt)
 	/* with the smaller absolute distance takes precedence.)	*/
 
 	if (test_insideness(start, &path->cifp_point)) {
-	    int tmpdist = abs(newdist);		/* save this value */	
+	    int tmpdist = abs(newdist);		/* save this value */
 	    if (path->cifp_x == path->cifp_next->cifp_x ||
 			path->cifp_y == path->cifp_next->cifp_y)
 	    {
@@ -1066,7 +1079,7 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 	edir = CIFEdgeDirection(first, last);
 	if (edir == CIF_DIAG_DL || edir == CIF_DIAG_UR)
 	{
-	    new->cifp_x = first->cifp_x; 
+	    new->cifp_x = first->cifp_x;
 	    new->cifp_y = last->cifp_y;
 	}
 	else  /* edir == CIF_DIAG_DR || edir == CIF_DIAG_UL */
@@ -1129,7 +1142,7 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1178,7 +1191,7 @@ CIFEdgeDirection(first, last)
 	return CIF_DOWN;
     return CIF_ZERO;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1219,7 +1232,7 @@ CIFCleanPath(pathHead)
 	path = next;
 	if (!path) return;
     }
-	 
+
     while (next = path->cifp_next)
     {
 	if ((dir2 = CIFEdgeDirection(path, next)) == CIF_ZERO)
@@ -1253,7 +1266,7 @@ path_inc:
     if (!pathHead->cifp_next)
     {
 	/* Ensure that the resulting path is closed. */
-	if ((pathHead->cifp_x != path->cifp_x) ||	
+	if ((pathHead->cifp_x != path->cifp_x) ||
 	    (pathHead->cifp_y != path->cifp_y))
 	{
 	    next = (CIFPath *) mallocMagic((unsigned) (sizeof (CIFPath)));
@@ -1281,7 +1294,7 @@ path_inc:
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1308,7 +1321,7 @@ CIFFreePath(path)
 	path = path->cifp_next;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1334,7 +1347,7 @@ cifCommandError()
     CIFSkipToSemi();
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1364,7 +1377,7 @@ cifParseEnd()
     }
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1386,7 +1399,7 @@ cifParseComment()
 {
     int		opens;
     int		ch;
-    
+
 	/*
 	 *	take the '('
 	 */
@@ -1411,7 +1424,7 @@ cifParseComment()
     } while (opens > 0);
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1453,7 +1466,7 @@ CIFDirectionToTrans(point)
 	point->p_x, point->p_y);
     return &GeoIdentityTransform;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1548,7 +1561,7 @@ CIFParseTransform(transformp)
 
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1585,6 +1598,7 @@ CIFReadFile(file)
     cifTotalWarnings = 0;
     cifTotalErrors = 0;
     CifPolygonCount = 0;
+    cifSeenSnapWarning = FALSE;
 
     cifInputFile = file;
     cifReadScale1 = 1;
