@@ -3,16 +3,16 @@
  *
  * Commands with names beginning with the letters L through Q.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -50,7 +50,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 void CmdPaintEraseButton();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -125,7 +125,7 @@ CmdLabelProc(text, font, size, rotate, offx, offy, pos, sticky, type)
     DBWLabelChanged(EditCellUse->cu_def, lab, DBW_ALLWINDOWS);
     lab->lab_rect = tmpArea;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -281,7 +281,7 @@ CmdLabel(w, cmd)
 		pos = GeoTransPos(&RootToEditTransform, pos);
 	}
     }
-    
+
     /*
      * Find and check validity of type parameter.  Accept prefix "-" on
      * layer as an indication of a "sticky" label (label is fixed to
@@ -309,7 +309,7 @@ CmdLabel(w, cmd)
     CmdLabelProc(p, font, size, rotate, offx, offy, pos, sticky, type);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -318,7 +318,7 @@ CmdLabel(w, cmd)
  * Implement the "load" command.
  *
  * Usage:
- *	load [name [scaled n [d]]] [-force] [-nowindow]
+ *	load [name [scaled n [d]]] [-force] [-nowindow] [-dereference]
  *
  * If name is supplied, then the window containing the point tool is
  * remapped so as to edit the cell with the given name.
@@ -331,6 +331,13 @@ CmdLabel(w, cmd)
  * An input file can be scaled by specifying the "scaled" option, for
  * which the geometry of the input file is multiplied by n/d.
  *
+ * Magic saves the path to instances to ensure correct versioning.  But
+ * this interferes with attempts to re-link instances from a different
+ * location (such as an abstract view instead of a full view, or vice
+ * versa).  So the "-dereference" option strips the instance paths from
+ * the input file and relies only on the search locations set up by the
+ * "path" command to find the location of instances.
+ *
  * Results:
  *	None.
  *
@@ -350,6 +357,7 @@ CmdLoad(w, cmd)
     int locargc = cmd->tx_argc;
     bool ignoreTech = FALSE;
     bool noWindow = FALSE;
+    bool dereference = FALSE;
     int keepGoing();			/* forward declaration */
 
     if (locargc > 2)
@@ -359,6 +367,11 @@ CmdLoad(w, cmd)
 	    locargc--;
 	    noWindow = TRUE;
 	}
+	if (!strncmp(cmd->tx_argv[locargc - 1], "-deref", 5))
+	{
+	    locargc--;
+	    dereference = TRUE;
+	}
 	if (!strncmp(cmd->tx_argv[locargc - 1], "-force", 6))
 	{
 	    locargc--;
@@ -372,7 +385,8 @@ CmdLoad(w, cmd)
 	        d = atoi(cmd->tx_argv[4]);
 	    else if (locargc != 4)
 	    {
-		TxError("Usage: %s name scaled n [d] [-force] [-nowindow]\n",
+		TxError("Usage: %s name scaled n [d] [-force] "
+			    "[-nowindow] [-dereference]\n",
 			    cmd->tx_argv[0]);
 		return;
 	    }
@@ -380,9 +394,10 @@ CmdLoad(w, cmd)
 	    DBLambda[1] *= n;
 	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
 	}
-	else if (!ignoreTech && !noWindow)
+	else if (!ignoreTech && !noWindow && !dereference)
 	{
-	    TxError("Usage: %s name [scaled n [d]] [-force] [-nowindow]\n",
+	    TxError("Usage: %s name [scaled n [d]] [-force] "
+			    "[-nowindow] [-dereference]\n",
 			    cmd->tx_argv[0]);
 	    return;
 	}
@@ -408,7 +423,7 @@ CmdLoad(w, cmd)
 	}
 #endif
 	DBWloadWindow((noWindow == TRUE) ? NULL : w, cmd->tx_argv[1],
-			ignoreTech, FALSE);
+			ignoreTech, FALSE, dereference);
 
 	if ((n > 1) || (d > 1))
 	{
@@ -441,7 +456,7 @@ CmdLoad(w, cmd)
 	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
 	}
     }
-    else DBWloadWindow(w, (char *) NULL, TRUE, FALSE);
+    else DBWloadWindow(w, (char *) NULL, TRUE, FALSE, FALSE);
 }
 
 /*
@@ -465,8 +480,8 @@ keepGoing(use, clientdata)
  * Implement the "move" command.
  *
  * Usage:
- *	move [direction [amount]]
- *	move to x y
+ *	move [origin] [direction [amount]]
+ *	move [origin] to x y
  *
  * Results:
  *	None.
@@ -487,12 +502,13 @@ CmdMove(w, cmd)
     Point rootPoint, editPoint;
     CellDef *rootDef;
     int argpos;
+    bool doOrigin = FALSE;
 
     if (cmd->tx_argc > 4)
     {
 	badUsage:
-	TxError("Usage: %s [direction [amount]]\n", cmd->tx_argv[0]);
-	TxError("   or: %s to x y\n", cmd->tx_argv[0]);
+	TxError("Usage: %s [origin] [direction [amount]]\n", cmd->tx_argv[0]);
+	TxError("   or: %s [origin] to x y\n", cmd->tx_argv[0]);
 	return;
     }
 
@@ -503,18 +519,26 @@ CmdMove(w, cmd)
 
 	if (!ToolGetEditBox((Rect *)NULL)) return;
 
-	if (strcmp(cmd->tx_argv[1], "to") == 0)
+	argpos = 1;
+
+	if (strcmp(cmd->tx_argv[1], "origin") == 0)
+	{
+	    doOrigin = TRUE;
+	    argpos++;
+	}
+
+	if (strcmp(cmd->tx_argv[argpos], "to") == 0)
 	{
 	    if (cmd->tx_argc != 4)
 		goto badUsage;
-	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
-	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[argpos + 1], FALSE, TRUE);
+	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[argpos + 2], FALSE, FALSE);
 	    GeoTransPoint(&EditToRootTransform, &editPoint, &rootPoint);
 	    goto moveToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, FALSE);
-	argpos = (indx < 0) ? 1 : 2;
+	indx = GeoNameToPos(cmd->tx_argv[argpos], FALSE, FALSE);
+	if (indx >= 0) argpos++;
 
 	if (cmd->tx_argc >= 3)
 	{
@@ -590,8 +614,9 @@ CmdMove(w, cmd)
 	 * but is retained for backward compatibility.
 	 */
 
-	if (ToolGetBox(&rootDef, &rootBox) && ((rootDef == SelectRootDef)
-                                           || (SelectRootDef == NULL)))
+	if (ToolGetBox(&rootDef, &rootBox) &&
+		((rootDef == SelectRootDef) || (SelectRootDef == NULL))
+		&& (doOrigin == FALSE))
 	{
 	    GeoTransRect(&t, &rootBox, &newBox);
 	    DBWSetBox(rootDef, &newBox);
@@ -602,7 +627,7 @@ CmdMove(w, cmd)
 	/* Use the displacement between the box lower-left corner and
 	 * the point as the transform.
 	 */
-	
+
 	MagWindow *window;
 
 	window = ToolGetPoint(&rootPoint, (Rect *) NULL);
@@ -625,13 +650,43 @@ moveToPoint:
 	}
 	GeoTransTranslate(rootPoint.p_x - rootBox.r_xbot,
 	    rootPoint.p_y - rootBox.r_ybot, &GeoIdentityTransform, &t);
-	GeoTransRect(&t, &rootBox, &newBox);
-	DBWSetBox(rootDef, &newBox);
+	if (doOrigin == FALSE)
+	{
+	    GeoTransRect(&t, &rootBox, &newBox);
+	    DBWSetBox(rootDef, &newBox);
+	}
     }
-    
-    SelectTransform(&t);
+
+    if (doOrigin)
+    {
+	DBMoveCell(rootDef, t.t_c, t.t_f);
+
+	/* Adjust box to maintain relative position */
+
+	if (ToolGetBox(&rootDef, &rootBox) &&
+		((rootDef == SelectRootDef) || (SelectRootDef == NULL)))
+	{
+	    t.t_c = -t.t_c;
+	    t.t_f = -t.t_f;
+	    GeoTransRect(&t, &rootBox, &newBox);
+	    DBWSetBox(rootDef, &newBox);
+	    t.t_c = -t.t_c;
+	    t.t_f = -t.t_f;
+	}
+
+        /* Adjust all window viewing positions and redraw */
+
+        WindTranslate(t.t_c, t.t_f);
+
+        /* This is harsh.  Might work to change all distance measures	*/
+        /* in the undo record, but this is simple and direct.           */
+
+        UndoFlush();
+    }
+    else
+	SelectTransform(&t);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -703,7 +758,7 @@ CmdPaint(w, cmd)
     if (DRCBackGround)
 	DRCCheckThis (EditCellUse->cu_def, TT_CHECKPAINT, &editRect);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -741,7 +796,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 
     WindPointToSurface(w, butPoint, (Point *) NULL, &rootRect);
 
-    DBSeeTypesAll(((CellUse *)w->w_surfaceID), &rootRect, 
+    DBSeeTypesAll(((CellUse *)w->w_surfaceID), &rootRect,
 	    crec->dbw_bitmask, &mask);
     TTMaskAndMask(&mask, &DBActiveLayerBits);
     TTMaskAndMask(&mask, &crec->dbw_visibleLayers);
@@ -912,7 +967,7 @@ CmdPath(w, cmd)
 	(void) StrDup(pathptr, srcptr);
     }
     return;
-	
+
 usage:
     TxError("Usage: %s [search|cell|sys] [[+]path]\n", cmd->tx_argv[0]);
 }
@@ -1060,32 +1115,45 @@ cmdPortLabelFunc2(scx, label, tpath, cdata)
 /* Find a label in the cell editDef.					*/
 /*									*/
 /* If "port" is true, then search only for labels that are ports.	*/
+/*    If false, then search only for labels that are not ports.		*/
 /* If "unique" is true, then return a label only if exactly one label	*/
-/* is found in the edit box.						*/ 
+/* is found in the edit box.						*/
 /*----------------------------------------------------------------------*/
 
 Label *
 portFindLabel(editDef, port, unique, nonEdit)
     CellDef *editDef;
     bool unique;
-    bool port;
+    bool port;		// If TRUE, only look for labels that are ports
     bool *nonEdit;	// TRUE if label is not in the edit cell
 {
-    int found;
+    int found, wrongkind;
     Label *lab, *sl;
     Rect editBox;
 
     /*
-     * Check for unique label in box area
+     * Check for unique label in box area.  Note that GEO_OVERLAP
+     * requires non-zero overlap area, so also check GEO_SURROUND to
+     * catch zero-area label rectangles.
      */
 
     ToolGetEditBox(&editBox);
     found = 0;
+    wrongkind = FALSE;
     if (nonEdit) *nonEdit = FALSE;
     lab = NULL;
     for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
     {
-	if (GEO_OVERLAP(&editBox, &sl->lab_rect))
+	/* Ignore labels based on whether label is or is not a port */
+	if (((port == TRUE) && !(sl->lab_flags & PORT_DIR_MASK)) ||
+		    ((port == FALSE) && (sl->lab_flags & PORT_DIR_MASK)))
+	{
+	    wrongkind = TRUE;	/* Found at least one label of the wrong kind */
+	    continue;
+	}
+
+	if (GEO_OVERLAP(&editBox, &sl->lab_rect) ||
+			GEO_SURROUND(&editBox, &sl->lab_rect))
 	{
 	    if (found > 0)
 	    {
@@ -1109,11 +1177,13 @@ portFindLabel(editDef, port, unique, nonEdit)
 	    if (nonEdit) *nonEdit = FALSE;
 	}
     }
+    if ((found == 0) && (wrongkind == TRUE)) return NULL;
 
     /* If no label was found, then search the hierarchy under the box.	*/
     /* The calling routine may determine whether a label that is not in	*/
     /* the edit cell may be valid for the command (e.g., if querying	*/
-    /* but not changing values).					*/
+    /* but not changing values).  NOTE:  Currently this does not apply	*/
+    /* the "port" option.						*/
 
     if (found == 0)
     {
@@ -1135,9 +1205,32 @@ portFindLabel(editDef, port, unique, nonEdit)
 	if (lab != NULL)
 	    if (nonEdit) *nonEdit = TRUE;
     }
-    
+
     return lab;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * complabel() --
+ *
+ * qsort() callback routine used by CmdPort when renumbering ports.
+ * Simply do a string comparison on the two labels.  There is no special
+ * meaning to the lexigraphic ordering;  it is meant only to enforce a
+ * consistent and repeatable port order.  However, note that since SPICE
+ * is case-insensitive, case-insensitive string comparison is used.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+complabel(const void *one, const void *two)
+{
+    Label *l1 = *((Label **)one);
+    Label *l2 = *((Label **)two);
+    return strcasecmp(l1->lab_text, l2->lab_text);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1149,9 +1242,11 @@ portFindLabel(editDef, port, unique, nonEdit)
  * usage, below).
  *
  * Usage:
- *	port make|makeall [num] [connect_direction(s)]
+ *	port make [num] [connect_direction(s)]
  * or
- *	port [name|num] class|use|index [value]
+ *	port makeall|renumber [connect_direction(s)]
+ * or
+ *	port [name|num] class|use|shape|index [value]
  *
  * num is the index of the port, usually beginning with 1.  This indicates
  *	the order in which ports should be written to a subcircuit record
@@ -1180,16 +1275,20 @@ portFindLabel(editDef, port, unique, nonEdit)
 
 #define PORT_CLASS	0
 #define PORT_USE	1
-#define PORT_INDEX	2
-#define PORT_EQUIV	3
-#define PORT_EXISTS	4
-#define PORT_CONNECT	5
-#define PORT_LAST	6
-#define PORT_MAKE	7
-#define PORT_MAKEALL	8
-#define PORT_NAME	9
-#define PORT_REMOVE	10
-#define PORT_HELP	11	
+#define PORT_SHAPE	2
+#define PORT_INDEX	3
+#define PORT_EQUIV	4
+#define PORT_EXISTS	5
+#define PORT_CONNECT	6
+#define PORT_FIRST	7
+#define PORT_NEXT	8
+#define PORT_LAST	9
+#define PORT_MAKE	10
+#define PORT_MAKEALL	11
+#define PORT_NAME	12
+#define PORT_REMOVE	13
+#define PORT_RENUMBER	14
+#define PORT_HELP	15
 
 void
 CmdPort(w, cmd)
@@ -1198,8 +1297,8 @@ CmdPort(w, cmd)
 {
     char **msg;
     int argstart;
-    int i, idx, pos, type, option, argc;
-    unsigned short dirmask;
+    int i, refidx, idx, pos, type, option, argc;
+    unsigned int dirmask;
     bool found;
     bool nonEdit = FALSE;
     Label *lab, *sl;
@@ -1210,15 +1309,19 @@ CmdPort(w, cmd)
     {
 	"class	[type]		get [set] port class type",
 	"use	[type]		get [set] port use type",
+	"shape	[type]		get [set] port shape type",
 	"index	[number]	get [set] port number",
 	"equivalent [number]	make port equivalent to another port",
 	"exists			report if a label is a port or not",
 	"connections [dir...]	get [set] port connection directions",
+	"first			report the lowest port number used",
+	"next [number]		report the next port number used",
 	"last			report the highest port number used",
 	"make [index] [dir...]	turn a label into a port",
 	"makeall [index] [dir]	turn all labels into ports",
 	"name			report the port name",
 	"remove			turn a port back into a label",
+	"renumber		renumber all ports",
 	"help			print this help information",
 	NULL
     };
@@ -1271,6 +1374,25 @@ CmdPort(w, cmd)
 	PORT_USE_CLOCK
     };
 
+    static char *cmdPortShapeTypes[] =
+    {
+	"default",
+	"abutment",
+	"ring",
+	"feedthrough",
+	"feedthru",
+	NULL
+    };
+
+    static int cmdShapeToBitmask[] =
+    {
+	PORT_SHAPE_DEFAULT,
+	PORT_SHAPE_ABUT,
+	PORT_SHAPE_RING,
+	PORT_SHAPE_THRU,
+	PORT_SHAPE_THRU
+    };
+
     argstart = 1;
     argc = cmd->tx_argc;
     if (argc > 6 || argc == 1)
@@ -1297,7 +1419,7 @@ CmdPort(w, cmd)
 			if (sl && ((sl->lab_flags & PORT_DIR_MASK) != 0))
 			    if ((sl->lab_flags & PORT_NUM_MASK) == portidx)
 			    {
-				lab = sl;
+			    	lab = sl;
 				break;
 			    }
 		}
@@ -1309,12 +1431,16 @@ CmdPort(w, cmd)
 			if (!strcmp(portname, sl->lab_text))
 			{
 			    lab = sl;
-			    break;
+			    if (sl && ((sl->lab_flags & PORT_DIR_MASK) != 0))
+			    	break;
 			}
 		}
 		if (lab == NULL)
 		{
-		    TxError("No label found with that name\n.");
+		    if (StrIsInt(cmd->tx_argv[1]))
+			TxError("No label found with index %s.\n", cmd->tx_argv[1]);
+		    else
+			TxError("No port found with name %s.\n", cmd->tx_argv[1]);
 		    return;
 		}
 		argstart = 2;
@@ -1332,10 +1458,15 @@ CmdPort(w, cmd)
     {
 	/* Check for options that require only one selected port */
 
-	if (option != PORT_LAST)
+	if (option != PORT_LAST && option != PORT_FIRST)
 	{
 	    if (lab == NULL)
-		lab = portFindLabel(editDef, TRUE, TRUE, &nonEdit);
+	    {
+		if (option == PORT_MAKE)
+		    lab = portFindLabel(editDef, FALSE, TRUE, &nonEdit);
+		else
+		    lab = portFindLabel(editDef, TRUE, TRUE, &nonEdit);
+	    }
 
 	    if (option == PORT_EXISTS)
 	    {
@@ -1353,13 +1484,17 @@ CmdPort(w, cmd)
 #endif
 		return;
 	    }
-
 	}
-	if ((option != PORT_LAST) && (option != PORT_MAKEALL) && (lab == NULL))
+	if ((option != PORT_LAST) && (option != PORT_FIRST) &&
+		(option != PORT_MAKEALL) && (option != PORT_RENUMBER)
+		&& (lab == NULL))
 	{
 	    /* Let "port remove" fail without complaining. */
 	    if (option != PORT_REMOVE)
+	    {
 		TxError("Exactly one label may be present under the cursor box.\n");
+		TxError("Use \"port <name> ...\" to specify a uniqe port.\n");
+	    }
 
 	    return;
 	}
@@ -1367,7 +1502,8 @@ CmdPort(w, cmd)
 	/* Check for options that require label to be a port */
 
 	if ((option != PORT_MAKE) && (option != PORT_MAKEALL)
-		&& (option != PORT_EXISTS) && (option != PORT_LAST))
+		&& (option != PORT_EXISTS) && (option != PORT_RENUMBER)
+		&& (option != PORT_LAST) && (option != PORT_FIRST))
 	{
 	    /* label "lab" must already be a port */
 	    if (!(lab->lab_flags & PORT_DIR_MASK))
@@ -1381,8 +1517,9 @@ CmdPort(w, cmd)
 	/* Check for options that cannot operate on a non-edit cell label */
 	if (nonEdit)
 	{
-	    if ((option == PORT_MAKE) || (option == PORT_MAKEALL) ||
-		(option == PORT_REMOVE) || (argc == 3))
+	    if ((option == PORT_MAKE) || (option == PORT_MAKEALL)
+		    || (option == PORT_REMOVE) || (option == PORT_RENUMBER)
+		    || (argc == 3))
 	    {
 		TxError("Cannot modify a port in an non-edit cell.\n");
 		return;
@@ -1406,6 +1543,44 @@ CmdPort(w, cmd)
 #endif
 		break;
 
+	    case PORT_FIRST:
+		i = PORT_NUM_MASK + 1;
+		for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+		{
+		    if (sl->lab_flags & PORT_DIR_MASK)
+		    {
+		    	idx = sl->lab_flags & PORT_NUM_MASK;
+		    	if (idx < i) i = idx;
+		    }
+		}
+		if (i == PORT_NUM_MASK + 1) i = -1;
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(i));
+#else
+		TxPrintf("%d\n", i);
+#endif
+		break;
+
+	    case PORT_NEXT:
+		refidx = lab->lab_flags & PORT_NUM_MASK;
+		i = PORT_NUM_MASK + 1;
+		for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+		{
+		    if (sl->lab_flags & PORT_DIR_MASK)
+		    {
+		    	idx = sl->lab_flags & PORT_NUM_MASK;
+		    	if (idx > refidx)
+		            if (idx < i) i = idx;
+		    }
+		}
+		if (i == PORT_NUM_MASK + 1) i = -1;
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(i));
+#else
+		TxPrintf("Index = %d\n", i);
+#endif
+		break;
+
 	    case PORT_EXISTS:
 		if (!(lab->lab_flags & PORT_DIR_MASK))
 		{
@@ -1476,8 +1651,16 @@ CmdPort(w, cmd)
 		    }
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_CLASS_MASK);
-			lab->lab_flags |= (PORT_CLASS_MASK & cmdClassToBitmask[type]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_CLASS_MASK);
+				sl->lab_flags |= (PORT_CLASS_MASK &
+					    cmdClassToBitmask[type]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1513,8 +1696,60 @@ CmdPort(w, cmd)
 		    }
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_USE_MASK);
-			lab->lab_flags |= (PORT_USE_MASK & cmdUseToBitmask[type]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_USE_MASK);
+				sl->lab_flags |= (PORT_USE_MASK & cmdUseToBitmask[type]);
+			    }
+			}
+			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		    }
+		}
+		else
+		    goto portWrongNumArgs;
+		break;
+
+	    case PORT_SHAPE:
+		if (argc == 2)
+		{
+		    type = lab->lab_flags & PORT_SHAPE_MASK;
+		    for (idx = 0; cmdPortShapeTypes[idx] != NULL; idx++)
+			if (cmdShapeToBitmask[idx] == type)
+			{
+#ifdef MAGIC_WRAPPER
+			    Tcl_AppendResult(magicinterp, cmdPortShapeTypes[idx],
+					NULL);
+#else
+			    TxPrintf("Shape = %s\n", cmdPortShapeTypes[idx]);
+#endif
+			    break;
+			}
+		}
+		else if (argc == 3)
+		{
+		    type = Lookup(cmd->tx_argv[argstart + 1], cmdPortShapeTypes);
+		    if (type < 0)
+		    {
+		        TxError("Usage:  port shape <type>, where <type> is one of:\n");
+			for (msg = &(cmdPortShapeTypes[0]); *msg != NULL; msg++)
+			{
+                	    TxError("    %s\n", *msg);
+			}
+		    }
+		    else
+		    {
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_SHAPE_MASK);
+				sl->lab_flags |= (PORT_SHAPE_MASK & cmdShapeToBitmask[type]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1538,8 +1773,15 @@ CmdPort(w, cmd)
 		        TxError("Usage:  port index <integer>\n");
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_NUM_MASK);
-			lab->lab_flags |= atoi(cmd->tx_argv[argstart + 1]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_NUM_MASK);
+				sl->lab_flags |= atoi(cmd->tx_argv[argstart + 1]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1580,6 +1822,57 @@ CmdPort(w, cmd)
 		editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		break;
 
+	    case PORT_RENUMBER:
+		/* Renumber ports in canonical order (by alphabetical
+		 * order of the label text).  NOTE:  Because SPICE is
+		 * case-insensitive, case-insensitive alphabetical order
+		 * is used.
+		 */
+		{
+		    int numlabels, n, p;
+		    Label **slablist, *tlab, *lastlab;
+		    extern int complabel();
+
+		    /* Create a sortable list of labels */
+		    numlabels = 0;
+		    for (lab = editDef->cd_labels; lab; lab = lab->lab_next)
+			numlabels++;
+
+		    slablist = (Label **)mallocMagic(numlabels * sizeof(Label *));
+		    numlabels = 0;
+		    for (lab = editDef->cd_labels; lab; lab = lab->lab_next)
+		    {
+			*(slablist + numlabels) = lab;
+			numlabels++;
+		    }
+
+		    /* Sort the list */
+		    qsort(slablist, numlabels, sizeof(Label *), complabel);
+
+		    /* Number the ports by sorted order */
+		    p = 0;
+		    lastlab = NULL;
+		    for (n = 0; n < numlabels; n++)
+		    {
+			tlab = *(slablist + n);
+			if (tlab->lab_flags & PORT_DIR_MASK)
+			{
+			    if (lastlab)
+				if (!strcmp(lastlab->lab_text, tlab->lab_text))
+				    p--;
+
+			    tlab->lab_flags &= ~PORT_NUM_MASK;
+			    tlab->lab_flags |= p;
+			    lastlab = tlab;
+			    p++;
+			}
+		    }
+
+		    /* Clean up */
+		    freeMagic((char *)slablist);
+		}
+		break;
+
 	    case PORT_MAKEALL:
 		lab = editDef->cd_labels;
 		// Fall through. . .
@@ -1601,7 +1894,7 @@ portWrongNumArgs:
 	}
 	return;
     }
-    
+
 parseindex:
 
     if ((option != PORT_MAKEALL) && (lab->lab_flags & PORT_DIR_MASK))
@@ -1662,7 +1955,7 @@ parseindex:
 			break;
 		    }
 		    else if ((sl->lab_flags & PORT_NUM_MASK) > idx)
-			idx = (sl->lab_flags & PORT_NUM_MASK); 
+			idx = (sl->lab_flags & PORT_NUM_MASK);
 		}
 	    }
 	    idx++;
@@ -1782,7 +2075,7 @@ CmdDoProperty(def, cmd, argstart)
     char *value;
     bool propfound;
     int locargc = cmd->tx_argc - argstart + 1;
-    
+
     if (locargc == 1)
     {
 	/* print all properties and their values */
@@ -1817,7 +2110,7 @@ CmdDoProperty(def, cmd, argstart)
 	    DBPropPut(def, cmd->tx_argv[argstart], value);
 	}
 	def->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
-    }    
+    }
     else
     {
 	TxError("Usage: property [name] [value]\n");
@@ -1896,7 +2189,7 @@ printPropertiesFunc(name, value)
     }
     Tcl_AppendElement(magicinterp, keyvalue);
     freeMagic(keyvalue);
-    
+
 #else
     TxPrintf("%s = %s\n", name, value);
 #endif
@@ -1937,7 +2230,7 @@ CmdNetlist(w, cmd)
     char **msg, *lastargv;
     Point cursor;
     static char *cmdNetlistOption[] =
-    {	
+    {
 	"help           print this help information",
 	"select		select the net nearest the cursor",
 	"join           join current net and net containing terminal nearest the cursor",
@@ -1983,3 +2276,158 @@ CmdNetlist(w, cmd)
 	    break;
     }
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdOrient --
+ *
+ * Implement the "orient" command.  Set the orientation of the selection and
+ * according to the orientation string, which can be in Magic's format
+ * (using number of degrees, and "h" for horizontal flip or "v" for vertical
+ * flip) or DEF format ("N", "S", "FN", etc.).  The cursor box is rotated to
+ * match.
+ *
+ * Usage:
+ *	orient [orientation] [-origin]
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modifies the edit cell.
+ *
+ * Notes:
+ *	The "-origin" option sets the orientation relative to the origin,
+ *	instead of the lower-left corner.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdOrient(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    Transform trans, t2;
+    int orientidx, locargc;
+    char *orientstr;
+    Rect rootBox,  bbox;
+    CellDef *rootDef;
+    bool noAdjust = FALSE;
+
+    static char *orientNames[] = {      "0", "90", "180", "270",
+                                        "v", "0v", "90v", "180v", "270v",
+                                        "h", "0h", "90h", "180h", "270h",
+                                        "N", "E", "S", "W",
+                                        "FN", "FE", "FS", "FW",
+                                        0 };
+
+    typedef enum { IDX_ORIENT_0, IDX_ORIENT_90, IDX_ORIENT_180, IDX_ORIENT_270,
+            IDX_ORIENT_V, IDX_ORIENT_0V, IDX_ORIENT_90V, IDX_ORIENT_180V,
+            IDX_ORIENT_270V, IDX_ORIENT_H, IDX_ORIENT_0H, IDX_ORIENT_90H,
+            IDX_ORIENT_180H, IDX_ORIENT_270H, IDX_ORIENT_N, IDX_ORIENT_E,
+            IDX_ORIENT_S, IDX_ORIENT_W, IDX_ORIENT_FN, IDX_ORIENT_FE,
+            IDX_ORIENT_FS, IDX_ORIENT_FW } orientType;
+
+    locargc = cmd->tx_argc;
+    if (!strncmp(cmd->tx_argv[locargc - 1], "-orig", 5))
+    {
+	noAdjust = TRUE;
+	locargc--;
+    }
+
+    if (!ToolGetEditBox((Rect *)NULL)) return;
+
+    if (locargc == 2)
+	orientstr = cmd->tx_argv[1];
+    else
+	goto badusage;
+
+    orientidx = Lookup(orientstr, orientNames);
+
+    switch (orientidx)
+    {
+        case IDX_ORIENT_0:
+        case IDX_ORIENT_N:
+            /* ORIENT_NORTH */
+            t2 = GeoIdentityTransform;
+            break;
+        case IDX_ORIENT_S:
+        case IDX_ORIENT_180:
+            /* ORIENT_SOUTH */
+            t2 = Geo180Transform;
+            break;
+        case IDX_ORIENT_E:
+        case IDX_ORIENT_90:
+            /* ORIENT_EAST */
+            t2 = Geo90Transform;
+            break;
+        case IDX_ORIENT_W:
+        case IDX_ORIENT_270:
+            /* ORIENT_WEST */
+            t2 = Geo270Transform;
+            break;
+        case IDX_ORIENT_FN:
+        case IDX_ORIENT_H:
+        case IDX_ORIENT_0H:
+            /* ORIENT_FLIPPED_NORTH */
+            t2 = GeoSidewaysTransform;
+            break;
+        case IDX_ORIENT_FS:
+        case IDX_ORIENT_180H:
+        case IDX_ORIENT_V:
+        case IDX_ORIENT_0V:
+            /* ORIENT_FLIPPED_SOUTH */
+            t2 = GeoUpsideDownTransform;
+            break;
+        case IDX_ORIENT_FE:
+        case IDX_ORIENT_90H:
+        case IDX_ORIENT_270V:
+            /* ORIENT_FLIPPED_EAST */
+            t2 = GeoRef135Transform;
+            break;
+        case IDX_ORIENT_FW:
+        case IDX_ORIENT_270H:
+        case IDX_ORIENT_90V:
+            /* ORIENT_FLIPPED_WEST */
+            t2 = GeoRef45Transform;
+            break;
+	default:
+	    goto badusage;
+    }
+
+    /* To orient the selection, first orient it relative to the origin
+     * then move it so its lower-left corner is at the same place
+     * that it used to be.  If the "-origin" option was selected, then
+     * only orient relative to the origin.
+     */
+
+    GeoTransRect(&t2, &SelectDef->cd_bbox, &bbox);
+    if (noAdjust)
+	trans = t2;
+    else
+    {
+	GeoTranslateTrans(&t2, SelectDef->cd_bbox.r_xbot - bbox.r_xbot,
+		SelectDef->cd_bbox.r_ybot - bbox.r_ybot, &trans);
+    }
+
+    SelectTransform(&trans);
+
+    /* Rotate the box, if it exists and is in the same window as the
+     * selection.
+     */
+
+    if (ToolGetBox(&rootDef, &rootBox) && (rootDef == SelectRootDef))
+    {
+	Rect newBox;
+
+	GeoTransRect(&trans, &rootBox, &newBox);
+	DBWSetBox(rootDef, &newBox);
+    }
+
+    return;
+
+    badusage:
+    TxError("Usage: %s [orientation]\n", cmd->tx_argv[0]);
+}
