@@ -3,16 +3,16 @@
  *
  * Area searching which spans cell boundaries.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -21,6 +21,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
 
 #include "utils/magic.h"
 #include "utils/malloc.h"
@@ -30,6 +31,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
+#include "dbwind/dbwind.h"
 #include "textio/textio.h"
 #include "utils/signals.h"
 #include "windows/windows.h"
@@ -51,7 +53,53 @@ struct seeTypesArg
     TileTypeBitMask *saa_mask;	/* Mask of tile types seen in search */
     Rect *saa_rect;		/* Search area in root coordinates */
 };
-
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DBSrCellPlaneArea --
+ *
+ * Searches a CellDef's cell plane and calls function func() for each
+ * cell use found.
+ *
+ * func() must be in the form:
+ *
+ *	int func(CellUse *use, ClientData cdata)
+ *
+ * and must return 0 to keep the search running, or 1 to end the search.
+ *
+ * Replaces the original TiSrArea() routine, but with the function's
+ * first argument as a CellUse pointer rather than a Tile pointer,
+ * since the tile plane has been replaced with the BPlane method.
+ *
+ * Returns 1 if the func() returns 1;  otherwise returns 0 to keep the
+ * search alive.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+DBSrCellPlaneArea(BPlane *plane, Rect *rect, int (*func)(), ClientData arg)
+{
+    BPEnum bpe;
+    CellUse *use;
+    int rval = 0;
+
+    BPEnumInit(&bpe, plane, rect, BPE_OVERLAP, "DBSrCellPlaneArea");
+
+    while (use = BPEnumNext(&bpe))
+    {
+	if ((*func)(use, arg))
+	{
+	    rval = 1;
+	    break;
+	}
+    }
+
+    BPEnumTerm(&bpe);
+    return rval;
+}
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -81,7 +129,7 @@ struct seeTypesArg
  *
  * The client procedure should not modify any of the paint planes in
  * the cells visited by DBTreeSrTiles, because we use DBSrPaintArea
- * instead of TiSrArea as our paint-tile enumeration function.
+ * as our paint-tile enumeration function.
  *
  * Results:
  *	0 is returned if the search finished normally.  1 is returned
@@ -112,7 +160,7 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
-    int dbCellTileSrFunc();
+    int dbCellPlaneSrFunc();
     TreeFilter filter;
 
     /* Set up the filter and call the recursive filter function */
@@ -124,7 +172,7 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     filter.tf_dinfo = 0;
     filter.tf_planes = DBTechTypesToPlanes(mask);
 
-    return dbCellTileSrFunc(scx, &filter);
+    return dbCellPlaneSrFunc(scx, &filter);
 }
 
 /*
@@ -154,7 +202,7 @@ DBTreeSrNMTiles(scx, dinfo, mask, xMask, func, cdarg)
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
-    int dbCellTileSrFunc();
+    int dbCellPlaneSrFunc();
     TreeFilter filter;
 
     /* Set up the filter and call the recursive filter function */
@@ -166,17 +214,17 @@ DBTreeSrNMTiles(scx, dinfo, mask, xMask, func, cdarg)
     filter.tf_dinfo = dinfo;
     filter.tf_planes = DBTechTypesToPlanes(mask);
 
-    return dbCellTileSrFunc(scx, &filter);
+    return dbCellPlaneSrFunc(scx, &filter);
 }
 
 /*
- * dbCellTileSrFunc --
+ * dbCellPlaneSrFunc --
  *
  * Recursive filter procedure applied to the cell by DBTreeSrTiles().
  */
 
 int
-dbCellTileSrFunc(scx, fp)
+dbCellPlaneSrFunc(scx, fp)
     SearchContext *scx;
     TreeFilter *fp;
 {
@@ -184,11 +232,14 @@ dbCellTileSrFunc(scx, fp)
     CellDef *def = scx->scx_use->cu_def;
     int pNum;
 
-    ASSERT(def != (CellDef *) NULL, "dbCellTileSrFunc");
+    ASSERT(def != (CellDef *) NULL, "dbCellPlaneSrFunc");
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
 
     context.tc_scx = scx;
     context.tc_filter = fp;
@@ -219,14 +270,14 @@ dbCellTileSrFunc(scx, fp)
 
     /*
      * Now apply ourselves recursively to each of the CellUses
-     * in our tile plane.
+     * in our cell plane.
      */
 
-    if (DBCellSrArea(scx, dbCellTileSrFunc, (ClientData) fp))
+    if (DBCellSrArea(scx, dbCellPlaneSrFunc, (ClientData) fp))
 	return 1;
     else return 0;
 }
-
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -270,7 +321,7 @@ DBTreeSrUniqueTiles(scx, mask, xMask, func, cdarg)
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
-    int dbCellTileSrFunc();
+    int dbCellPlaneSrFunc();
     TreeFilter filter;
 
     /* Set up the filter and call the recursive filter function */
@@ -288,7 +339,7 @@ DBTreeSrUniqueTiles(scx, mask, xMask, func, cdarg)
  * dbCellUniqueTileSrFunc --
  *
  * Recursive filter procedure applied to the cell by DBTreeSrUniqueTiles().
- * This is similar to dbCellTileSrFunc, except that for each plane searched,
+ * This is similar to dbCellPlaneSrFunc, except that for each plane searched,
  * only the tile types having that plane as their home plane will be passed
  * to the filter function.  Contacts will therefore be processed only once.
  */
@@ -307,7 +358,10 @@ dbCellUniqueTileSrFunc(scx, fp)
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
 
     context.tc_scx = scx;
     context.tc_filter = fp;
@@ -342,14 +396,12 @@ dbCellUniqueTileSrFunc(scx, fp)
     else return 0;
 }
 
-
+
 /*
  *-----------------------------------------------------------------------------
  *
  * DBNoTreeSrTiles --
  *
- * (*** Move to database module after tested) 
- * 
  * NOTE: THIS PROCEDURE IS EXACTLY LIKE DBTreeSrTiles EXCEPT THAT IT DOES
  * NOT SEARCH SUBCELLS.
  *
@@ -377,7 +429,7 @@ dbCellUniqueTileSrFunc(scx, fp)
  *
  * The client procedure should not modify any of the paint planes in
  * the cells visited by DBTreeSrTiles, because we use DBSrPaintArea
- * instead of TiSrArea as our paint-tile enumeration function.
+ * as our paint-tile enumeration function.
  *
  * Results:
  *	0 is returned if the search finished normally.  1 is returned
@@ -418,7 +470,10 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
 	return 0;
 
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
 
     filter.tf_func = func;
     filter.tf_arg = cdarg;
@@ -446,7 +501,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
     return 0;
 }
 
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -526,7 +581,10 @@ DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
     if (!DBDescendSubcell(cellUse, xMask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
 
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
@@ -538,7 +596,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
 	    /* For non-manhattan searches, label must be in or	*/
 	    /* touch the triangle.  (to-do:  needs a proper	*/
 	    /* insideness test)					*/
-	
+
 	    if (flags & TF_LABEL_ATTACH_CORNER)
 	    {
 		Rect r1 = *r;
@@ -569,8 +627,11 @@ DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
 	    else
 		is_touching = GEO_TOUCH(&lab->lab_rect, r);
 	}
-	if (!is_touching && (flags & TF_LABEL_DISPLAY) && (lab->lab_font >= 0))
+	if (!is_touching && (flags & TF_LABEL_DISPLAY) && lab->lab_font >= 0)
+	{
+	    /* Check against bounds of the rendered label text */
 	    is_touching = GEO_TOUCH(&lab->lab_bbox, r);
+	}
 
 	if (is_touching && TTMaskHasType(mask, lab->lab_type))
 	    if ((*func)(scx, lab, tpath, cdarg))
@@ -592,7 +653,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
      * instead of just TOUCH.  Be careful when expanding:  can't expand
      * any coordinate past infinity.
      */
-    
+
     scx2 = *scx;
     if (scx2.scx_area.r_xbot > TiPlaneRect.r_xbot) scx2.scx_area.r_xbot -= 1;
     if (scx2.scx_area.r_ybot > TiPlaneRect.r_ybot) scx2.scx_area.r_ybot -= 1;
@@ -627,8 +688,11 @@ dbCellLabelSrFunc(scx, fp)
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE, NULL)) return 0;
-    
+    {
+	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
+
     if (fp->tf_tpath != (TerminalPath *) NULL)
     {
 	TerminalPath *tp = fp->tf_tpath;
@@ -678,7 +742,7 @@ cleanup:
 
     return (result);
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -741,8 +805,11 @@ DBTreeSrCells(scx, xMask, func, cdarg)
     if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
     if ((cellUse->cu_def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE, NULL))
+    {
+	bool dereference = (cellUse->cu_def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE, dereference, NULL))
 	    return 0;
+    }
 
     context.tc_scx = scx;
     context.tc_filter = &filter;
@@ -787,13 +854,16 @@ dbTreeCellSrFunc(scx, fp)
     else
     {
 	if ((use->cu_def->cd_flags & CDAVAILABLE) == 0)
-	    if (!DBCellRead(use->cu_def, (char *) NULL, TRUE, NULL))
+	{
+	    bool dereference = (use->cu_def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	    if (!DBCellRead(use->cu_def, (char *) NULL, TRUE, dereference, NULL))
 		return 0;
+	}
 	result = DBCellSrArea(scx, dbTreeCellSrFunc, (ClientData) fp);
     }
     return result;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -937,7 +1007,7 @@ DBSrRoots(baseDef, transform, func, cdarg)
     }
     return 0;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -976,7 +1046,7 @@ DBIsAncestor(cellDef1, cellDef2)
     }
     return (FALSE);
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1030,7 +1100,6 @@ DBCellSrArea(scx, func, cdarg)
 {
     TreeFilter filter;
     TreeContext context;
-    Rect expanded;
     int dbCellSrFunc();
 
     filter.tf_func = func;
@@ -1039,44 +1108,26 @@ DBCellSrArea(scx, func, cdarg)
     context.tc_scx = scx;
 
     if ((scx->scx_use->cu_def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(scx->scx_use->cu_def, (char *) NULL, TRUE, NULL))
+    {
+	bool dereference = (scx->scx_use->cu_def->cd_flags & CDDEREFERENCE) ?
+		TRUE : FALSE;
+	if (!DBCellRead(scx->scx_use->cu_def, (char *) NULL, TRUE, dereference, NULL))
 	    return 0;
-    
-    /* In order to make this work with zero-size areas, we first expand
-     * the area by before searching the tile plane.  DbCellSrFunc will
-     * check carefully to throw out things that don't overlap the original
-     * area.  The expansion is tricky because we mustn't expand infinities.
-     */
+    }
 
-    expanded = scx->scx_area;
-    if (expanded.r_xbot > TiPlaneRect.r_xbot) expanded.r_xbot -= 1;
-    if (expanded.r_ybot > TiPlaneRect.r_ybot) expanded.r_ybot -= 1;
-    if (expanded.r_xtop < TiPlaneRect.r_xtop) expanded.r_xtop += 1;
-    if (expanded.r_ytop < TiPlaneRect.r_ytop) expanded.r_ytop += 1;
-
-    if (TiSrArea((Tile *) NULL, scx->scx_use->cu_def->cd_planes[PL_CELL],
-		&expanded, dbCellSrFunc, (ClientData) &context))
+    if (DBSrCellPlaneArea(scx->scx_use->cu_def->cd_cellPlane,
+		&scx->scx_area, dbCellSrFunc, (ClientData) &context))
 	return 1;
-    else return 0;
+    return 0;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
  * dbCellSrFunc --
  *
  * Filter procedure for DBCellSrArea.  Applies the procedure given
- * to DBCellSrArea to any of the CellUses in the tile that are
- * enumerable.
- *
- * Since subcells are allowed to overlap, a single tile body may
- * refer to many subcells and a single subcell may be referred to
- * by many tile bodies.  To insure that each CellUse is enumerated
- * exactly once, the procedure given to DBCellSrArea is only applied
- * to a CellUse when its lower right corner is contained in the
- * tile to dbCellSrFunc (or otherwise at the last tile encountered
- * in the event the lower right corner of the CellUse is outside the
- * search rectangle).
+ * to DBCellSrArea to any of the CellUses in the given area.
  *
  * Results:
  *	0 is normally returned, and 1 is returned if an abort occurred.
@@ -1089,99 +1140,68 @@ DBCellSrArea(scx, func, cdarg)
  */
 
 int
-dbCellSrFunc(tile, cxp)
-    Tile *tile;
+dbCellSrFunc(use, cxp)
+    CellUse *use;
     TreeContext *cxp;
 {
     TreeFilter *fp = cxp->tc_filter;
     SearchContext *scx = cxp->tc_scx;
-    CellUse *use;
     Rect *bbox;
-    CellTileBody *body;
     SearchContext newScx;
     Transform t, tinv;
-    Rect tileArea;
-    int srchBot, srchRight;
     int xlo, xhi, ylo, yhi, xbase, ybase, xsep, ysep, clientResult;
 
-    srchBot = scx->scx_area.r_ybot;
-    srchRight = scx->scx_area.r_xtop;
-    TITORECT(tile, &tileArea);
+    bbox = &use->cu_bbox;
+    newScx.scx_use = use;
 
-    /* Make sure that this tile really does overlap the search area
-     * (it could be just touching because of the expand-by-one in
-     * DBCellSrArea).
-     */
-    
-    if (!GEO_OVERLAP(&tileArea, &scx->scx_area)) return 0;
-
-    for (body = (CellTileBody *) TiGetBody(tile);
-	    body != NULL;
-	    body = body->ctb_next)
+    /* If not an array element, life is much simpler */
+    if (use->cu_xlo == use->cu_xhi && use->cu_ylo == use->cu_yhi)
     {
-	use = newScx.scx_use = body->ctb_use;
-	ASSERT(use != (CellUse *) NULL, "dbCellSrFunc");
-
-	/* The check below is to ensure that we only enumerate each
-	 * cell once, even though it appears in many different tiles
-	 * in the subcell plane.
-	 */
-
-	bbox = &use->cu_bbox;
-	if (   (tileArea.r_ybot <= bbox->r_ybot ||
-		(tileArea.r_ybot <= srchBot && bbox->r_ybot < srchBot))
-	    && (tileArea.r_xtop >= bbox->r_xtop ||
-		(tileArea.r_xtop >= srchRight && bbox->r_xtop >= srchRight)))
-	{
-	    /* If not an array element, life is much simpler */
-	    if (use->cu_xlo == use->cu_xhi && use->cu_ylo == use->cu_yhi)
-	    {
-		newScx.scx_x = use->cu_xlo, newScx.scx_y = use->cu_yhi;
-		if (SigInterruptPending) return 1;
-		GEOINVERTTRANS(&use->cu_transform, &tinv);
-		GeoTransTrans(&use->cu_transform, &scx->scx_trans,
+	newScx.scx_x = use->cu_xlo, newScx.scx_y = use->cu_yhi;
+	if (SigInterruptPending) return 1;
+	GEOINVERTTRANS(&use->cu_transform, &tinv);
+	GeoTransTrans(&use->cu_transform, &scx->scx_trans,
 				&newScx.scx_trans);
-		GEOTRANSRECT(&tinv, &scx->scx_area, &newScx.scx_area);
-		if ((*fp->tf_func)(&newScx, fp->tf_arg) == 1)
-		    return 1;
-		continue;
-	    }
-
-	    /*
-	     * More than a single array element;
-	     * check to see which ones overlap our search area.
-	     */
-	    DBArrayOverlap(use, &scx->scx_area, &xlo, &xhi, &ylo, &yhi);
-	    xsep = (use->cu_xlo > use->cu_xhi) ? -use->cu_xsep : use->cu_xsep;
-	    ysep = (use->cu_ylo > use->cu_yhi) ? -use->cu_ysep : use->cu_ysep;
-	    for (newScx.scx_y = ylo; newScx.scx_y <= yhi; newScx.scx_y++)
-		for (newScx.scx_x = xlo; newScx.scx_x <= xhi; newScx.scx_x++)
-		{
-		    if (SigInterruptPending) return 1;
-		    xbase = xsep * (newScx.scx_x - use->cu_xlo);
-		    ybase = ysep * (newScx.scx_y - use->cu_ylo);
-		    GeoTransTranslate(xbase, ybase, &use->cu_transform, &t);
-		    GEOINVERTTRANS(&t, &tinv);
-		    GeoTransTrans(&t, &scx->scx_trans, &newScx.scx_trans);
-		    GEOTRANSRECT(&tinv, &scx->scx_area, &newScx.scx_area);
-		    clientResult = (*fp->tf_func)(&newScx, fp->tf_arg);
-		    if (clientResult == 2) goto skipArray;
-		    else if (clientResult == 1) return 1;
-		}
-	}
-	skipArray: continue;
+	GEOTRANSRECT(&tinv, &scx->scx_area, &newScx.scx_area);
+	if ((*fp->tf_func)(&newScx, fp->tf_arg) == 1)
+	    return 1;
+	return 0;
     }
+
+    /*
+     * More than a single array element;
+     * check to see which ones overlap our search area.
+     */
+    DBArrayOverlap(use, &scx->scx_area, &xlo, &xhi, &ylo, &yhi);
+    xsep = (use->cu_xlo > use->cu_xhi) ? -use->cu_xsep : use->cu_xsep;
+    ysep = (use->cu_ylo > use->cu_yhi) ? -use->cu_ysep : use->cu_ysep;
+    for (newScx.scx_y = ylo; newScx.scx_y <= yhi; newScx.scx_y++)
+        for (newScx.scx_x = xlo; newScx.scx_x <= xhi; newScx.scx_x++)
+	{
+	    if (SigInterruptPending) return 1;
+	    xbase = xsep * (newScx.scx_x - use->cu_xlo);
+	    ybase = ysep * (newScx.scx_y - use->cu_ylo);
+	    GeoTransTranslate(xbase, ybase, &use->cu_transform, &t);
+	    GEOINVERTTRANS(&t, &tinv);
+	    GeoTransTrans(&t, &scx->scx_trans, &newScx.scx_trans);
+	    GEOTRANSRECT(&tinv, &scx->scx_area, &newScx.scx_area);
+	    clientResult = (*fp->tf_func)(&newScx, fp->tf_arg);
+	    if (clientResult == 2) return 0;
+	    else if (clientResult == 1) return 1;
+	}
+
     return 0;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
  * DBCellEnum --
  *
- * Apply the supplied procedure once to each CellUse in the subcell tile
+ * Apply the supplied procedure once to each CellUse in the subcell
  * plane of the supplied CellDef.  This procedure is not a geometric
- * search, but rather a hierarchical enumeration.
+ * search, but rather a hierarchical enumeration.  Use DBSrCellPlaneArea()
+ * for geometric searches over an area.
  *
  * The procedure should be of the following form:
  *	int
@@ -1215,24 +1235,23 @@ DBCellEnum(cellDef, func, cdarg)
     filter.tf_func = func;
     filter.tf_arg = cdarg;
     if ((cellDef->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(cellDef, (char *) NULL, TRUE, NULL)) return 0;
-    if (TiSrArea((Tile *) NULL, cellDef->cd_planes[PL_CELL],
+    {
+	bool dereference = (cellDef->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+	if (!DBCellRead(cellDef, (char *) NULL, TRUE, dereference, NULL)) return 0;
+    }
+    if (DBSrCellPlaneArea(cellDef->cd_cellPlane,
 		&TiPlaneRect, dbEnumFunc, (ClientData) &filter))
 	return 1;
     else return 0;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
  * dbEnumFunc --
  *
  * Filter procedure for DBCellEnum.  Applies the procedure given
- * to DBCellEnum to any of the CellUses in the tile that are
- * enumerable.
- *
- * The scheme used for handling overlapping subcells is the same
- * as used in DBCellSrArea above.
+ * to DBCellEnum to the visited CellUse.
  *
  * Results:
  *	0 normally, 1 if abort occurred.
@@ -1245,28 +1264,17 @@ DBCellEnum(cellDef, func, cdarg)
  */
 
 int
-dbEnumFunc(tile, fp)
-    Tile *tile;
+dbEnumFunc(use, fp)
+    CellUse *use;
     TreeFilter *fp;
 {
-    CellUse *use;
-    CellTileBody *body;
     Rect *bbox;
 
-    for (body = (CellTileBody *) TiGetBody(tile);
-	    body != NULL;
-	    body = body->ctb_next)
-    {
-	use = body->ctb_use;
-	ASSERT(use != (CellUse *) NULL, "dbCellSrFunc");
-
-	bbox = &use->cu_bbox;
-	if ((BOTTOM(tile) <= bbox->r_ybot) && (RIGHT(tile) >= bbox->r_xtop))
-	    if ((*fp->tf_func)(use, fp->tf_arg)) return 1;
-    }
+    bbox = &use->cu_bbox;
+    if ((*fp->tf_func)(use, fp->tf_arg)) return 1;
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1355,6 +1363,42 @@ typedef struct LCU1		/* A linked celluse record */
     struct LCU1 *cu_next;	/* Pointer to another linked celluse record */
 } LinkedCellUse;
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBMovePoint ---
+ *
+ *    Move a point by a position (origx, origy).  Ignore positions which
+ *    are marked as (M)INFINITY.
+ *
+ * Results:
+ *	TRUE if the point was moved (point position was not infinite)
+ *
+ * Side effects:
+ *	Point structure pointed to by the first argument is repositioned.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBMovePoint(p, origx, origy)
+    Point *p;
+    int origx, origy;
+{
+    int result = FALSE;
+    if ((p->p_x < (INFINITY - 2)) && (p->p_x > (MINFINITY + 2)))
+    {
+	p->p_x -= origx;
+	result = TRUE;
+    }
+    if ((p->p_y < (INFINITY + 2)) && (p->p_y > (MINFINITY + 2)))
+    {
+	p->p_y -= origy;
+	result = TRUE;
+    }
+    return result;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1454,7 +1498,6 @@ DBScaleEverything(scalen, scaled)
     int scalen, scaled;
 {
     void ToolScaleBox();
-    void DBWScaleCrosshair();
 
     int dbCellDefEnumFunc();
     LinkedCellDef *lhead, *lcd;
@@ -1464,7 +1507,7 @@ DBScaleEverything(scalen, scaled)
     SigDisableInterrupts();
 
     lhead = NULL;
-    (void) DBCellSrDefs(0, dbCellDefEnumFunc, (ClientData) &lhead);    
+    (void) DBCellSrDefs(0, dbCellDefEnumFunc, (ClientData) &lhead);
 
     /* Apply scaling function to each CellDef */
 
@@ -1484,6 +1527,9 @@ DBScaleEverything(scalen, scaled)
 	lcd = lcd->cd_next;
     }
 
+    /* Scale all elements */
+    DBWScaleElements(scalen, scaled);
+
     /* Recovery of global plane pointers */
     MZAttachHintPlanes();
 
@@ -1507,6 +1553,14 @@ struct scaleArg {
    bool modified;
 };
 
+struct moveArg {
+   int origx;
+   int origy;
+   int pnum;
+   Plane *ptarget;
+   bool modified;
+};
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1588,16 +1642,84 @@ dbTileScaleFunc(tile, scvals)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbMovePlane --
+ *
+ *   Relocation procedure called on a single plane.  Copies paint into the
+ *   new plane at a delta position (-origx, -origy)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+dbMovePlane(oldplane, newplane, pnum, origx, origy)
+    Plane *oldplane, *newplane;
+    int pnum;
+    int origx, origy;
+{
+    int dbTileMoveFunc();		/* forward declaration */
+    struct moveArg arg;
+
+    arg.origx = origx;
+    arg.origy = origy;
+    arg.ptarget = newplane;
+    arg.pnum = pnum;
+    arg.modified = FALSE;
+    (void) DBSrPaintArea((Tile *) NULL, oldplane, &TiPlaneRect,
+		&DBAllButSpaceBits, dbTileMoveFunc, (ClientData) &arg);
+
+    return arg.modified;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbTileMoveFunc --
+ *
+ *   Repositioning procedure called on each tile being copied from one plane
+ *   to another.  Delta position (-origx, -origy) is stored inside the struct
+ *   moveArg before calling.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+dbTileMoveFunc(tile, mvvals)
+    Tile *tile;
+    struct moveArg *mvvals;
+{
+    TileType type;
+    Rect targetRect;
+    TileType exact;
+
+    TiToRect(tile, &targetRect);
+
+    mvvals->modified = TRUE;
+    DBMovePoint(&targetRect.r_ll, mvvals->origx, mvvals->origy);
+    DBMovePoint(&targetRect.r_ur, mvvals->origx, mvvals->origy);
+
+    type = TiGetTypeExact(tile);
+    exact = type;
+    if (IsSplit(tile))
+	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
+    DBNMPaintPlane(mvvals->ptarget, exact, &targetRect,
+		DBStdPaintTbl(type, mvvals->pnum),
+		(PaintUndoInfo *)NULL);
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
  * DBSrCellUses --
  *
  *   Do function "func" for each cell use in cellDef, passing "arg" as
- *   client data.  Unlike DBEnumCell, this routine first collects a linked
- *   list of cell uses, then performs the function on the list, so that
- *   the search cannot be corrupted by, removing or reallocating the use
- *   structure from the cell def.  Function "func" takes 2 arguments:
+ *   client data.  This routine first collects a linked list of cell uses,
+ *   then performs the function on the list, so that the search cannot be
+ *   corrupted by (specifically) removing the use structure from the cell
+ *   def's bplane.  Function "func" takes 2 arguments:
  *
  *	int func(Celluse *use, ClientData arg) {}
  *
@@ -1654,6 +1776,100 @@ DBSrCellUses(cellDef, func, arg)
     return retval;
 }
 
+/* Structure used by dbScaleProp() and dbMoveProp() */
+typedef struct _cellpropstruct {
+    Point cps_point;
+    CellDef *cps_def;
+} CellPropStruct;
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbScaleProp --
+ *
+ *  Callback function for dbScaleCell.  Finds properties that represent
+ *  internal geometry (FIXED_BBOX and MASKHINTS_*) and scale the values
+ *  by the numerator / denominator values passed as a pointer to a Point
+ *  structure, where p_x is the numerator value and p_y is the denominator
+ *  value.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int dbScaleProp(name, value, cps)
+    char *name;
+    char *value;
+    CellPropStruct *cps;
+{
+    int scalen, scaled;
+    char *newvalue;
+    Rect r;
+
+    if (!strcmp(name, "FIXED_BBOX") || !strncmp(name, "MASKHINTS_", 10))
+    {
+	if (sscanf(value, "%d %d %d %d", &r.r_xbot, &r.r_ybot,
+			&r.r_xtop, &r.r_ytop) == 4)
+	{
+	    /* Scale numerator held in point X value, */
+	    /* scale denominator held in point Y value */
+
+	    scalen = cps->cps_point.p_x;
+	    scaled = cps->cps_point.p_y;
+
+	    DBScalePoint(&r.r_ll, scalen, scaled);
+	    DBScalePoint(&r.r_ur, scalen, scaled);
+
+	    newvalue = (char *)mallocMagic(40);
+	    sprintf(newvalue, "%d %d %d %d", r.r_xbot, r.r_ybot,
+			r.r_xtop, r.r_ytop);
+	    DBPropPut(cps->cps_def, name, newvalue);
+	}
+    }
+    return 0;	/* Keep enumerating through properties */
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbMoveProp --
+ *
+ *  Callback function for ??.  Finds properties that represent
+ *  internal geometry (FIXED_BBOX and MASKHINTS_*) and modifies the values
+ *  by the X, Y values passed as a pointer to a Point structure in ClientData.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int dbMoveProp(name, value, cps)
+    char *name;
+    char *value;
+    CellPropStruct *cps;
+{
+    int origx, origy;
+    char *newvalue;
+    Rect r;
+
+    if (!strcmp(name, "FIXED_BBOX") || !strncmp(name, "MASKHINTS_", 10))
+    {
+	if (sscanf(value, "%d %d %d %d", &r.r_xbot, &r.r_ybot,
+			&r.r_xtop, &r.r_ytop) == 4)
+	{
+	    origx = cps->cps_point.p_x;
+	    origy = cps->cps_point.p_y;
+
+	    DBMovePoint(&r.r_ll, origx, origy);
+	    DBMovePoint(&r.r_ur, origx, origy);
+
+	    newvalue = (char *)mallocMagic(40);
+	    sprintf(newvalue, "%d %d %d %d", r.r_xbot, r.r_ybot,
+			r.r_xtop, r.r_ytop);
+	    DBPropPut(cps->cps_def, name, newvalue);
+	}
+    }
+    return 0;	/* Keep enumerating through properties */
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1671,12 +1887,14 @@ dbScaleCell(cellDef, scalen, scaled)
 			 */
     int scalen, scaled; /* scale numerator and denominator. */
 {
-    int dbCellTileEnumFunc(), dbCellUseEnumFunc();
+    int dbCellScaleFunc(), dbCellUseEnumFunc();
     Label *lab;
     int pNum;
     LinkedTile *lhead, *lt;
     LinkedCellUse *luhead, *lu;
     Plane *newplane;
+    BPlane *cellPlane, *cellPlaneOrig;
+    CellPropStruct cps;
 
     /* DBCellEnum() attempts to read unavailable celldefs.  We don't	*/
     /* want to do that here, so check CDAVAILABLE flag first.	  	*/
@@ -1693,6 +1911,7 @@ dbScaleCell(cellDef, scalen, scaled)
 
     (void) DBCellEnum(cellDef, dbCellUseEnumFunc, (ClientData) &luhead);
 
+    cellPlane = BPNew();
     lu = luhead;
     while (lu != NULL)
     {
@@ -1720,9 +1939,16 @@ dbScaleCell(cellDef, scalen, scaled)
 	DBScaleValue(&use->cu_array.ar_xsep, scalen, scaled);
 	DBScaleValue(&use->cu_array.ar_ysep, scalen, scaled);
 
+	BPAdd(cellPlane, use);
+
 	lu = lu->cu_next;
     }
 
+    /* Swap the CellDef's cell plane */
+    cellPlaneOrig = cellDef->cd_cellPlane;
+    cellDef->cd_cellPlane = cellPlane;
+    BPFree(cellPlaneOrig);
+
     /* Free this linked cellUse structure */
     lu = luhead;
     while (lu != NULL)
@@ -1731,32 +1957,6 @@ dbScaleCell(cellDef, scalen, scaled)
 	lu = lu->cu_next;
     }
 
-    /* Scale the position of all subcell uses.  Count all of the tiles in the	*/
-    /* subcell plane, and scale those without reference to the actual cells (so	*/
-    /* we don't count the cells multiple times).				*/
-
-    lhead = NULL;
-    (void) TiSrArea((Tile *)NULL, cellDef->cd_planes[PL_CELL], &TiPlaneRect,
-		dbCellTileEnumFunc, (ClientData) &lhead);
-
-    /* Scale each of the tiles in the linked list by (n / d)	*/
-    /* Don't scale (M)INFINITY on boundary tiles!		*/
-
-    lt = lhead;
-    while (lt != NULL)
-    {
-	DBScalePoint(&lt->tile->ti_ll, scalen, scaled);
-	lt = lt->t_next;
-    }
-
-    /* Free this linked tile structure */
-    lt = lhead;
-    while (lt != NULL)
-    {
-	freeMagic((char *)lt);
-	lt = lt->t_next;
-    }
-
     /* Scale all of the paint tiles in this cell by creating a new plane */
     /* and copying all tiles into the new plane at scaled dimensions.	 */
 
@@ -1773,9 +1973,6 @@ dbScaleCell(cellDef, scalen, scaled)
         cellDef->cd_planes[pNum] = newplane;
     }
 
-    /* Check consistency of several global pointers	*/
-    /* WARNING:  This list may not be complete!		*/
-
     /* Also scale the position of all labels. */
     /* If labels are the rendered-font type, scale the size as well */
 
@@ -1794,7 +1991,7 @@ dbScaleCell(cellDef, scalen, scaled)
 
 		DBScalePoint(&lab->lab_bbox.r_ll, scalen, scaled);
 		DBScalePoint(&lab->lab_bbox.r_ur, scalen, scaled);
-		
+
 		for (i = 0; i < 4; i++)
 		    DBScalePoint(&lab->lab_corners[i], scalen, scaled);
 	    }
@@ -1835,33 +2032,18 @@ donecell:
 	    }
 	}
     }
-    return 0;
-}
 
-/*
- * ----------------------------------------------------------------------------
- *
- * dbCellTileEnumFunc --
- *
- *   Enumeration procedure called on each tile encountered in the search of
- *   the cell plane.  Adds the tile to a linked list of tiles.
- *
- * ----------------------------------------------------------------------------
- */
+    /* Check all properties for ones with keys beginning with "MASKHINTS_"
+     * or the key "FIXED_BBOX", and scale them by the same amount as all
+     * the geometry.
+     */
 
-int
-dbCellTileEnumFunc(tile, arg)
-    Tile *tile;
-    LinkedTile **arg;
-{
-    LinkedTile *lt;
 
-    lt = (LinkedTile *) mallocMagic(sizeof(LinkedTile));
+    cps.cps_point.p_x = scalen;
+    cps.cps_point.p_y = scaled;
+    cps.cps_def = cellDef;
+    DBPropEnum(cellDef, dbScaleProp, &cps);
 
-    lt->tile = tile;
-    lt->t_next = (*arg);
-    (*arg) = lt;
- 
     return 0;
 }
 
@@ -1888,7 +2070,7 @@ dbCellDefEnumFunc(cellDef, arg)
     lcd->cellDef = cellDef;
     lcd->cd_next = (*arg);
     (*arg) = lcd;
- 
+
     return 0;
 }
 
@@ -1915,6 +2097,147 @@ dbCellUseEnumFunc(cellUse, arg)
     lcu->cellUse = cellUse;
     lcu->cu_next = (*arg);
     (*arg) = lcu;
- 
+
     return 0;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBMoveCell --
+ *
+ *   Reposition a cell to a different origin.  This routine is equivalent to
+ *   unexpanding all contents of a cell, selecting everything, and issuing a
+ *   move command.  However, for very large layouts this becomes memory- and
+ *   compute- intensive.  The process of reorienting an entire layout to a
+ *   new position can be done much more efficiently.  This routine is
+ *   essentially a copy of dbScaleCell() but changes only position and not
+ *   scale.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+DBMoveCell(cellDef, origx, origy)
+    CellDef *cellDef;	/* Pointer to CellDef to be saved.  This def might
+			 * be an internal buffer; if so, we ignore it.
+			 */
+    int origx, origy; /* Internal unit coordinates which will become the new origin */
+{
+    int dbCellTileEnumFunc(), dbCellUseEnumFunc();
+    Label *lab;
+    int pNum;
+    LinkedTile *lhead, *lt;
+    LinkedCellUse *luhead, *lu;
+    Plane *newplane;
+    BPlane *cellPlane, *cellPlaneOrig;
+    CellPropStruct cps;
+
+    /* Unlike dbScaleCell(), this routine is only run on valid edit defs */
+
+    cellDef->cd_flags |= CDBOXESCHANGED;
+
+    /* Enumerate all unique cell uses, and move their position in the	*/
+    /* bounding box and transform.					*/
+
+    luhead = NULL;
+
+    (void) DBCellEnum(cellDef, dbCellUseEnumFunc, (ClientData) &luhead);
+
+    cellPlane = BPNew();
+    lu = luhead;
+    while (lu != NULL)
+    {
+	CellUse *use;
+	Rect *bbox;
+
+	use = lu->cellUse;
+	bbox = &use->cu_bbox;
+
+	/* TxPrintf("CellUse: BBox is ll (%d, %d), transform [%d %d %d %d %d %d]\n",
+		bbox->r_xbot, bbox->r_ybot,
+		use->cu_transform.t_a, use->cu_transform.t_b, use->cu_transform.t_c,
+		use->cu_transform.t_d, use->cu_transform.t_e, use->cu_transform.t_f); */
+
+	DBMovePoint(&bbox->r_ll, origx, origy);
+	DBMovePoint(&bbox->r_ur, origx, origy);
+
+	bbox = &use->cu_extended;
+	DBMovePoint(&bbox->r_ll, origx, origy);
+	DBMovePoint(&bbox->r_ur, origx, origy);
+
+	use->cu_transform.t_c -= origx;
+	use->cu_transform.t_f -= origy;
+
+	BPAdd(cellPlane, use);
+
+	lu = lu->cu_next;
+    }
+
+    /* Swap the CellDef's cell plane */
+    cellPlaneOrig = cellDef->cd_cellPlane;
+    cellDef->cd_cellPlane = cellPlane;
+    BPFree(cellPlaneOrig);
+
+    /* Free this linked cellUse structure */
+    lu = luhead;
+    while (lu != NULL)
+    {
+	freeMagic((char *)lu);
+	lu = lu->cu_next;
+    }
+
+    /* Move all of the paint tiles in this cell by creating a new plane */
+    /* and copying all tiles into the new plane at the new position.	 */
+
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    {
+	if (cellDef->cd_planes[pNum] == NULL) continue;
+	newplane = DBNewPlane((ClientData) TT_SPACE);
+	DBClearPaintPlane(newplane);
+	if (dbMovePlane(cellDef->cd_planes[pNum], newplane, pNum,
+		origx, origy, FALSE))
+	    cellDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+        DBFreePaintPlane(cellDef->cd_planes[pNum]);
+	TiFreePlane(cellDef->cd_planes[pNum]);
+        cellDef->cd_planes[pNum] = newplane;
+    }
+
+    /* Also move the position of all labels. */
+
+    if (cellDef->cd_labels)
+    {
+	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
+	{
+	    DBMovePoint(&lab->lab_rect.r_ll, origx, origy);
+	    DBMovePoint(&lab->lab_rect.r_ur, origx, origy);
+
+	    if (lab->lab_font >= 0)
+	    {
+		DBMovePoint(&lab->lab_bbox.r_ll, origx, origy);
+		DBMovePoint(&lab->lab_bbox.r_ur, origx, origy);
+	    }
+	}
+    }
+
+donecell:
+
+    /* The cellDef bounding box gets moved to match the new position. */
+
+    DBMovePoint(&cellDef->cd_bbox.r_ll, origx, origy);
+    DBMovePoint(&cellDef->cd_bbox.r_ur, origx, origy);
+    DBMovePoint(&cellDef->cd_extended.r_ll, origx, origy);
+    DBMovePoint(&cellDef->cd_extended.r_ur, origx, origy);
+
+    /* Check all properties for ones with keys beginning with "MASKHINTS_"
+     * or the key "FIXED_BBOX", and move them by the same amount as all
+     * the geometry.
+     */
+
+    cps.cps_point.p_x = origx;
+    cps.cps_point.p_y = origy;
+    cps.cps_def = cellDef;
+    DBPropEnum(cellDef, dbMoveProp, &cps);
+    return 0;
+}
+
