@@ -6,16 +6,16 @@
  *	and one	that operates hierarchically, across cell boundaries
  *	(DBTreeCopyConnect).
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -47,7 +47,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  * of information that must be passed through multiple
  * levels of search function (used by dbSrConnectFunc).
  */
-	
+
 struct conSrArg
 {
     CellDef *csa_def;			/* Definition being searched. */
@@ -67,7 +67,7 @@ struct conSrArg
  * of information that must be passed through multiple
  * levels of search function (used by dbcConnectFunc).
  */
-	
+
 typedef struct
 {
     Rect		area;		/* Area to process */
@@ -117,7 +117,7 @@ DBTransformDiagonal(oldtype, trans)
 {
     TileType dinfo;
     int o1, o2, o3, dir, side;
-	
+
     o1 = ((trans->t_e > 0) || (trans->t_d > 0)) ? 1 : 0;
     o2 = ((trans->t_a > 0) || (trans->t_b > 0)) ? 1 : 0;
     o3 = (trans->t_a != 0) ? 1 : 0;
@@ -169,7 +169,7 @@ DBInvTransformDiagonal(oldtype, trans)
     return dinfo;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -201,7 +201,7 @@ DBInvTransformDiagonal(oldtype, trans)
  *	1 then the search is aborted.
  *
  *				*** WARNING ***
- *	
+ *
  *	Func should not modify any paint during the search, since this
  *	will mess up pointers kept by these procedures and likely cause
  *	a core-dump.
@@ -262,7 +262,7 @@ DBSrConnect(def, startArea, mask, connect, bounds, func, clientData)
     if (startTile == NULL) return 0;
     /* The following lets us call DBSrConnect recursively */
     else if (startTile->ti_client == (ClientData)1) return 0;
-   
+
 
     /* Pass 1.  During this pass the client function gets called. */
 
@@ -353,7 +353,7 @@ DBSrConnectOnePass(def, startArea, mask, connect, bounds, func, clientData)
     if (startTile == NULL) return 0;
     /* The following lets us call DBSrConnect recursively */
     else if (startTile->ti_client == (ClientData)1) return 0;
-   
+
 
     /* Pass 1.  During this pass the client function gets called. */
 
@@ -367,7 +367,7 @@ DBSrConnectOnePass(def, startArea, mask, connect, bounds, func, clientData)
     return result;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -442,7 +442,7 @@ dbSrConnectFunc(tile, csa)
      * connected tiles.  For each one found, call ourselves
      * recursively.
      */
-    
+
     if (IsSplit(tile))
     {
 	if (SplitSide(tile))
@@ -575,7 +575,7 @@ donesides:
     /* Lastly, check to see if this tile connects to anything on
      * other planes.  If so, search those planes.
      */
-    
+
     planes = DBConnPlanes[loctype];
     planes &= ~(PlaneNumToMaskBit(csa->csa_plane));
     if (planes != 0)
@@ -606,7 +606,7 @@ donesides:
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -632,11 +632,11 @@ int
 dbcUnconnectFunc(tile, clientData)
     Tile *tile;				/* Current tile	*/
     ClientData clientData;		/* Unused.	*/
-    
+
 {
     return 1;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -647,6 +647,11 @@ dbcUnconnectFunc(tile, clientData)
  *	when a label is found which is connected to paint belonging to the
  *	network, it adds it to the destination definition.
  *
+ *	In addition to simply adding a label to the destination, this
+ *	routine also checks port connections on abstract cells.  Abstract
+ *      cells can have multiple ports of the same name with implied
+ *	connectivity between them.
+ *
  * Results:
  *	Always 0.
  *
@@ -667,6 +672,8 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
     Rect r;
     Point offset;
     int pos, rotate;
+    char newlabtext[FLATTERMSIZE];
+    char *newlabptr;
     int dbcConnectFunc();		/* Forward declaration */
 
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &r);
@@ -674,15 +681,37 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
     GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &offset);
     rotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
-    /* Only add labels if they are on the search top level  */
-    /* (NOTE:  Could add hierachical labels using tpath)    */
+    /* Do not add any labels to the destination unless they are on  */
+    /* the top level (Note:  Could alter label to be placed with    */
+    /* tpath).							    */
 
-    if (scx->scx_use == csa2->csa2_topscx->scx_use)
+    if (scx->scx_use != csa2->csa2_topscx->scx_use)
     {
-	DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
-	DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
-		pos, lab->lab_text, lab->lab_type, lab->lab_flags);
+	if (tpath)
+	{
+	    int newllen = tpath->tp_next - tpath->tp_first;
+	    newlabtext[0] = '\0';
+	    if (newllen > 0)
+		strncpy(newlabtext, tpath->tp_first, newllen);
+	    sprintf(newlabtext + newllen, "%s", lab->lab_text);
+	    newlabptr = newlabtext;
+	}
+	else return 0;
     }
+    else
+	newlabptr = lab->lab_text;
+
+    /* Do not repeat a label copy;  check that the label doesn't	*/
+    /* already exist in the destination def first.			*/
+    if (DBCheckLabelsByContent(def, &r, lab->lab_type, lab->lab_text))
+	return 0;
+
+    if (DBCheckLabelsByContent(def, &r, lab->lab_type, newlabptr))
+	return 0;
+
+    DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
+    DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
+		pos, newlabptr, lab->lab_type, lab->lab_flags);
 
     if (lab->lab_flags & PORT_DIR_MASK)
     {
@@ -721,7 +750,7 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
 		    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 				&newarea, connectMask, dbcUnconnectFunc,
 				(ClientData) NULL) == 1)
-		        continue;
+			continue;
 
 		    newarea.r_xbot--;
 		    newarea.r_xtop++;
@@ -732,20 +761,20 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
 
 		    if (++csa2->csa2_top == csa2->csa2_size)
 		    {
-		        /* Reached list size limit---need to enlarge the list      */
-		        /* Double the size of the list every time we hit the limit */
+			/* Reached list size limit---need to enlarge the list	   */
+			/* Double the size of the list every time we hit the limit */
 
-		        conSrArea *newlist;
-		        int i, lastsize = csa2->csa2_size;
+			conSrArea *newlist;
+			int i, lastsize = csa2->csa2_size;
 
-		        csa2->csa2_size *= 2;
+			csa2->csa2_size *= 2;
 
-		        newlist = (conSrArea *)mallocMagic(csa2->csa2_size
+			newlist = (conSrArea *)mallocMagic(csa2->csa2_size
 					* sizeof(conSrArea));
-		        memcpy((void *)newlist, (void *)csa2->csa2_list,
+			memcpy((void *)newlist, (void *)csa2->csa2_list,
 					(size_t)lastsize * sizeof(conSrArea));
-		        freeMagic((char *)csa2->csa2_list);
-		        csa2->csa2_list = newlist;
+			freeMagic((char *)csa2->csa2_list);
+			csa2->csa2_list = newlist;
 		    }
 
 		    csa2->csa2_list[csa2->csa2_top].area = newarea;
@@ -758,7 +787,7 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
     }
     return 0;
 }
-			
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -801,7 +830,6 @@ dbcConnectFunc(tile, cx)
     TileType loctype = TiGetTypeExact(tile);
     TileType dinfo = 0;
     int pNum = cx->tc_plane;
-    unsigned char searchtype;
     CellDef *def;
 
     TiToRect(tile, &tileArea);
@@ -825,7 +853,7 @@ dbcConnectFunc(tile, cx)
     /* Clip the current area down to something that overlaps the
      * area of interest.
      */
-	
+
     csa2 = (struct conSrArg2 *)cx->tc_filter->tf_arg;
     GeoClip(&newarea, csa2->csa2_bounds);
     if (GEO_RECTNULL(&newarea)) return 0;
@@ -841,7 +869,7 @@ dbcConnectFunc(tile, cx)
      * If so, then there's no need to process the current area,
      * since any processing that is needed was already done before.
      */
-	
+
     connectMask = &csa2->csa2_connect[loctype];
 
     /* In the case of contact bits, the types underneath
@@ -893,7 +921,7 @@ dbcConnectFunc(tile, cx)
      */
 
     /* Only extend those sides bordering the diagonal tile */
-	  
+
     if (dinfo & TT_DIAGONAL)
     {
 	if (dinfo & TT_SIDE)			/* right */
@@ -926,7 +954,7 @@ dbcConnectFunc(tile, cx)
 
 	csa2->csa2_size *= 2;
 
-	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
+	newlist = (conSrArea *)mallocMagic((size_t)(csa2->csa2_size) * sizeof(conSrArea));
 	memcpy((void *)newlist, (void *)csa2->csa2_list,
 			(size_t)lastsize * sizeof(conSrArea));
 	freeMagic((char *)csa2->csa2_list);
@@ -940,7 +968,7 @@ dbcConnectFunc(tile, cx)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -967,7 +995,7 @@ dbcConnectFunc(tile, cx)
  */
 
 void
-DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
+DBTreeCopyConnect(scx, mask, xMask, connect, area, doLabels, destUse)
     SearchContext *scx;			/* Describes starting area.  The
 					 * scx_use field gives the root of
 					 * the hierarchy to search, and the
@@ -991,6 +1019,10 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
 					 * clipped to this area.  Pass
 					 * TiPlaneRect to get everything.
 					 */
+    bool doLabels;			/* If TRUE, copy connected labels
+					 * and paint.  If FALSE, copy only
+					 * connected paint.
+					 */
     CellUse *destUse;			/* Result use in which to place
 					 * anything connected to material of
 					 * type mask in area of rootUse.
@@ -1019,6 +1051,13 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
     DBTreeSrTiles(scx, mask, xMask, dbcConnectFunc, (ClientData) &csa2);
     while (csa2.csa2_top >= 0)
     {
+        char pathstring[FLATTERMSIZE];
+        TerminalPath tpath;
+
+        tpath.tp_first = tpath.tp_next = pathstring;
+        tpath.tp_last = pathstring + FLATTERMSIZE;
+	pathstring[0] = '\0';
+
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
 	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
@@ -1059,7 +1098,8 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
 	        searchtype |= TF_LABEL_ATTACH_NOT_SE;
 	    }
 	}
-	DBTreeSrLabels(scx, newmask, xMask, NULL, searchtype,
+	if (doLabels)
+	    DBTreeSrLabels(scx, newmask, xMask, &tpath, searchtype,
 			dbcConnectLabelFunc, (ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
@@ -1067,6 +1107,6 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
     /* Recompute the bounding box of the destination and record its area
      * for redisplay.
      */
-    
+
     DBReComputeBbox(destUse->cu_def);
 }
