@@ -3,16 +3,16 @@
  *
  * Reading and writing of cells
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -45,6 +45,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include <paths.h>
 #endif
 
+#include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -92,7 +93,7 @@ bool dbReadUse();
 /*
  * ----------------------------------------------------------------------------
  *
- *   file_is_not_writeable -- 
+ *   file_is_not_writeable --
  *
  *  Check to see if file is not writeable. (wen-king@cs.caltech.edu)
  *  Modified to deal with changed semantics of access() (rajit@cs.caltech.edu)
@@ -205,7 +206,7 @@ file_is_not_writeable(name)
  *	the direction of the corner made by the right triangle.
  *	If the split tile contains more than one type, separate entries
  *	are output for each.
- *	
+ *
  *	6. Zero or more groups of lines describing cell uses.  Each group
  *	is of the form
  *		use <filename> <id> [<path>]
@@ -264,7 +265,7 @@ file_is_not_writeable(name)
  */
 
 bool
-dbCellReadDef(f, cellDef, name, ignoreTech)
+dbCellReadDef(f, cellDef, name, ignoreTech, dereference)
     FILE *f;		/* The file, already opened by the caller */
     CellDef *cellDef;	/* Pointer to definition of cell to be read in */
     char *name;		/* Name of file from which to read definition.
@@ -279,6 +280,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 			 * names do not match, but an attempt will be
 			 * made to read the file anyway.
 			 */
+    bool dereference;	/* If TRUE, ignore path references in the input */
 {
     int cellStamp = 0, rectCount = 0, rectReport = 10000;
     char line[2048], tech[50], layername[50];
@@ -344,7 +346,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	}
 	if (dbFgets(line, sizeof line, f) == NULL)
 	    goto badfile;
-	
+
 	if (line[0] == 'm')
 	{
 	    if (!strncmp(line, "magscale", 8))
@@ -355,13 +357,13 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 		    n = d = 1;
 		}
 	    }
-		
+
 	    /* For backward compatibility, accept (and throw away) lines
 	     * whose first word is "maxlabscale".
 	     */
 	    else if (!strncmp(line, "maxlabscale", 11))
 		TxError("Deprecated keyword \"maxlabscale\" in input file.\n");
-	    else 
+	    else
 		TxError("Expected magscale but got: %s", line);
 	    if (dbFgets(line, sizeof line, f) == NULL)
 		goto badfile;
@@ -408,9 +410,9 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
     }
     if (n > 1)
     {
-	TxPrintf("Scaled magic input cell %s geometry by factor of %d", 
+	TxPrintf("Scaled magic input cell %s geometry by factor of %d",
 		cellDef->cd_name, n);
-	if (d > 1) 
+	if (d > 1)
 	{
 	    TxPrintf("/ %d\n", d);
 	    TxError("Warning:  Geometry may be lost because internal grid"
@@ -437,7 +439,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	 */
 	if (sscanf(line, "<< %s >>", layername) != 1)
 	{
-	    if (!dbReadUse(cellDef, line, sizeof line, f, n, d))
+	    if (!dbReadUse(cellDef, line, sizeof line, f, n, d, dereference))
 		goto badfile;
 	    continue;
 	}
@@ -480,7 +482,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	/*
 	 * Record presence of material in cell.
 	 */
-	    
+
 	if (DBPlane(type) > 0)
 	{
 	    if (type < DBNumUserLayers)
@@ -519,7 +521,7 @@ nextrect:
 			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
 			TT_DIRECTION : 0);
 	    }
-            else  
+            else
 	    {
 		dinfo = 0;
 		if (!GetRect(f, 4, rp, n, d)) goto badfile;
@@ -658,7 +660,7 @@ badfile:
  *	If DBbackupFile is non-NULL, then memory is freed, and the backup
  *	file is removed from the filesystem temp directory.  Otherwise,
  *	nothing happens.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -752,16 +754,16 @@ DBFileRecovery(filename)
 				continue;
 
 			recent = sbuf.st_ctime;
-			DBbackupFile = StrDup(&DBbackupFile, tempname);
+			StrDup(&DBbackupFile, tempname);
 		    }
-		}  
-	    } 
+		}
+	    }
 	}
 	closedir(cwd);
     }
     else
     {
-	DBbackupFile = StrDup(&DBbackupFile, filename);
+	StrDup(&DBbackupFile, filename);
 	recent = 1;
     }
 
@@ -858,12 +860,12 @@ DBReadBackup(name)
 
  	    cellDef = DBCellLookDef(rootname);
 	    if (cellDef == (CellDef *)NULL)
-		cellDef = DBCellNewDef(rootname, (char *)NULL);
+		cellDef = DBCellNewDef(rootname);
 
 	    cellDef->cd_flags &= ~CDNOTFOUND;
 	    cellDef->cd_flags |= CDAVAILABLE;
 
-	    if (dbCellReadDef(f, cellDef, filename, TRUE) == FALSE)
+	    if (dbCellReadDef(f, cellDef, filename, TRUE, FALSE) == FALSE)
 		return FALSE;
 
 	    if (dbFgets(line, sizeof(line), f) == NULL)
@@ -873,7 +875,7 @@ DBReadBackup(name)
 		return FALSE;
 	    }
 	    /* Update timestamp flags from dbCellReadDef() */
-	    DBFlagMismatches();
+	    DBFlagMismatches(cellDef);
 	}
 	else
 	{
@@ -919,7 +921,7 @@ DBReadBackup(name)
  */
 
 bool
-DBCellRead(cellDef, name, ignoreTech, errptr)
+DBCellRead(cellDef, name, ignoreTech, dereference, errptr)
     CellDef *cellDef;	/* Pointer to definition of cell to be read in */
     char *name;		/* Name of file from which to read definition.
 			 * If NULL, then use cellDef->cd_file; if that
@@ -933,6 +935,7 @@ DBCellRead(cellDef, name, ignoreTech, errptr)
 			 * names do not match, but an attempt will be
 			 * made to read the file anyway.
 			 */
+    bool dereference;	/* If TRUE then ignore path argument to uses */
     int *errptr;	/* Copy of errno set by file reading routine
 			 * is placed here, unless NULL.
 			 */
@@ -950,7 +953,7 @@ DBCellRead(cellDef, name, ignoreTech, errptr)
 
     else
     {
-	result = (dbCellReadDef(f, cellDef, name, ignoreTech));
+	result = (dbCellReadDef(f, cellDef, name, ignoreTech, dereference));
 
 #ifdef FILE_LOCKS
 	/* Close files that were locked by another user */
@@ -1052,23 +1055,22 @@ dbReadOpen(cellDef, name, setFileName, errptr)
 
 	    if (f != NULL)
 	    {
-		if (pptr != NULL) *pptr = '.';
-
 		/* NOTE:  May not want to present this as an error, as	*/
 		/* it is a common technique to read files from, say, a	*/
 		/* LEF file but not save them locally, and then expect	*/
 		/* that the layout views will be picked up from		*/
 		/* somewhere else in the search paths.			*/
 
+		if (pptr != NULL) *pptr = '.';
 		TxError("Warning:  Parent cell lists instance of \"%s\" at bad file "
 			"path %s.\n", cellDef->cd_name, cellDef->cd_file);
-		TxError("The cell exists in the search paths at %s.\n", filename);
-		TxError("The discovered version will be used.\n");
 
 		/* Write the new path to cd_file or else magic will	*/
 		/* generate another error later.			*/
+		StrDup(&cellDef->cd_file, filename);
 
-		cellDef->cd_file = StrDup(&cellDef->cd_file, filename);
+		TxError("The cell exists in the search paths at %s.\n", filename);
+		TxError("The discovered version will be used.\n");
 	    }
 	}
 
@@ -1097,7 +1099,7 @@ dbReadOpen(cellDef, name, setFileName, errptr)
 	    realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
 			+ strlen(DBSuffix) + 1));
 	    (void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
-	    cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
+	    StrDup(&cellDef->cd_file, realname);
 	}
 	if (errptr) TxError("%s\n", strerror(*errptr));
 
@@ -1131,7 +1133,14 @@ dbReadOpen(cellDef, name, setFileName, errptr)
     cellDef->cd_flags &= ~CDNOTFOUND;
 #endif
     if (setFileName)
+    {
+	/* Remove any ".mag" file extension */
+	char *pptr = strrchr(filename, '.');
+	if (pptr != NULL)
+	    if (!strcmp(pptr, DBSuffix)) *pptr = '\0';
+
 	(void) StrDup(&cellDef->cd_file, filename);
+    }
     cellDef->cd_flags |= CDAVAILABLE;
     return (f);
 }
@@ -1170,7 +1179,7 @@ DBTestOpen(name, fullPath)
     return FALSE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1193,13 +1202,14 @@ DBTestOpen(name, fullPath)
  */
 
 bool
-dbReadUse(cellDef, line, len, f, scalen, scaled)
+dbReadUse(cellDef, line, len, f, scalen, scaled, dereference)
     CellDef *cellDef;	/* Cell whose cells are being read */
     char *line;		/* Line containing "use ..." */
     int len;		/* Size of buffer pointed to by line */
     FILE *f;		/* Input file */
     int scalen;		/* Multiply values in file by this */
     int scaled;		/* Divide values in file by this */
+    bool dereference;	/* If TRUE, ignore path references */
 {
     int xlo, xhi, ylo, yhi, xsep, ysep, childStamp;
     int absa, absb, absd, abse, nconv;
@@ -1230,7 +1240,7 @@ dbReadUse(cellDef, line, len, f, scalen, scaled)
 
     pathptr = &path[0];
     while (*pathptr == ' ' || *pathptr == '\t') pathptr++;
-    if (*pathptr == '\n') *pathptr = '\0';
+    if ((dereference == TRUE) || (*pathptr == '\n')) *pathptr = '\0';
 
     locked = (useid[0] == CULOCKCHAR) ? TRUE : FALSE;
 
@@ -1343,7 +1353,7 @@ badTransform:
     subCellDef = DBCellLookDef(cellname);
     if (subCellDef == (CellDef *) NULL)
     {
-	subCellDef = DBCellNewDef(cellname, (char *)NULL);
+	subCellDef = DBCellNewDef(cellname);
 	subCellDef->cd_timestamp = childStamp;
 
 	/* Make sure rectangle is non-degenerate */
@@ -1364,7 +1374,7 @@ badTransform:
     else if (DBIsAncestor(subCellDef, cellDef))
     {
 	/*
-	 * Watchout for attempts to create circular structures.
+	 * Watch out for attempts to create circular structures.
 	 * If this happens, disregard the subcell.
 	 */
 	TxPrintf("Subcells are used circularly!\n");
@@ -1373,6 +1383,33 @@ badTransform:
 	goto nextLine;
     }
 
+#ifdef MAGIC_WRAPPER
+    /* If path starts with '$' then check for a possible Tcl variable	*/
+    /* replacement.							*/
+
+    if (*pathptr == '$')
+    {
+	char *varstart, *varend, savechar, *tvar;
+
+	varstart = pathptr + 1;
+	if (*varstart == '{') varstart++;
+	varend = varstart + 1;
+	while (*varend != '\0' && *varend != '}' && *varend != '/'
+			&& *varend != '\n' && *varend != ' ') varend++;
+	savechar = *varend;
+	*varend = '\0';
+
+	tvar = (char *)Tcl_GetVar(magicinterp, varstart, TCL_GLOBAL_ONLY);
+	*varend = savechar;
+	if (savechar == '}') varend++;
+	if (tvar)
+	{
+	    memmove(pathptr + strlen(tvar), varend, strlen(varend) + 1);
+	    memmove(pathptr, tvar, strlen(tvar));
+	}
+    }
+#endif
+
     /* Relative path handling:  If path does not have a leading "/"	*/
     /* or "~" and cellDef->cd_file has path components, then the path	*/
     /* should be interpreted relative to the path of the parent cell.	*/
@@ -1437,9 +1474,23 @@ badTransform:
 	    slashptr = strrchr(subCellDef->cd_file, '/');
 	    if (slashptr != NULL)
 	    {
+		bool pathOK = FALSE;
 		*slashptr = '\0';
 
-		if (strcmp(subCellDef->cd_file, pathptr))
+		/* Avoid generating error message if pathptr starts with '~' */
+		/* and the tilde-expanded name matches the subCellDef name   */
+
+		if (*pathptr == '~')
+		{
+		    char *homedir = getenv("HOME");
+		    if (!strncmp(subCellDef->cd_file, homedir, strlen(homedir))
+			    && (!strcmp(subCellDef->cd_file + strlen(homedir),
+			    pathptr + 1)))
+			pathOK = TRUE;
+		}
+
+		if ((pathOK == FALSE) && strcmp(subCellDef->cd_file, pathptr)
+			    && (dereference == FALSE))
 		{
 		    TxError("Duplicate cell in %s:  Instance of cell %s is from "
 				"path %s but cell was previously read from %s.\n",
@@ -1467,7 +1518,7 @@ badTransform:
 	    StrDup(&subCellDef->cd_file, path);
 	}
     }
-    
+
     subCellUse = DBCellNewUse(subCellDef, (useid[0]) ?
 		((locked) ? useid + 1 : useid) : (char *) NULL);
 
@@ -1516,7 +1567,7 @@ nextLine:
     return (dbFgets(line, len, f) != NULL);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1628,7 +1679,7 @@ dbReadProperties(cellDef, line, len, f, scalen, scaled)
 			    locbbox.r_xbot, locbbox.r_ybot,
 			    locbbox.r_xtop, locbbox.r_ytop);
 		    (void) DBPropPut(cellDef, propertyname, storedvalue);
-	    
+
 		}
 	    }
 	    else
@@ -1850,7 +1901,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
     int scalen;		/* Scale up by this factor */
     int scaled;		/* Scale down by this factor */
 {
-    char layername[50], text[1024], port_use[50], port_class[50];
+    char layername[50], text[1024], port_use[50], port_class[50], port_shape[50];
     TileType type;
     int ntok, orient, size, rotate, font, flags;
     Point offset;
@@ -1959,9 +2010,9 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 
 	    if (((lab = cellDef->cd_lastLabel) == NULL) ||
 			(lab->lab_flags & PORT_DIR_MASK) ||
-			(((ntok = sscanf(line, "port %d %4s %49s %49s",
-				&idx, ppos, port_use, port_class)) != 2) &&
-			(ntok != 4)))
+			(((ntok = sscanf(line, "port %d %4s %49s %49s %49s",
+				&idx, ppos, port_use, port_class, port_shape)) != 2) &&
+			(ntok != 4) && (ntok != 5)))
 	    {
 		TxError("Skipping bad \"port\" line: %s", line);
 		goto nextlabel;
@@ -1986,7 +2037,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 			break;
 		}
 	    }
-	    if (ntok == 4)
+	    if (ntok >= 4)
 	    {
 		switch(port_use[0])
 		{
@@ -2037,7 +2088,26 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 			TxError("Ignoring unknown \"port\" use: %s", port_use);
 			break;
 		}
-
+		if (ntok == 5) {
+		    switch(port_shape[0])
+		    {
+			case 'a':
+			    lab->lab_flags |= PORT_SHAPE_ABUT;
+			    break;
+			case 'r':
+			    lab->lab_flags |= PORT_SHAPE_RING;
+			    break;
+			case 'f':
+			    lab->lab_flags |= PORT_SHAPE_THRU;
+			    break;
+			case 'd':
+			    lab->lab_flags |= PORT_SHAPE_DEFAULT;
+			    break;
+			default:
+			    TxError("Ignoring unknown \"port\" shape: %s", port_shape);
+			    break;
+		    }
+		}
 	    }
 	    goto nextlabel;
 	}
@@ -2124,7 +2194,7 @@ nextlabel:
     return (TRUE);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2236,7 +2306,7 @@ DBCellFindScale(cellDef)
     }
 
     /* Finally, cell uses */
-	
+
     if (DBCellEnum(cellDef, dbFindCellGCFFunc, (ClientData) &ggcf))
 	return 1;
 
@@ -2260,7 +2330,7 @@ dbFindGCFFunc(tile, ggcf)
 	*ggcf = FindGCF(r.r_ytop, *ggcf);
     if (r.r_ybot % (*ggcf) != 0)
 	*ggcf = FindGCF(r.r_ybot, *ggcf);
-   
+
     return (*ggcf == 1) ? 1 : 0;
 }
 
@@ -2303,7 +2373,7 @@ dbFindCellGCFFunc(cellUse, ggcf)
 
     return (*ggcf == 1) ? 1 : 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2482,7 +2552,7 @@ DBCellWriteFile(cellDef, f)
 		sprintf(lstring, "port %d %s", lab->lab_flags & PORT_NUM_MASK,
 			ppos);
 
-		if (lab->lab_flags & (PORT_USE_MASK | PORT_CLASS_MASK))
+		if (lab->lab_flags & (PORT_USE_MASK | PORT_CLASS_MASK | PORT_SHAPE_MASK))
 		{
 		    switch (lab->lab_flags & PORT_USE_MASK)
 		    {
@@ -2527,15 +2597,28 @@ DBCellWriteFile(cellDef, f)
 			    strcat(lstring, " default");
 			    break;
 		    }
+
+		    switch (lab->lab_flags & PORT_SHAPE_MASK)
+		    {
+			case PORT_SHAPE_ABUT:
+			    strcat(lstring, " abutment");
+			    break;
+			case PORT_SHAPE_RING:
+			    strcat(lstring, " ring");
+			    break;
+			case PORT_SHAPE_THRU:
+			    strcat(lstring, " feedthrough");
+			    break;
+		    }
 		}
 		strcat(lstring, "\n");
-		FPRINTF(f, lstring); 
+		FPRINTF(f, lstring);
 	    }
 	}
     }
 
     /* Now any persistent elements */
-    estring = DBWPrintElements(cellDef, DBW_ELEMENT_PERSISTENT);
+    estring = DBWPrintElements(cellDef, DBW_ELEMENT_PERSISTENT, reducer);
     if (estring != NULL)
     {
 	FPRINTF(f, "<< elements >>\n");
@@ -2635,7 +2718,7 @@ dbWritePropFunc(key, value, cdata)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2645,7 +2728,7 @@ dbWritePropFunc(key, value, cdata)
  * Mark the cell as having been written out.  Before calling this
  * procedure, the caller should make sure that timestamps have been
  * updated where appropriate.
- * 
+ *
  * This code is fairly tricky to ensure that we never destroy the
  * original contents of a cell in the event of an I/O error.  We
  * try the following approaches in order.
@@ -2686,7 +2769,7 @@ DBCellWrite(cellDef, fileName)
 #define NAME_SIZE	1000
     char *template = ".XXXXXXX";
     char *realname, *tmpname, *expandname;
-    char *cp1, *cp2;
+    char *cp1, *cp2, *dotptr;
     char expandbuf[NAME_SIZE];
     FILE *realf, *tmpf;
     int tmpres;
@@ -2698,29 +2781,34 @@ DBCellWrite(cellDef, fileName)
     /*
      * Figure out the name of the file we will eventually write.
      */
-    if (fileName)
+    if (!fileName)
+    {
+	if (cellDef->cd_file)
+	    fileName = cellDef->cd_file;
+	else if (cellDef->cd_name)
+	    fileName = cellDef->cd_name;
+	else
+	    return FALSE;
+    }
+
+    /* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		    */
+    /* cd_name and cd_file to ALWAYS be the same, otherwise ugly    */
+    /* surprises can occur after saving a file as a different	    */
+    /* filename.						    */
+
+    if (fileName != cellDef->cd_file)
+	StrDup(&cellDef->cd_file, fileName);
+
+    /* The cd_file should not have the .mag suffix, but make sure   */
+    /* it doesn't before adding one.				    */
+
+    if ((strlen(fileName) < 4) || (strcmp(fileName + strlen(fileName) - 4, DBSuffix)))
     {
 	realname = (char *) mallocMagic(strlen(fileName) + strlen(DBSuffix) + 1);
 	(void) sprintf(realname, "%s%s", fileName, DBSuffix);
-
-	/* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		*/
-	/* cd_name and cd_file to ALWAYS be the same, otherwise ugly	*/
-	/* surprises can occur after saving a file as a different	*/
-	/* filename.							*/
-
-	cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
     }
-    else if (cellDef->cd_file)
-    {
-	realname = StrDup((char **) NULL, cellDef->cd_file);
-    }
-    else if (cellDef->cd_name)
-    {
-	realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
-		+ strlen(DBSuffix) + 1));
-	(void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
-    }
-    else return (FALSE);
+    else
+	realname = StrDup((char **)NULL, fileName);
 
     /*
      * Expand the filename, removing the leading ~, if any.
@@ -2818,7 +2906,7 @@ DBCellWrite(cellDef, fileName)
 	    close(cellDef->cd_fd);
 	    cellDef->cd_fd = -1;
 	}
-#endif  
+#endif
 
 	/*
 	 * The temp file is in good shape -- rename it to the real name,
@@ -2839,9 +2927,12 @@ DBCellWrite(cellDef, fileName)
 
 #ifdef FILE_LOCKS
 	else
+	{
+	    bool dereference = (cellDef->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
 	    /* Re-aquire the lock on the new file by opening it. */
-	    DBCellRead(cellDef, NULL, TRUE, NULL);
-#endif  
+	    DBCellRead(cellDef, NULL, TRUE, dereference, NULL);
+	}
+#endif
 
     }
     else if (exists)
@@ -2905,17 +2996,25 @@ DBCellWrite(cellDef, fileName)
 	}
     }
 
+    /* Copy expandname back to cellDef->cd_file, if the name was changed.   */
+    /* The file extension does not get copied into cd_file.		    */
+
+    dotptr = strrchr(expandname, '.');
+    if (dotptr) *dotptr = '\0';
+    if (strcmp(expandname, cellDef->cd_file))
+	StrDup(&cellDef->cd_file, expandname);
+    if (dotptr) *dotptr = '.';
+
     /* Everything worked so far. */
 
-    (void) StrDup(&cellDef->cd_file, expandname);
     result = TRUE;
     {
 	struct stat thestat;
-	realf = fopen(expandname,"r");
+	realf = fopen(expandname, "r");
 	if (realf == NULL)
 	{
 	    cellDef->cd_flags |= CDMODIFIED;
-	    TxError("Warning: Cannot open file for writing!\n");
+	    TxError("Warning: Cannot open file \"%s\" for writing!\n", expandname);
 	}
 	else
 	{
@@ -2923,7 +3022,7 @@ DBCellWrite(cellDef, fileName)
 	    if (thestat.st_size != DBFileOffset)
 	    {
 		cellDef->cd_flags |= CDMODIFIED;
-		TxError("Warning: I/O error in writing file\n");
+		TxError("Warning: I/O error in writing file \"%s\"\n", expandname);
 	    }
 	    fclose(realf);
 	}
@@ -2936,7 +3035,7 @@ cleanup:
     freeMagic(tmpname);
     return result;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2969,8 +3068,8 @@ dbWritePaintFunc(tile, cdarg)
 
     int dir;
 
-    if (IsSplit(tile))      
-    { 
+    if (IsSplit(tile))
+    {
 	lMask = DBResidueMask(SplitLeftType(tile));
 	rMask = DBResidueMask(SplitRightType(tile));
 
@@ -3024,7 +3123,7 @@ dbWritePaintFunc(tile, cdarg)
     FPRINTR(arg->wa_file,pstring);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -3075,6 +3174,7 @@ dbWriteCellFunc(cellUse, cdarg)
     struct writeArg *arg = (struct writeArg *) cdarg;
     Transform *t;
     Rect *b;
+    bool subbed = FALSE;
     char     cstring[256], *pathend, *pathstart, *parent;
 
     t = &(cellUse->cu_transform);
@@ -3102,7 +3202,13 @@ dbWriteCellFunc(cellUse, cdarg)
 	    }
 	    else
 		break;
+
 	}
+
+	/* If there are no common components, then restore the leading '/' */
+	if ((*pathorigin == '/') && (pathstart == pathorigin + 1))
+	    pathstart = pathorigin;
+
 	if (pathend != NULL)
 	{
 	    *pathend = '\0';
@@ -3120,25 +3226,90 @@ dbWriteCellFunc(cellUse, cdarg)
     }
     else
     {
-	/* If path starts with home path, then replace with "~"	*/
-	/* to make IP semi-portable between home directories	*/
-	/* with the same file structure.			*/
+#ifdef MAGIC_WRAPPER
+	char *tvar;
 
-	char *homedir = getenv("HOME");
+	/* Check for the leading component of the file path being equal to	*/
+	/* one of several common variable names for the PDK location, and	*/
+	/* if there is a match, then substitute the variable name for the	*/
+	/* matching leading path component.					*/
 
-	if (!strncmp(cellUse->cu_def->cd_file, homedir, strlen(homedir))
-		&& (*(cellUse->cu_def->cd_file + strlen(homedir)) == '/'))
+	if (subbed == FALSE)
 	{
-	    sprintf(cstring, "use %s %c%s ~%s\n", cellUse->cu_def->cd_name,
+	    tvar = (char *)Tcl_GetVar(magicinterp, "PDK_PATH", TCL_GLOBAL_ONLY);
+	    if (tvar)
+		if (!strncmp(pathstart, tvar, strlen(tvar)))
+		{
+	    	    sprintf(cstring, "use %s %c%s $PDK_PATH%s\n",
+				cellUse->cu_def->cd_name,
+				(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+				cellUse->cu_id, pathstart + strlen(tvar));
+		    subbed = TRUE;
+		}
+	}
+	if (subbed == FALSE)
+	{
+	    tvar = (char *)Tcl_GetVar(magicinterp, "PDKPATH", TCL_GLOBAL_ONLY);
+	    if (tvar)
+		if (!strncmp(pathstart, tvar, strlen(tvar)))
+		{
+	    	    sprintf(cstring, "use %s %c%s $PDKPATH%s\n",
+				cellUse->cu_def->cd_name,
+				(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+				cellUse->cu_id, pathstart + strlen(tvar));
+		    subbed = TRUE;
+		}
+	}
+	if (subbed == FALSE)
+	{
+	    tvar = (char *)Tcl_GetVar(magicinterp, "PDK_ROOT", TCL_GLOBAL_ONLY);
+	    if (tvar)
+		if (!strncmp(pathstart, tvar, strlen(tvar)))
+		{
+	    	    sprintf(cstring, "use %s %c%s $PDK_ROOT%s\n",
+				cellUse->cu_def->cd_name,
+				(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+				cellUse->cu_id, pathstart + strlen(tvar));
+		    subbed = TRUE;
+		}
+	}
+	if (subbed == FALSE)
+	{
+	    tvar = (char *)Tcl_GetVar(magicinterp, "PDKROOT", TCL_GLOBAL_ONLY);
+	    if (tvar)
+		if (!strncmp(pathstart, tvar, strlen(tvar)))
+		{
+	    	    sprintf(cstring, "use %s %c%s $PDKROOT%s\n",
+				cellUse->cu_def->cd_name,
+				(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+				cellUse->cu_id, pathstart + strlen(tvar));
+		    subbed = TRUE;
+		}
+	}
+#endif
+
+	if (subbed == FALSE)
+	{
+	    /* If path starts with home path, then replace with "~"	*/
+	    /* to make IP semi-portable between home directories	*/
+	    /* with the same file structure.			*/
+
+	    char *homedir = getenv("HOME");
+
+	    if (!strncmp(cellUse->cu_def->cd_file, homedir, strlen(homedir))
+			&& (*(cellUse->cu_def->cd_file + strlen(homedir)) == '/'))
+	    {
+	    	sprintf(cstring, "use %s %c%s ~%s\n", cellUse->cu_def->cd_name,
 			(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
 			cellUse->cu_id, cellUse->cu_def->cd_file +
 			strlen(homedir));
-	}
-	else
-	{
-	    sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
+	    }
+	    else
+	    {
+	        sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
 			(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
 			cellUse->cu_id, pathstart);
+	    }
 	}
     }
     FPRINTR(arg->wa_file, cstring);
@@ -3167,7 +3338,7 @@ dbWriteCellFunc(cellUse, cdarg)
     FPRINTR(arg->wa_file,cstring)
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -3284,7 +3455,7 @@ DBWriteBackup(filename)
 		return FALSE;
 	    }
 	    close(fd);
-	    DBbackupFile = StrDup(&DBbackupFile, template);
+	    StrDup(&DBbackupFile, template);
 	    freeMagic(template);
 	    TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
 	}
@@ -3294,10 +3465,10 @@ DBWriteBackup(filename)
     {
 	if (strlen(filename) == 0)
 	{
-	    DBbackupFile = StrDup(&DBbackupFile, (char *)NULL);
+	    StrDup(&DBbackupFile, (char *)NULL);
 	    return TRUE;
 	}
-	DBbackupFile = StrDup(&DBbackupFile, filename);
+	StrDup(&DBbackupFile, filename);
 	TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
     }
 
@@ -3344,7 +3515,7 @@ dbWriteBackupFunc(def, f)
     if (name == NULL) name = def->cd_name;
 
     fprintf(f, "file %s\n", name);
-    
+
     /* Save/restore flags such that the crash recovery file write does	*/
     /* *not* clear the CDMODIFIED, et al., bits				*/
 
