@@ -4,16 +4,16 @@
  *	Procedures to interface the database with the window package
  *	for the purposes of window creation, deletion, and modification.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -58,13 +58,13 @@ extern void DBWredisplay();		/* Defined in DBWdisplay.c */
  */
 
 static int (*dbwButtonHandler)() = NULL;
-
+
 /*
  * ----------------------------------------------------------------------------
  *
  * DBWcreate
  *
- *	A new window has been created, create and initialize the needed 
+ *	A new window has been created, create and initialize the needed
  *	structures.
  *
  * Results:
@@ -95,7 +95,7 @@ DBWcreate(window, argc, argv)
      */
 
     newBitMask = (dbwBitMask + 1) | dbwBitMask;
-    if (newBitMask > MAX_BITMASK) 
+    if (newBitMask > MAX_BITMASK)
 	return FALSE;
     bitMask = newBitMask ^ dbwBitMask;
     dbwBitMask = newBitMask;
@@ -127,15 +127,15 @@ DBWcreate(window, argc, argv)
 
     window->w_clientData = (ClientData) crec;
     if (argc > 0)
-	DBWloadWindow(window, argv[0], TRUE, FALSE);
+	DBWloadWindow(window, argv[0], TRUE, FALSE, FALSE);
     else if (ToolGetBox(&boxDef, &box))
     {
-	DBWloadWindow(window, boxDef->cd_name, TRUE, FALSE);
+	DBWloadWindow(window, boxDef->cd_name, TRUE, FALSE, FALSE);
 
 	/* Zoom in on the box, leaving a 10% border or at least 2 units
 	 * on each side.
 	 */
-	
+
 	expand = (box.r_xtop - box.r_xbot)/20;
 	if (expand < 2) expand = 2;
 	box.r_xtop += expand;
@@ -148,12 +148,12 @@ DBWcreate(window, argc, argv)
     }
     else
     {
-	DBWloadWindow(window, (char *) NULL, TRUE, FALSE);
+	DBWloadWindow(window, (char *) NULL, TRUE, FALSE, FALSE);
     }
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * DBWdelete --
@@ -184,7 +184,7 @@ DBWdelete(window)
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -215,14 +215,14 @@ dbwLoadFunc(w, clientData)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
  * DBWreload --
  *
  * Re-load all windows to contain the named cell as a root.
- * This is intended to be called during startup or when restarting a saved 
+ * This is intended to be called during startup or when restarting a saved
  * image of Magic.
  *
  * Results:
@@ -249,10 +249,10 @@ dbwReloadFunc(w, name)
     MagWindow *w;
     char *name;
 {
-    DBWloadWindow(w, name, TRUE, FALSE);
+    DBWloadWindow(w, name, TRUE, FALSE, FALSE);
     return (0);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -274,26 +274,29 @@ dbwReloadFunc(w, name)
  *	cell doesn't change.
  *
  *	If "expand" is true, unexpands all subcells of the root cell.
+ *	If "dereference" is true, ignore path reference in the input file.
  *
  * ----------------------------------------------------------------------------
  */
 
 void
-DBWloadWindow(window, name, ignoreTech, expand)
+DBWloadWindow(window, name, ignoreTech, expand, dereference)
     MagWindow *window;	/* Identifies window to which cell is to be bound */
     char *name;		/* Name of new cell to be bound to this window */
     bool ignoreTech;	/* If FALSE, indicates that the technology of
 			 * the layout must match the current technology.
 			 */
     bool expand;	/* Indicates whether or not to expand the cell */
+    bool dereference;	/* If TRUE, ignore path references in the input */
 {
-    CellDef *newEditDef;
+    CellDef *newEditDef, *deleteDef;
     CellUse *newEditUse;
     void DisplayWindow();
     int res, newEdit, error_val;
     int xadd, yadd;
     Rect loadBox;
     char *rootname;
+    bool isUnnamed;
     int UnexpandFunc();		/* forward declaration */
 
     loadBox.r_xbot = loadBox.r_ybot = 0;
@@ -303,6 +306,19 @@ DBWloadWindow(window, name, ignoreTech, expand)
     newEdit = !WindSearch((WindClient) DBWclientID, (ClientData) NULL,
 			    (Rect *) NULL, dbwLoadFunc, (ClientData) window);
 
+    /* The (UNNAMED) cell generally gets in the way, so delete it if	*/
+    /* any new cell is loaded and (UNNAMED) has no contents.		*/
+
+    if (window->w_surfaceID == (ClientData)NULL)
+	deleteDef = NULL;
+    else
+    {
+	deleteDef = ((CellUse *)window->w_surfaceID)->cu_def;
+	if (strcmp(deleteDef->cd_name, "(UNNAMED)") ||
+		deleteDef->cd_flags & (CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED))
+	    deleteDef = NULL;
+    }
+
     if ((name == (char *) NULL) || (name[0] == '\0'))
     {
 	/*
@@ -312,7 +328,7 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	newEditDef = DBCellLookDef(UNNAMED);
 	if (newEditDef == (CellDef *) NULL)
 	{
-	    newEditDef = DBCellNewDef(UNNAMED, (char *) NULL);
+	    newEditDef = DBCellNewDef(UNNAMED);
 	    DBCellSetAvail(newEditDef);
 	}
     }
@@ -381,9 +397,11 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	    }
 	}
 	if (newEditDef == (CellDef *) NULL)
-	    newEditDef = DBCellNewDef(rootname, (char *) NULL);
+	    newEditDef = DBCellNewDef(rootname);
 
-	if (!DBCellRead(newEditDef, name, ignoreTech, &error_val))
+	if (dereference) newEditDef->cd_flags |= CDDEREFERENCE;
+
+	if (!DBCellRead(newEditDef, name, ignoreTech, dereference, &error_val))
 	{
 	    if (error_val == ENOENT)
 	    {
@@ -413,7 +431,7 @@ DBWloadWindow(window, name, ignoreTech, expand)
 		    newEditDef = DBCellLookDef(UNNAMED);
 		    if (newEditDef == (CellDef *) NULL)
 		    {
-			newEditDef = DBCellNewDef(UNNAMED, (char *) NULL);
+			newEditDef = DBCellNewDef(UNNAMED);
 			DBCellSetAvail(newEditDef);
 		    }
 		}
@@ -475,7 +493,7 @@ DBWloadWindow(window, name, ignoreTech, expand)
 	}
 
 	/* enforce a minimum size of 60 and a border of 10% around the sides */
-	xadd = MAX(0, (60 - (loadBox.r_xtop - loadBox.r_xbot)) / 2) + 
+	xadd = MAX(0, (60 - (loadBox.r_xtop - loadBox.r_xbot)) / 2) +
 		(loadBox.r_xtop - loadBox.r_xbot + 1) / 10;
 	yadd = MAX(0, (60 - (loadBox.r_ytop - loadBox.r_ybot)) / 2) +
 		(loadBox.r_ytop - loadBox.r_ybot + 1) / 10;
@@ -499,6 +517,12 @@ DBWloadWindow(window, name, ignoreTech, expand)
     if (newEdit)
 	DBWAreaChanged(newEditDef, &newEditDef->cd_bbox, DBW_ALLWINDOWS,
 	    &DBAllButSpaceBits);
+
+    /* If the cell before loading was (UNNAMED) and it was	*/
+    /* never modified, then delete it now.			*/
+
+    if (deleteDef != NULL)
+	DBCellDelete(deleteDef->cd_name, TRUE);
 }
 
 /* This function is called for each cell whose expansion status changed.
@@ -517,7 +541,7 @@ UnexpandFunc(use, windowMask)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * DBWexit --
@@ -538,7 +562,7 @@ DBWexit()
 {
     return (CmdWarnWrite() == 1);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -586,7 +610,7 @@ DBWcommands(w, cmd)
 
     DBFixMismatch();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -606,7 +630,7 @@ DBWcommands(w, cmd)
  *	From now on, all button pushes within database windows are
  *	passed to buttonProc, in the following form:
  *
- *	    int 
+ *	    int
  *	    buttonProc(w, cmd)
  *		MagWindow *w;
  *		TxCommand *cmd;
@@ -625,7 +649,7 @@ int (*(DBWNewButtonHandler(buttonProc)))()
     dbwButtonHandler = buttonProc;
     return result;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  * DBWupdate--
@@ -648,7 +672,7 @@ DBWupdate()
     DBWFeedbackShow();
     DBWHLUpdate();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  * DBWinit --
@@ -659,7 +683,7 @@ DBWupdate()
  *	None.
  *
  * Side effects:
- *	A client is added, and an initial empty window is made 
+ *	A client is added, and an initial empty window is made
  * ----------------------------------------------------------------------------
  */
 
@@ -679,7 +703,7 @@ DBWinit()
 	"    releasing the initial button.\n";
 
     /* Initialize */
-    DBWclientID = WindAddClient("layout", DBWcreate, DBWdelete, 
+    DBWclientID = WindAddClient("layout", DBWcreate, DBWdelete,
 	    DBWredisplay, DBWcommands, DBWupdate, DBWexit,
 	    (void (*)()) NULL,
 	    (GrGlyph *) NULL);
