@@ -2,23 +2,23 @@
  * DRCcif.c --
  *
  ******************************************************************************
- * Copyright (C) 1989 Digital Equipment Corporation                           
- * Permission to use, copy, modify, and distribute this              
- * software and its documentation for any purpose and without       
- * fee is hereby granted, provided that the above copyright        
- * notice appear in all copies.  Digital Equipment Corporation    
- * makes no representations about the suitability of this        
- * software for any purpose.  It is provided "as is" without    
- * express or implied warranty.  
- *                                                               
- * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL  
+ * Copyright (C) 1989 Digital Equipment Corporation
+ * Permission to use, copy, modify, and distribute this
+ * software and its documentation for any purpose and without
+ * fee is hereby granted, provided that the above copyright
+ * notice appear in all copies.  Digital Equipment Corporation
+ * makes no representations about the suitability of this
+ * software for any purpose.  It is provided "as is" without
+ * express or implied warranty.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
  * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT         
- * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL 
- * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR    
- * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
- * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS 
- * SOFTWARE. 
+ * MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
+ * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
+ * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+ * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
+ * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
+ * SOFTWARE.
  ****************************************************************************
  *
  */
@@ -48,7 +48,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/malloc.h"
 #include "utils/utils.h"
 
-extern char *drcWhyDup(); 
 extern int  drcCifTile();
 extern int  areaCifCheck();
 extern void drcCheckCifMaxwidth();
@@ -149,7 +148,7 @@ drcCifWarning()
 /*
  * ----------------------------------------------------------------------------
  *
- * drcCifWidth -- same this as drcCifWidth, except that it works on 
+ * drcCifWidth -- same this as drcCifWidth, except that it works on
  *	cif layers
  *
  * Results:
@@ -158,6 +157,13 @@ drcCifWarning()
  * Side effects:
  *	Updates the DRC technology variables.
  *
+ * Notes:  "centidistance" is by default in centimicrons, but it is really in
+ * whatever units are declared in the cifoutput section;  so if cifoutput
+ * declares nanometers, then units are in nanometers.  This may be in
+ * different units from the non-CIF part of the DRC section.  The escape
+ * sequence substitution in the "Why" text will make sure that the correct
+ * physical dimensions are presented to the end user.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -169,7 +175,7 @@ drcCifWidth(argc, argv)
     char *layername = argv[1];
     int scalefactor;
     int centidistance = atoi(argv[2]);
-    char *why = drcWhyDup(argv[3]);
+    int why = drcWhyCreate(argv[3]);
     TileTypeBitMask set, setC, tmp1;
     int	thislayer = -1;
     DRCCookie *dpnew,*dpnext;
@@ -181,8 +187,8 @@ drcCifWidth(argc, argv)
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
-	 
-	 if (strcmp(layer->cl_name,layername) == 0) 
+
+	 if (strcmp(layer->cl_name,layername) == 0)
 	 {
 	      thislayer = i;
 	      break;
@@ -195,13 +201,13 @@ drcCifWidth(argc, argv)
     }
 
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
 
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centidistance, dpnext, &CIFSolidBits,
     		&CIFSolidBits, why, centidistance,
-		DRC_FORWARD, thislayer, 0);
+		DRC_FORWARD | DRC_CIFRULE, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
@@ -210,7 +216,7 @@ drcCifWidth(argc, argv)
 /*
  * ----------------------------------------------------------------------------
  *
- * drcCifSpacing -- same this as drcSpacing, except that it works on cif 
+ * drcCifSpacing -- same this as drcSpacing, except that it works on cif
  * 	layers.
  *
  * Results:
@@ -228,7 +234,7 @@ drcCifSpacing(argc, argv)
     char *argv[];
 {
     char *adjacency = argv[4];
-    char *why = drcWhyDup(argv[5]);
+    int why = drcWhyCreate(argv[5]);
     DRCCookie *dpnext, *dpnew;
     int needReverse = FALSE;
     TileType i, j;
@@ -241,7 +247,7 @@ drcCifSpacing(argc, argv)
 
     layers[0] = argv[1];
     layers[1] = argv[2];
-    
+
     if (drcCifStyle == NULL)
 	return drcCifWarning();
 
@@ -250,12 +256,12 @@ drcCifSpacing(argc, argv)
          for (i = 0; i < drcCifStyle->cs_nLayers;i++)
          {
     	      CIFLayer	*l = drcCifStyle->cs_layers[i];
-	 
+
 	      if (strcmp(l->cl_name,layers[k]) == 0)
 	      {
 	      	   layer[k]=i;
 	           break;
-	      } 
+	      }
          }
          if (i == drcCifStyle->cs_nLayers || layer[k] == -1)
          {
@@ -286,13 +292,13 @@ drcCifSpacing(argc, argv)
 	TechError("Badly formed drc spacing line\n");
 	return (0);
     }
-    
+
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[layer[0]][DRC_CIF_SOLID];
     dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
     drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-    		&cmask, why, centidistance, DRC_FORWARD, layer[1], 0);
+    		&cmask, why, centidistance, DRC_FORWARD | DRC_CIFRULE, layer[1], 0);
     drcCifRules[layer[0]][DRC_CIF_SOLID] = dpnew;
     if (needReverse) dpnew->drcc_flags |= DRC_BOTHCORNERS;
 
@@ -300,9 +306,9 @@ drcCifSpacing(argc, argv)
     dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
     drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-    		&cmask, why, centidistance, DRC_REVERSE, layer[1], 0);
+    		&cmask, why, centidistance, DRC_REVERSE | DRC_CIFRULE, layer[1], 0);
     drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
-    
+
     if (needReverse)
     {
 	 // This is not so much "reverse" as it is just the
@@ -312,29 +318,33 @@ drcCifSpacing(argc, argv)
          dpnext = drcCifRules[layer[1]][DRC_CIF_SOLID];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-		why, centidistance, DRC_FORWARD|DRC_BOTHCORNERS, layer[0], 0);
+		why, centidistance, DRC_FORWARD|DRC_BOTHCORNERS | DRC_CIFRULE,
+		layer[0], 0);
          drcCifRules[layer[1]][DRC_CIF_SOLID] = dpnew;
 
 	 // Add rule in reverse direction
 	 dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
 	 dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
 	 drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-		why, centidistance, DRC_REVERSE|DRC_BOTHCORNERS, layer[0], 0);
+		why, centidistance, DRC_REVERSE|DRC_BOTHCORNERS | DRC_CIFRULE,
+		layer[0], 0);
 	 drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
-    
+
 	 if (layer[0] == layer[1])
 	 {
               dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits,
-			&cmask, why, centidistance, DRC_REVERSE | DRC_BOTHCORNERS,
+			&cmask, why, centidistance,
+			DRC_REVERSE | DRC_BOTHCORNERS | DRC_CIFRULE,
 			layer[0], 0);
                  drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
-	 
+
               dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
               dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
               drcCifAssign(dpnew, centidistance, dpnext, &DBSpaceBits, &cmask,
-			why, centidistance, DRC_REVERSE | DRC_BOTHCORNERS,
+			why, centidistance,
+			DRC_REVERSE | DRC_BOTHCORNERS | DRC_CIFRULE,
 			layer[1], 0);
                  drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
 	 }
@@ -345,16 +355,16 @@ drcCifSpacing(argc, argv)
          dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, scalefactor, dpnext, &DBSpaceBits, &DBZeroTypeBits,
-			why, scalefactor, DRC_FORWARD, layer[0], 0);
+			why, scalefactor, DRC_FORWARD | DRC_CIFRULE, layer[0], 0);
          drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
          dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
          dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
          drcCifAssign(dpnew, scalefactor, dpnext, &DBSpaceBits, &DBZeroTypeBits,
-			why, scalefactor, DRC_FORWARD, layer[1], 0);
+			why, scalefactor, DRC_FORWARD | DRC_CIFRULE, layer[1], 0);
          drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
     }
-    
+
     return ((centidistance+scalefactor-1)/scalefactor);
 }
 
@@ -460,7 +470,7 @@ drcCifFinal()
     for (i = 0; i != MAXCIFLAYERS; i++)
     {
         DRCCookie *dp;
-     	  
+
 	for (dp = drcCifRules[i][DRC_CIF_SPACE]; dp; dp = dp->drcc_next)
 	{
 	    drcCifValid = TRUE;
@@ -487,7 +497,7 @@ drcCifFinal()
  *
  * Side effects:
  *	Error paint, CIF layer generation, lots of stuff going on.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -513,7 +523,7 @@ drcCifCheck(arg)
 	    TxPrintf("Loading DRC CIF style.\n");
 	    CIFCurStyle = NULL;
 	    CIFLoadStyle(drcNeedStyle);
-	    if (drcCifValid == FALSE)
+	    if (drcCifValid != FALSE)
 		CIFCurStyle = CIFSaveStyle;
 	    else
 		drcCifStyle = CIFCurStyle;
@@ -525,7 +535,12 @@ drcCifCheck(arg)
 	}
 	CIFCurStyle = drcCifStyle;
     }
-    if (drcCifValid == FALSE) return;
+    if (drcCifValid == FALSE)
+    {
+	if (CIFSaveStyle != NULL)
+	    CIFCurStyle = CIFSaveStyle;
+	return;
+    }
 
     scale = drcCifStyle->cs_scaleFactor;
     cifrect = *checkRect;
@@ -536,17 +551,18 @@ drcCifCheck(arg)
     arg->dCD_rect = &cifrect;
     oldTiles = DRCstatTiles;
 
-    CIFGen(arg->dCD_celldef, checkRect, CIFPlanes, &DBAllTypeBits, TRUE, TRUE);
+    CIFGen(arg->dCD_celldef, arg->dCD_celldef, checkRect, CIFPlanes,
+	    &DBAllTypeBits, TRUE, TRUE, FALSE, (ClientData)NULL);
 
     for (i = 0; i < drcCifStyle->cs_nLayers; i++)
     {
         for (j = 0; j != 2; j++)
 	{
-	    for (drcCifCur = drcCifRules[i][j]; 
+	    for (drcCifCur = drcCifRules[i][j];
 	       		drcCifCur; drcCifCur = drcCifCur->drcc_next)
             {
 	  	TileTypeBitMask	*mask;
-	  
+
 		arg->dCD_plane = i;
 	        DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &cifrect,
 			(j == DRC_CIF_SOLID) ? &DBSpaceBits : &CIFSolidBits,
@@ -864,7 +880,7 @@ drcCifTile (tile, arg)
 			    /* Nothing */;
 		    if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
 		    {
-			errRect.r_xtop += cptr->drcc_cdist; 	
+			errRect.r_xtop += cptr->drcc_cdist;
 			if (DRCEuclidean)
 			    arg->dCD_radial |= 0x4000;
 		    }
@@ -879,7 +895,7 @@ drcCifTile (tile, arg)
 			    /* Nothing */;
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
 			{
-			    errRect.r_xbot -= cptr->drcc_cdist; 	
+			    errRect.r_xbot -= cptr->drcc_cdist;
 			    if (DRCEuclidean)
 				arg->dCD_radial |= 0x1000;
 			}
@@ -968,7 +984,7 @@ drcCifTile (tile, arg)
 /*
  * ----------------------------------------------------------------------------
  *
- * areaCifCheck -- 
+ * areaCifCheck --
  *
  * Call the function passed down from DRCBasicCheck() if the current tile
  * violates the rule in the given DRCCookie.  If the rule's connectivity
@@ -988,7 +1004,7 @@ drcCifTile (tile, arg)
  */
 
 int
-areaCifCheck(tile, arg) 
+areaCifCheck(tile, arg)
     Tile *tile;
     struct drcClientData *arg;
 {
@@ -1060,7 +1076,7 @@ drcCifArea(argc, argv)
     char *layers = argv[1];
     int centiarea = atoi(argv[2]);
     int	centihorizon = atoi(argv[3]);
-    char *why = drcWhyDup(argv[4]);
+    int why = drcWhyCreate(argv[4]);
     TileTypeBitMask set, setC, tmp1;
     DRCCookie *dpnext, *dpnew;
     TileType i, j;
@@ -1074,8 +1090,8 @@ drcCifArea(argc, argv)
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
-	 
-	 if (strcmp(layer->cl_name,layers) == 0) 
+
+	 if (strcmp(layer->cl_name,layers) == 0)
 	 {
 	      thislayer = i;
 	      break;
@@ -1092,7 +1108,7 @@ drcCifArea(argc, argv)
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centihorizon, dpnext, &CIFSolidBits, &CIFSolidBits,
-		why, centiarea, DRC_AREA | DRC_FORWARD, thislayer, 0);
+		why, centiarea, DRC_AREA | DRC_FORWARD | DRC_CIFRULE, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
@@ -1121,7 +1137,7 @@ drcCifMaxwidth(argc, argv)
     char *layers = argv[1];
     int centidistance = atoi(argv[2]);
     char *bends = argv[3];
-    char *why = drcWhyDup(argv[4]);
+    int why = drcWhyCreate(argv[4]);
     TileTypeBitMask set, setC, tmp1;
     DRCCookie *dpnext, *dpnew;
     TileType i, j;
@@ -1136,8 +1152,8 @@ drcCifMaxwidth(argc, argv)
     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
     {
     	 CIFLayer	*layer = drcCifStyle->cs_layers[i];
-	 
-	 if (strcmp(layer->cl_name,layers) == 0) 
+
+	 if (strcmp(layer->cl_name,layers) == 0)
 	 {
 	      thislayer = i;
 	      break;
@@ -1156,13 +1172,13 @@ drcCifMaxwidth(argc, argv)
     	 TechError("unknown bend option %s\n",bends);
 	 return (0);
     }
-    
+
     scalefactor = drcCifStyle->cs_scaleFactor;
-    centidistance *= drcCifStyle->cs_expander;		// BSI
+    // centidistance *= drcCifStyle->cs_expander;		// BSI
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
     dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
     drcCifAssign(dpnew, centidistance, dpnext, &CIFSolidBits, &CIFSolidBits,
-		why, centidistance, DRC_MAXWIDTH | bend, thislayer, 0);
+		why, centidistance, DRC_MAXWIDTH | DRC_CIFRULE | bend, thislayer, 0);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
@@ -1173,7 +1189,7 @@ drcCifMaxwidth(argc, argv)
  *
  * drcCifCheckArea--
  *
- *	checks to see that a collection of cif tiles 
+ *	checks to see that a collection of cif tiles
  *	have more than a minimum area.
  *
  * Results:
@@ -1197,7 +1213,7 @@ drcCheckCifArea(starttile, arg, cptr)
      Tile		*tile,*tp;
      Rect		*cliprect = arg->dCD_rect;
      int 		scale = drcCifStyle->cs_scaleFactor;
-     
+
     arg->dCD_cptr = (DRCCookie *)cptr;
     if (DRCstack == (Stack *) NULL)
 	DRCstack = StackNew(64);
@@ -1249,7 +1265,7 @@ drcCheckCifArea(starttile, arg, cptr)
 		 arg->dCD_cptr, arg->dCD_clientData);
 	     (*(arg->dCD_errors))++;
 	 }
-	 
+
      }
 forgetit:
      /* reset the tiles */
@@ -1323,7 +1339,7 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
     TileTypeBitMask	*oktypes = &cptr->drcc_mask;
     Tile		*tile,*tp;
     int 		scale = drcCifStyle->cs_scaleFactor;
-     
+
     arg->dCD_cptr = (DRCCookie *)cptr;
     if (DRCstack == (Stack *) NULL)
 	DRCstack = StackNew(64);
@@ -1359,7 +1375,7 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
     {
 	tile = (Tile *) STACKPOP(DRCstack);
 	if (tile->ti_client != (ClientData)DRC_PENDING) continue;
-	
+
 	if (boundrect.r_xbot > LEFT(tile)) boundrect.r_xbot = LEFT(tile);
 	if (boundrect.r_xtop < RIGHT(tile)) boundrect.r_xtop = RIGHT(tile);
 	if (boundrect.r_ybot > BOTTOM(tile)) boundrect.r_ybot = BOTTOM(tile);
@@ -1387,7 +1403,7 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
      }
 
      if (boundrect.r_xtop - boundrect.r_xbot > edgelimit &&
-             boundrect.r_ytop - boundrect.r_ybot > edgelimit) 
+             boundrect.r_ytop - boundrect.r_ybot > edgelimit)
      {
 	 Rect	rect;
 	 TiToRect(starttile,&rect);
@@ -1403,7 +1419,7 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
 		  (*(arg->dCD_errors))++;
 	      }
 	 }
-	 
+
      }
      /* reset the tiles */
      starttile->ti_client = (ClientData)DRC_UNPROCESSED;
