@@ -6,16 +6,16 @@
  * records areas that need to be rechecked, and provides a
  * routine to perform those checks in background.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -46,7 +46,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 #ifdef MAGIC_WRAPPER
 
-/* Global variable which indicates that the background checker is 
+/* Global variable which indicates that the background checker is
  * still registered as an idle process.
  */
 
@@ -146,7 +146,7 @@ extern int drcCheckTile();
 extern CellDef *DRCErrorDef;
 extern TileType DRCErrorType;
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * DRCCheckThis --
@@ -197,14 +197,19 @@ DRCCheckThis (celldef, operation, area)
 					 *  of CellDefs waiting for DRC
 					 */
 
-    /* Ignore read-only, internal, and vendor GDS cells.  None of these	*/
-    /* can contain DRC errors that could be fixed in magic.		*/
-
-    if (celldef->cd_flags & (CDVENDORGDS | CDNOEDIT | CDINTERNAL)) return;
+    /* Ignore internal cells. */
+    if (celldef->cd_flags & CDINTERNAL) return;
 
     /* Insert celldef into list of Defs waiting to be checked, unless	*/
     /* it is already there.						*/
 
+#if (0)
+
+    /* The switch to copying up DRC errors from non-interacting	    */
+    /* child cells means that the child cells must be processed	    */
+    /* first.  So this routine changes from prepending the cell	    */
+    /* to the list to appending it.				    */
+
     pback = &DRCPendingRoot;
     p = DRCPendingRoot;
 
@@ -226,6 +231,33 @@ DRCCheckThis (celldef, operation, area)
     p->dpc_next = DRCPendingRoot;
     DRCPendingRoot = p;
 
+#endif
+    /* Append new cell to check to the pending list */
+    if (DRCPendingRoot == NULL)
+    {
+	p = (DRCPendingCookie *) mallocMagic(sizeof (DRCPendingCookie));
+	p->dpc_def = celldef;
+	p->dpc_next = NULL;
+	DRCPendingRoot = p;
+    }
+    else
+    {
+	DRCPendingCookie *plast;
+	plast = DRCPendingRoot;
+	while (plast->dpc_next != NULL)
+	{
+	    if (plast->dpc_def == celldef) break;
+	    plast = plast->dpc_next;
+	}
+	if (plast->dpc_next == NULL)
+	{
+	    p = (DRCPendingCookie *) mallocMagic(sizeof (DRCPendingCookie));
+	    p->dpc_def = celldef;
+	    p->dpc_next = NULL;
+	    plast->dpc_next = p;
+	}
+    }
+
     /* Mark the area in this celldef (but don't worry about this stuff
      * for undo purposes).  Also, it's important to disable interrupts
      * in here, or the paint operation could get aborted underneath us.
@@ -268,7 +300,7 @@ DRCCheckThis (celldef, operation, area)
 	DRCCheckThis (cu->cu_parent, TT_CHECKSUBCELL, &transRect);
     }
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -345,7 +377,7 @@ DRCInit()
     DRCdef = DBCellLookDef(DRCYANK);
     if (DRCdef == (CellDef *) NULL)
     {
-	DRCdef = DBCellNewDef (DRCYANK,(char *) NULL);
+	DRCdef = DBCellNewDef(DRCYANK);
 	ASSERT(DRCdef != (CellDef *) NULL, "DRCInit");
 	DBCellSetAvail(DRCdef);
 	DRCdef->cd_flags |= CDINTERNAL;
@@ -410,7 +442,7 @@ DRCBreak()
     if (DRCHasWork && (DRCStatus == DRC_IN_PROGRESS))
     {
 	UndoEnable();
-	
+
 	/* fprintf(stderr, "DRC breaking. . .\n"); fflush(stderr); */
 
 	/* As a convenience for debugging DRC stuff, we pretend the DRC
@@ -441,7 +473,7 @@ DRCBreak()
  *	None.
  *
  * Side effects:
- *	Modifies the DRC_CHECK and DRC_ERROR planes 
+ *	Modifies the DRC_CHECK and DRC_ERROR planes
  *	of the CellDefs on the DRCPending list.
  * ----------------------------------------------------------------------------
  */
@@ -550,7 +582,7 @@ checkDone:
 #endif
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * drcCheckTile --
@@ -627,7 +659,7 @@ drcCheckTile(tile, arg)
     /* Find the checkerboard square containing the lower-left corner
      * of the check tile, then find all check tiles within that square.
      */
-    
+
     DRCstatSquares += 1;
     square.r_xbot = (LEFT(tile)/DRCStepSize) * DRCStepSize;
     if (square.r_xbot > LEFT(tile)) square.r_xbot -= DRCStepSize;
@@ -658,8 +690,6 @@ drcCheckTile(tile, arg)
     DBClearPaintPlane(drcDisplayPlane);
     (void) DBSrPaintArea((Tile *) NULL, celldef->cd_planes[PL_DRC_ERROR],
 	&square, &DBAllButSpaceBits, drcXorFunc, (ClientData) NULL);
-    
-    /* Check #1:  recheck the paint of the cell, ignoring subcells. */
 
     DRCErrorType = TT_ERROR_P;
     DBClearPaintPlane(drcTempPlane);
@@ -670,10 +700,7 @@ drcCheckTile(tile, arg)
      * computed within DRCInteractionCheck()).
      */
 
-    /* DRCBasicCheck (celldef, &checkbox, &erasebox, drcPaintError,
-		(ClientData) drcTempPlane); */
-
-    /* Check #2:  check interactions between paint and subcells, and
+    /* Check interactions between paint and subcells, and
      * also between subcells and other subcells.  If any part of a
      * square is rechecked for interactions, the whole thing has to
      * be rechecked.  We use TT_ERROR_S tiles for this so that we
@@ -683,13 +710,7 @@ drcCheckTile(tile, arg)
 
     DRCErrorType = TT_ERROR_S;
     (void) DRCInteractionCheck(celldef, &square, &erasebox,
-		drcPaintError, (ClientData) drcTempPlane);
-    
-    /* Check #3:  check for array formation errors in the area. */
-
-    DRCErrorType = TT_ERROR_P;
-    (void) DRCArrayCheck(celldef, &erasebox, drcPaintError,
-	(ClientData) drcTempPlane);
+		drcPaintError, (ClientData)drcTempPlane);
 
     /* If there was an interrupt, return without modifying the cell
      * at all.
@@ -707,7 +728,7 @@ drcCheckTile(tile, arg)
     DBPaintPlane(celldef->cd_planes[PL_DRC_CHECK], &erasebox,
 	DBStdEraseTbl(TiGetType(tile), PL_DRC_CHECK),
 	(PaintUndoInfo *) NULL);
-    DBPaintPlane(celldef->cd_planes[PL_DRC_ERROR], &checkbox,
+    DBPaintPlane(celldef->cd_planes[PL_DRC_ERROR], &erasebox,
 	DBStdEraseTbl(TT_ERROR_P, PL_DRC_ERROR),
 	(PaintUndoInfo *) NULL);
     DBPaintPlane(celldef->cd_planes[PL_DRC_ERROR], &checkbox,
@@ -721,7 +742,7 @@ drcCheckTile(tile, arg)
      * clip to square and redisplay.  If check tiles are being
      * displayed, then always redisplay the entire area.
      */
-    
+
     (void) DBSrPaintArea((Tile *) NULL, celldef->cd_planes[PL_DRC_ERROR],
 	&square, &DBAllButSpaceBits, drcXorFunc, (ClientData) NULL);
     if (DBBoundPlane(drcDisplayPlane, &redisplayArea))
@@ -771,7 +792,7 @@ drcPutBackFunc(tile, cellDef)
 	(PaintUndoInfo *) NULL);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
