@@ -37,6 +37,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "textio/txcommands.h"
 #endif
 #include "extflat/extflat.h"
+#include "extflat/EFint.h"
 #include "extract/extract.h"	/* for extDevTable */
 #include "utils/runstats.h"
 #include "utils/malloc.h"
@@ -91,18 +92,20 @@ FILE *esLabF = NULL;
 static unsigned short esFormat = MIT ;
 
 struct {
-   short resClassSD ;  /* the resistance class of the src/drn of the dev */
-   short resClassSub ; /* the resistance class of the substrate of the dev */
-   char  *defSubs ;    /* the default substrate node */
+   short resClassSource ;   /* The resistance class of the source of the dev */
+   short resClassDrain ;    /* The resistance class of the drain of the dev */
+   short resClassSub ;	    /* The resistance class of the substrate of the dev */
+   TileType devType ;	    /* Magic tile type of the device */
+   char  *defSubs ;	    /* The default substrate node */
 } fetInfo[MAXDEVTYPES];
 
 typedef struct {
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClient;
 
 typedef struct {
 	HierName *lastPrefix;
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClientHier;
 
 #define NO_RESCLASS	-1
@@ -225,7 +228,7 @@ Exttosim_Init(interp)
 
 #endif /* EXT2SIM_AUTO */
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -257,14 +260,14 @@ CmdExtToSim(w, cmd)
     int value;
     int option = EXTTOSIM_RUN;
     int argc = cmd->tx_argc;
-    char **argv = cmd->tx_argv; 
+    char **argv = cmd->tx_argv;
     char **msg;
     bool err_result;
 
-    short sd_rclass;
-    short sub_rclass;
+    short s_rclass, d_rclass, sub_rclass;
     char *devname;
     char *subname;
+    TileType devtype;
     int idx;
 
     static EFCapValue LocCapThreshold = 2;
@@ -371,7 +374,7 @@ CmdExtToSim(w, cmd)
 		LocCapThreshold = INFINITE_THRESHOLD_F;
 	    else if (StrIsNumeric(cmd->tx_argv[2]))
 		LocCapThreshold = atoCap(cmd->tx_argv[2]);
-	    else 
+	    else
 		TxError("exttosim: numeric value or \"infinite\" expected.\n");
 	    break;
 	case EXTTOSIM_RTHRESH:
@@ -389,7 +392,7 @@ CmdExtToSim(w, cmd)
 		LocResistThreshold = atoi(cmd->tx_argv[2]);
 	    else if (!strncmp(cmd->tx_argv[2], "inf", 3))
 		LocResistThreshold = INFINITE_THRESHOLD;
-	    else 
+	    else
 		TxError("exttosim: integer value or \"infinite\" expected.\n");
 	    break;
 	case EXTTOSIM_FORMAT:
@@ -439,7 +442,7 @@ CmdExtToSim(w, cmd)
 		    break;
 	    }
 	    break;
-	    
+
 	case EXTTOSIM_DEFAULT:
 	    LocCapThreshold = 2;
 	    LocResistThreshold = 10;
@@ -492,7 +495,7 @@ runexttosim:
     if (inName == NULL)
     {
 	/* Assume that we want to do exttospice on the currently loaded cell */
-	
+
 	if (w == (MagWindow *) NULL)
 	    windCheckOnlyWindow(&w, DBWclientID);
 
@@ -571,9 +574,11 @@ runexttosim:
 
     for ( i = 0 ; i < MAXDEVTYPES ; i++ )
     {
-	fetInfo[i].resClassSD = NO_RESCLASS;
+	fetInfo[i].resClassSource = NO_RESCLASS;
+	fetInfo[i].resClassDrain = NO_RESCLASS;
 	fetInfo[i].resClassSub = NO_RESCLASS;
 	fetInfo[i].defSubs = NULL;
+	fetInfo[i].devType = TT_SPACE;
     }
 
     /* Get fetInfo information from the current extraction style 	 */
@@ -581,7 +586,8 @@ runexttosim:
     /* command)								 */
 
     idx = 0;
-    while (ExtGetDevInfo(idx++, &devname, &sd_rclass, &sub_rclass, &subname))
+    while (ExtGetDevInfo(idx++, &devname, &devtype, &s_rclass, &d_rclass,
+		&sub_rclass, &subname))
     {
 	if (idx == MAXDEVTYPES)
 	{
@@ -592,16 +598,18 @@ runexttosim:
 
 	if (EFStyle != NULL)
 	{
-	    fetInfo[i].resClassSD = sd_rclass;
+	    fetInfo[i].resClassSource = s_rclass;
+	    fetInfo[i].resClassDrain = d_rclass;
 	    fetInfo[i].resClassSub = sub_rclass;
 	    fetInfo[i].defSubs = subname;
+	    fetInfo[i].devType = devtype;
 	}
     }
 
     /* Write the output file */
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
-		   (esFormat == MIT) ? "MIT" :  
+		   (esFormat == MIT) ? "MIT" :
 		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
 
     /* Convert the hierarchical description to a flat one */
@@ -669,24 +677,26 @@ main(argc, argv)
     /* create default fetinfo entries (MOSIS) which can be overriden by
        the command line arguments */
     for ( i = 0 ; i < MAXDEVTYPES ; i++ ) {
-	fetInfo[i].resClassSD = NO_RESCLASS;
+	fetInfo[i].resClassSource = NO_RESCLASS;
+	fetInfo[i].resClassDrain = NO_RESCLASS;
 	fetInfo[i].resClassSub = NO_RESCLASS;
 	fetInfo[i].defSubs = NULL;
+	fetInfo[i].devType = TT_SPACE;
     }
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "nfet");
-    fetInfo[i].resClassSD = 0 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0 ;
     fetInfo[i].resClassSub = NO_RESCLASS ;
     fetInfo[i].defSubs = "Gnd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "pfet");
-    fetInfo[i].resClassSD = 1 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1 ;
     fetInfo[i].resClassSub = 6 ;
     fetInfo[i].defSubs = "Vdd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "nmos");
-    fetInfo[i].resClassSD = 0 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0 ;
     fetInfo[i].resClassSub = NO_RESCLASS ;
     fetInfo[i].defSubs = "Gnd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "pmos");
-    fetInfo[i].resClassSD = 1 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1 ;
     fetInfo[i].resClassSub = 6 ;
     fetInfo[i].defSubs = "Vdd!";
     /* Process command line arguments */
@@ -730,7 +740,7 @@ main(argc, argv)
     }
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
-		   (esFormat == MIT) ? "MIT" :  
+		   (esFormat == MIT) ? "MIT" :
 		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
 
     /* Convert the hierarchical description to a flat one */
@@ -769,7 +779,7 @@ main(argc, argv)
 }
 
 #endif		/* MAGIC_WRAPPER */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -887,12 +897,14 @@ simmainArgs(pargc, pargv)
 	    	if ( sscanf(rp, "%d/%s",  &rClass, subsNode) != 2 ) goto usage;
 	    }
 	    ndx = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, cp);
-	    fetInfo[ndx].resClassSD = rClass;
+	    fetInfo[ndx].resClassSource = rClass;
+	    fetInfo[ndx].resClassDrain = rClass;
 	    fetInfo[ndx].resClassSub = rClassSub;
 	    fetInfo[ndx].defSubs = (char *) mallocMagic((unsigned) (strlen(subsNode)+1));
+	    fetInfo[ndx].devType = TT_SPACE;
 	    strcpy(fetInfo[ndx].defSubs,subsNode);
-	    TxError("Info: fet %s(%d) sdRclass=%d subRclass=%d dSub=%s\n", 
-	    	cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub, 
+	    TxError("Info: fet %s(%d) sdRclass=%d subRclass=%d dSub=%s\n",
+	    	cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub,
             	fetInfo[ndx].defSubs);
 	    break;
 	    }
@@ -924,7 +936,7 @@ usage:
     exit (1);
 #endif
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -949,7 +961,7 @@ HierName *suffix;
 	return(((EFNodeName *) HashGetValue(he))->efnn_node);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -981,9 +993,9 @@ HierName *suffix;
  */
 
 int
-simdevVisit(dev, hierName, scale, trans)
+simdevVisit(dev, hc, scale, trans)
     Dev *dev;		/* Device being output */
-    HierName *hierName;	/* Hierarchical path down to this device */
+    HierContext *hc;	/* Hierarchical context down to this device */
     float scale;	/* Scale transform for output */
     Transform *trans;	/* Coordinate transform */
 {
@@ -992,6 +1004,7 @@ simdevVisit(dev, hierName, scale, trans)
     int l, w;
     Rect r;
     char name[12];
+    HierName *hierName = hc->hc_hierName;
 
     sprintf(name, "output");
 
@@ -1041,6 +1054,41 @@ simdevVisit(dev, hierName, scale, trans)
 	case DEV_CAPREV:
 	    fprintf(esSimF, "c");	/* sim format extension */
 	    break;
+	case DEV_FET:
+	case DEV_MOSFET:
+	case DEV_ASYMMETRIC:
+	case DEV_MSUBCKT:
+	    /* The sim file format only understands "n" and "p" for FETs.   */
+	    /* The extraction method says nothing about which is which.	    */
+	    /* The EFDevTypes[] should ideally start with "n" or "p".  If   */
+	    /* it doesn't, then dev->dev_type should.  If neither does,	    */
+	    /* then use EFDevTypes[] but flag an error.			    */
+
+	    if (EFDevTypes[dev->dev_type][0] == 'n' ||
+		    EFDevTypes[dev->dev_type][0] == 'p')
+	    {
+		fprintf(esSimF, "%c", EFDevTypes[dev->dev_type][0]);
+	    }
+	    else
+	    {
+		TileType ttype = fetInfo[dev->dev_type].devType;
+
+		if (DBTypeLongNameTbl[ttype][0] == 'n' ||
+			    DBTypeLongNameTbl[ttype][0] == 'p')
+		{
+		    fprintf(esSimF, "%c", DBTypeLongNameTbl[ttype][0]);
+		}
+		else
+		{
+		    TxError("Error: MOSFET device type \"%s\" does not start with "
+			    "\"n\" or \"p\" as required for the .sim format\n",
+			    EFDevTypes[dev->dev_type]);
+
+		    /* Default to "n" */
+		    fprintf(esSimF, "n");
+		}
+	    }
+	    break;
 	default:
 	    fprintf(esSimF, "%c", EFDevTypes[dev->dev_type][0]);
 	    break;
@@ -1062,13 +1110,13 @@ simdevVisit(dev, hierName, scale, trans)
     if (dev->dev_class == DEV_BJT && subnode)
     {
         sprintf(name, "fet");
-        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF);
     }
     else if ((dev->dev_class == DEV_DIODE || dev->dev_class == DEV_PDIODE ||
 		dev->dev_class == DEV_NDIODE) && dev->dev_nterm == 1 && subnode)
     {
         sprintf(name, "fet");
-        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF);
     }
     else if (dev->dev_nterm > 2)
         simdevOutNode(hierName, drain->dterm_node->efnode_name->efnn_hier, name, esSimF);
@@ -1078,7 +1126,7 @@ simdevVisit(dev, hierName, scale, trans)
     {
 	putc(' ', esSimF);
 	simdevSubstrate(hierName, subnode->efnode_name->efnn_hier,
-	             dev->dev_type, 0, FALSE, esSimF);
+	             dev->dev_type, 0.0, FALSE, esSimF);
     }
 
     GeoTransRect(trans, &dev->dev_rect, &r);
@@ -1091,7 +1139,7 @@ simdevVisit(dev, hierName, scale, trans)
 	* comparator will deal with any problem arising from
 	* swapping L and W.
 	*/
-	
+
 	int n;
 	double cl, cw;
 	double chp = 0.0;
@@ -1158,12 +1206,12 @@ simdevVisit(dev, hierName, scale, trans)
 	   if ( esFormat == SU ) {
 	       fprintf(esSimF, "%s", (source->dterm_attrs) ? "," : " s=" );
 	       if (hierS)
-	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSD, 
+	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSource,
 		      scale, esSimF);
 	       else {
 	         snode= SimGetNode(hierName,
 			     source->dterm_node->efnode_name->efnn_hier);
-	         simnAP(snode, fetInfo[dev->dev_type].resClassSD, scale, esSimF);
+	         simnAP(snode, fetInfo[dev->dev_type].resClassSource, scale, esSimF);
 	       }
 	   }
 	   if (drain->dterm_attrs) {
@@ -1175,13 +1223,13 @@ simdevVisit(dev, hierName, scale, trans)
 	   }
 	   if ( esFormat == SU ) {
 	       fprintf(esSimF, "%s", (drain->dterm_attrs) ? "," : " d=" );
-	       if (hierD) 
-	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassSD, 
+	       if (hierD)
+	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassDrain,
 		      scale, esSimF);
 	       else {
 	         dnode = SimGetNode(hierName,
 			      drain->dterm_node->efnode_name->efnn_hier);
-	         simnAP(dnode, fetInfo[dev->dev_type].resClassSD, 
+	         simnAP(dnode, fetInfo[dev->dev_type].resClassDrain,
 		      scale, esSimF);
 	       }
 	   }
@@ -1196,7 +1244,8 @@ int
 simdevSubstrate( prefix, suffix, type, scale, doAP, outf)
 HierName *prefix;
 HierName *suffix;
-int type, scale;
+int type;
+float scale;
 bool doAP;
 FILE *outf;
 {
@@ -1250,7 +1299,7 @@ FILE *outf;
  *
  * simnAP, simnAPHier --
  *
- * Output the area perimeter of the node with type type if it has not 
+ * Output the area perimeter of the node with type type if it has not
  * been visited.
  * The simnAPHier version outputs the area and perimeter only within the
  * local subcell with hierarchical name hierName.
@@ -1261,15 +1310,16 @@ FILE *outf;
  *
  * ----------------------------------------------------------------------------
  */
- 
+
 bool simnAP(node, resClass, scale, outf)
 EFNode *node;
-int  resClass, scale;
+int  resClass;
+float scale;
 FILE *outf;
 {
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL ) 
+	if ( node->efnode_client == (ClientData) NULL )
 		initNodeClient(node);
 	if ( resClass == NO_RESCLASS ||
 	     beenVisited((nodeClient *)node->efnode_client, resClass) ) {
@@ -1277,8 +1327,8 @@ FILE *outf;
 		return FALSE;
 	}
 	markVisited((nodeClient *)node->efnode_client, resClass);
-	a = node->efnode_pa[resClass].pa_area*scale*scale;
-	p = node->efnode_pa[resClass].pa_perim*scale;
+	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
+	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
 	if ( a < 0 ) a = 0;
 	if ( p < 0 ) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
@@ -1288,14 +1338,15 @@ FILE *outf;
 bool simnAPHier(dterm, hierName, resClass, scale, outf)
     DevTerm *dterm;
     HierName *hierName;
-    int  resClass, scale;
+    int  resClass;
+    float scale;
     FILE *outf;
 {
 	EFNode *node = dterm->dterm_node;
 	nodeClientHier   *nc ;
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL ) 
+	if ( node->efnode_client == (ClientData) NULL )
 		initNodeClientHier(node);
 	nc = (nodeClientHier *)node->efnode_client;
 	if ( nc->lastPrefix != hierName ) {
@@ -1308,15 +1359,15 @@ bool simnAPHier(dterm, hierName, resClass, scale, outf)
 		return FALSE;
 	}
 	markVisited((nodeClientHier *)node->efnode_client, resClass);
-	a = node->efnode_pa[resClass].pa_area*scale*scale;
-	p = node->efnode_pa[resClass].pa_perim*scale;
+	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
+	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
 	if ( a < 0 ) a = 0;
 	if ( p < 0 ) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
 	return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1362,7 +1413,7 @@ simdevOutNode(prefix, suffix, name, outf)
 	initNodeClient(nn->efnn_node);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1404,7 +1455,7 @@ int simcapVisit(hierName1, hierName2, cap)
     fprintf(esSimF, esCapFormat, cap);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1444,7 +1495,7 @@ simresistVisit(hierName1, hierName2, res)
     fprintf(esSimF, " %g\n", res / 1000.);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1623,7 +1674,7 @@ Dev    *dev;
 	} \
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1640,9 +1691,9 @@ Dev    *dev;
  * ----------------------------------------------------------------------------
  */
 int
-simmergeVisit(dev, hierName, scale, trans)
+simmergeVisit(dev, hc, scale, trans)
 Dev *dev;		/* Dev to examine */
-HierName *hierName;	/* Hierarchical path down to this dev */
+HierContext *hc;	/* Hierarchical context down to this dev */
 float scale;		/* Scale transform */
 Transform *trans;	/* Coordinate transform (not used) */
 {
@@ -1653,6 +1704,7 @@ Transform *trans;	/* Coordinate transform (not used) */
 	int      pmode, l, w;
 	float	 m;
 	devMerge *fp, *cfp;
+	HierName *hierName = hc->hc_hierName;
 
 	if (dev->dev_nterm < 2) {
 		TxError("outPremature\n");
@@ -1670,11 +1722,11 @@ Transform *trans;	/* Coordinate transform (not used) */
 	dnode = SimGetNode (hierName, drain->dterm_node->efnode_name->efnn_hier);
 
 	EFGetLengthAndWidth(dev, &l, &w);
-	fp = simmkDevMerge((int)(l*scale), (int)(w*scale), gnode, snode, 
+	fp = simmkDevMerge((int)(l*scale), (int)(w*scale), gnode, snode,
 			dnode, subnode, hierName, dev);
 
 	/*
-	 * run the list of devs. compare the current one with 
+	 * run the list of devs. compare the current one with
 	 * each one in the list. if they fullfill the matching requirements
 	 * merge them.
 	 */
@@ -1686,14 +1738,14 @@ Transform *trans;	/* Coordinate transform (not used) */
 		cg = &cfp->dev->dev_terms[0];
 		cs = cd = &cfp->dev->dev_terms[1];
 		if (cfp->dev->dev_nterm >= 3) {
-			if ( pmode == PARALLEL ) 
+			if ( pmode == PARALLEL )
 				cd = &cfp->dev->dev_terms[2];
-			else if ( pmode == PARALLEL_R ) 
+			else if ( pmode == PARALLEL_R )
 				cs = &cfp->dev->dev_terms[2];
 		}
-		
+
 		m = esFMult[cfp->esFMIndex] + ((float)fp->w/(float)cfp->w);
-		setDevMult(fp->esFMIndex, DEV_KILLED); 
+		setDevMult(fp->esFMIndex, DEV_KILLED);
 		setDevMult(cfp->esFMIndex, m);
 		esDevsMerged++;
 		freeMagic(fp);
