@@ -7,7 +7,7 @@
  *
  * Flattens the tree rooted at file.ext, reading in additional .ext
  * files as specified by "use" lines in file.ext.
- * 
+ *
  */
 
 #include <stdio.h>
@@ -16,21 +16,22 @@
 #include <ctype.h>
 #include <math.h>	/* for INFINITY */
 
+#ifdef MAGIC_WRAPPER
 #include "tcltk/tclmagic.h"
+#endif
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
 #include "utils/styles.h"
 #include "tiles/tile.h"
-#ifdef MAGIC_WRAPPER
 #include "database/database.h"
 #include "windows/windows.h"
 #include "textio/textio.h"
 #include "dbwind/dbwind.h"
 #include "textio/txcommands.h"
-#endif
 #include "extflat/extflat.h"
+#include "extflat/EFint.h"
 #include "extract/extract.h"
 #include "extract/extractInt.h"
 #include "utils/malloc.h"
@@ -40,12 +41,12 @@ int antennacheckArgs();
 int antennacheckVisit();
 
 typedef struct {
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClient;
 
 typedef struct {
 	HierName *lastPrefix;
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClientHier;
 
 #define NO_RESCLASS	-1
@@ -83,14 +84,14 @@ static int efAntennaDebug = FALSE;
 TileType *EFDeviceTypes;
 
 typedef struct _aas {
-    int *accum;		/* Pointer to array of accumulated areas per type */
+    dlong *accum;	/* Pointer to array of accumulated areas per type */
     int pNum;		/* Plane of check */
     Rect r;		/* Holds any one visited rectangle */
     CellDef *def;	/* CellDef for adding feedback */
 } AntennaAccumStruct;
 
 typedef struct _gdas {
-    int accum;		/* Accumulated area of all gates/diff */
+    dlong accum;	/* Accumulated area of all gates/diff */
     Rect r;		/* Holds any one visited rectangle */
     CellDef *def;	/* CellDef for adding feedback */
 } GateDiffAccumStruct;
@@ -100,7 +101,7 @@ typedef struct _ams {
     CellDef *def;	/* CellDef for adding feedback */
 } AntennaMarkStruct;
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -126,14 +127,13 @@ CmdAntennaCheck(w, cmd)
     int option = ANTENNACHECK_RUN;
     int value;
     int argc = cmd->tx_argc;
-    char **argv = cmd->tx_argv; 
+    char **argv = cmd->tx_argv;
     char **msg;
     bool err_result;
 
     short sd_rclass;
     short sub_rclass;
     char *devname;
-    char *subname;
     int idx;
 
     CellUse *editUse;
@@ -196,7 +196,7 @@ runantennacheck:
     if (inName == NULL)
     {
 	/* Assume that we want to do exttospice on the currently loaded cell */
-	
+
 	if (w == (MagWindow *) NULL)
 	    windCheckOnlyWindow(&w, DBWclientID);
 
@@ -244,7 +244,7 @@ runantennacheck:
     efAntennaDebug = FALSE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -336,32 +336,34 @@ HierName *suffix;
  */
 
 int
-antennacheckVisit(dev, hierName, scale, trans, editUse)
+antennacheckVisit(dev, hc, scale, trans, editUse)
     Dev *dev;		/* Device being output */
-    HierName *hierName;	/* Hierarchical path down to this device */
+    HierContext *hc;	/* Hierarchical context down to this device */
     float scale;	/* Scale transform for output */
     Transform *trans;	/* Coordinate transform */
     CellUse *editUse;	/* ClientData is edit cell use */
 {
     DevTerm *gate;
     TileType t, conType;
-    int pos, pNum, pNum2, pmax, p, i, j, gatearea, diffarea, total;
+    int pos, pNum, pNum2, pmax, p, i, j, total;
+    dlong gatearea, diffarea;
     double anttotal;
-    float saveRatio;
-    int *antennaarea;
+    float saveRatio, ratioTotal;
+    dlong *antennaarea;
     Rect r, gaterect;
     EFNode *gnode;
     SearchContext scx;
     TileTypeBitMask gatemask, saveConMask;
     bool antennaError;
+    HierName *hierName = hc->hc_hierName;
 
     extern CellDef *extPathDef;	    /* see extract/ExtLength.c */
     extern CellUse *extPathUse;	    /* see extract/ExtLength.c */
 
     extern int  areaAccumFunc(), antennaAccumFunc(), areaMarkFunc();
 
-    antennaarea = (int *)mallocMagic(DBNumTypes * sizeof(int));
-    
+    antennaarea = (dlong *)mallocMagic(DBNumTypes * sizeof(dlong));
+
     switch(dev->dev_class)
     {
 	case DEV_FET:
@@ -400,6 +402,13 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 	    efGates++;
 	    if (efGates % 100 == 0) TxPrintf("   %d gates analyzed.\n", efGates);
 
+	    /* Diagnostic for debugging */
+	    /*
+	    TxPrintf("Gate %d : (%d %d) to (%d %d) net %s\n", efGates, r.r_xbot,
+		    r.r_ybot, r.r_xtop, r.r_ytop,
+		    gnode->efnode_name->efnn_hier->hn_name);
+	    */
+
 	    /* Find the plane of the gate type */
 	    t = EFDeviceTypes[dev->dev_type];
 	    pNum = DBPlane(t);
@@ -482,7 +491,7 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 			break;
 		    }
 
-	        for (i = 0; i < DBNumTypes; i++) antennaarea[i] = 0;
+	        for (i = 0; i < DBNumTypes; i++) antennaarea[i] = (dlong)0;
 	        gatearea = 0;
 	        diffarea = 0;
 
@@ -496,10 +505,10 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 		/* To do:  Mark tiles so area count can be progressive */
 
 		DBTreeCopyConnect(&scx, &DBConnectTbl[t], 0,
-			DBConnectTbl, &TiPlaneRect, extPathUse);
+			DBConnectTbl, &TiPlaneRect, FALSE, extPathUse);
 
 		/* Search planes of tie types and accumulate all tiedown areas */
-		gdas.accum = 0;
+		gdas.accum = (dlong)0;
 		for (p = 0;  p < DBNumPlanes; p++)
 		    DBSrPaintArea((Tile *)NULL, extPathUse->cu_def->cd_planes[p],
 			    &TiPlaneRect, &ExtCurStyle->exts_antennaTieTypes,
@@ -507,7 +516,7 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 		diffarea = gdas.accum;
 
 		/* Search plane of gate type and accumulate all gate area */
-		gdas.accum = 0;
+		gdas.accum = (dlong)0;
 		DBSrPaintArea((Tile *)NULL, extPathUse->cu_def->cd_planes[pNum],
 			&TiPlaneRect, &gatemask, areaAccumFunc, (ClientData)&gdas);
 		gatearea = gdas.accum;
@@ -547,15 +556,16 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 			antennaError = TRUE;
 			if (efAntennaDebug == TRUE)
 			{
+			    TxError("Cell: %s\n", hc->hc_use->use_id);
 			    TxError("Antenna violation detected at plane %s\n",
 				    DBPlaneLongNameTbl[pNum2]);
 			    TxError("Effective antenna ratio %g > limit %g\n",
 				    saveRatio * (float)anttotal / (float)gatearea,
 				    saveRatio);
-			    TxError("Gate rect (%d %d) to (%d %d)\n", 
+			    TxError("Gate rect (%d %d) to (%d %d)\n",
 				    gdas.r.r_xbot, gdas.r.r_ybot,
 				    gdas.r.r_xtop, gdas.r.r_ytop);
-			    TxError("Antenna rect (%d %d) to (%d %d)\n", 
+			    TxError("Antenna rect (%d %d) to (%d %d)\n",
 				    aas.r.r_xbot, aas.r.r_ybot,
 				    aas.r.r_xtop, aas.r.r_ytop);
 			}
@@ -566,13 +576,15 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 		    anttotal = 0.0;
 		    saveRatio = 0.0;
 		    for (i = 0; i < DBNumTypes; i++)
-			if (ExtCurStyle->exts_antennaRatio[i].ratioDiff != INFINITY)
+			if (ExtCurStyle->exts_antennaRatio[i].ratioDiffB != INFINITY)
 			{
-			    if (ExtCurStyle->exts_antennaRatio[i].ratioDiff > 0)
-				anttotal += (double)antennaarea[i] /
-				    (double)ExtCurStyle->exts_antennaRatio[i].ratioDiff;
-			    if (ExtCurStyle->exts_antennaRatio[i].ratioDiff > saveRatio)
-				saveRatio = ExtCurStyle->exts_antennaRatio[i].ratioDiff;
+			    ratioTotal = ExtCurStyle->exts_antennaRatio[i].ratioDiffB +
+				diffarea * ExtCurStyle->exts_antennaRatio[i].ratioDiffA;
+
+			    if (ratioTotal > 0)
+				anttotal += (double)antennaarea[i] / (double)ratioTotal;
+			    if (ratioTotal > saveRatio)
+				saveRatio = ratioTotal;
 			}
 
 		    if (anttotal > (double)gatearea)
@@ -585,10 +597,10 @@ antennacheckVisit(dev, hierName, scale, trans, editUse)
 			    TxError("Effective antenna ratio %g > limit %g\n",
 				    saveRatio * (float)anttotal / (float)gatearea,
 				    saveRatio);
-			    TxError("Gate rect (%d %d) to (%d %d)\n", 
+			    TxError("Gate rect (%d %d) to (%d %d)\n",
 				    gdas.r.r_xbot, gdas.r.r_ybot,
 				    gdas.r.r_xtop, gdas.r.r_ytop);
-			    TxError("Antenna rect (%d %d) to (%d %d)\n", 
+			    TxError("Antenna rect (%d %d) to (%d %d)\n",
 				    aas.r.r_xbot, aas.r.r_ybot,
 				    aas.r.r_xtop, aas.r.r_ytop);
 			}
@@ -665,10 +677,11 @@ areaAccumFunc(tile, gdas)
     GateDiffAccumStruct *gdas;
 {
     Rect *rect = &(gdas->r);
-    int area, type;
+    int type;
+    dlong area;
 
     TiToRect(tile, rect);
-    area = (rect->r_xtop - rect->r_xbot) * (rect->r_ytop - rect->r_ybot);
+    area = (dlong)(rect->r_xtop - rect->r_xbot) * (dlong)(rect->r_ytop - rect->r_ybot);
     gdas->accum += area;
     return 0;
 }
@@ -692,9 +705,9 @@ antennaAccumFunc(tile, aaptr)
     AntennaAccumStruct *aaptr;
 {
     Rect *rect = &(aaptr->r);
-    int area;
+    dlong area;
     int type;
-    int *typeareas = aaptr->accum;
+    dlong *typeareas = aaptr->accum;
     int plane = aaptr->pNum;
     float thick;
 
@@ -702,7 +715,7 @@ antennaAccumFunc(tile, aaptr)
 
     TiToRect(tile, rect);
 
-    if (ExtCurStyle->exts_antennaModel & ANTENNAMODEL_SIDEWALL)
+    if (ExtCurStyle->exts_antennaRatio[type].areaType & ANTENNAMODEL_SIDEWALL)
     {
 	/* Accumulate perimeter of tile where tile abuts space */
 
@@ -771,7 +784,7 @@ antennaAccumFunc(tile, aaptr)
 		    if (DBTypeOnPlane(ttype, plane))
 		    {
 			thick = ExtCurStyle->exts_thick[ttype];
-			typeareas[ttype] += (int)((float)perimeter * thick);
+			typeareas[ttype] += (dlong)((float)perimeter * thick);
 		    }
 
 	    if (type >= DBNumUserLayers)
@@ -782,27 +795,28 @@ antennaAccumFunc(tile, aaptr)
 			if (DBTypeOnPlane(ttype, plane))
 			{
 			    thick = ExtCurStyle->exts_thick[ttype];
-			    typeareas[ttype] += (int)((float)perimeter * thick);
+			    typeareas[ttype] += (dlong)((float)perimeter * thick);
 			    break;
 			}
 	    }
 	    else
 	    {
 		thick = ExtCurStyle->exts_thick[type];
-		typeareas[type] += (int)((float)perimeter * thick);
+		typeareas[type] += (dlong)((float)perimeter * thick);
 	    }
 	}
 	else
 	{
 	    /* Area is perimeter times layer thickness */
 	    thick = ExtCurStyle->exts_thick[type];
-	    typeareas[type] += (int)((float)perimeter * thick);
+	    typeareas[type] += (dlong)((float)perimeter * thick);
 	}
     }
-    else
+    else if (ExtCurStyle->exts_antennaRatio[type].areaType & ANTENNAMODEL_SURFACE)
     {
 	/* Simple tile area calculation */
-	area = (rect->r_xtop - rect->r_xbot) * (rect->r_ytop - rect->r_ybot);
+	area = (dlong)(rect->r_xtop - rect->r_xbot)
+		* (dlong)(rect->r_ytop - rect->r_ybot);
 
 	/* If type is a contact, then add area to both residues as well	*/
 	/* as the contact type.						*/
