@@ -4,21 +4,21 @@
  * Procedures for building up the hierarchical representation
  * of a circuit.  These are all called from efReadDef() in EFread.c.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFbuild.c,v 1.6 2010/06/24 12:37:17 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header$";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -63,7 +63,9 @@ void efNodeMerge();
 bool efConnBuildName();
 bool efConnInitSubs();
 
-
+extern float locScale;
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -153,23 +155,37 @@ efBuildNode(def, isSubsnode, nodeName, nodeCap, x, y, layerName, av, ac)
 	if (efWarn)
 	    efReadError("Warning: duplicate node name %s\n", nodeName);
 
-	/* Just add to C, perim, area of existing node */
-	newnode = newname->efnn_node;
-	newnode->efnode_cap += (EFCapValue) nodeCap;
-	for (n = 0; n < efNumResistClasses && ac > 1; n++, ac -= 2)
+	/* newnode should exist if newname does.  Having a NULL node	*/
+	/* may be caused by detached labels "connected" to space.	*/
+
+	if ((newnode = newname->efnn_node) != NULL)
 	{
-	    newnode->efnode_pa[n].pa_area += atoi(*av++);
-	    newnode->efnode_pa[n].pa_perim += atoi(*av++);
+	    /* Just add to C, perim, area of existing node */
+	    newnode->efnode_cap += (EFCapValue) nodeCap;
+	    for (n = 0; n < efNumResistClasses && ac > 1; n++, ac -= 2)
+	    {
+		newnode->efnode_pa[n].pa_area += atoi(*av++);
+		newnode->efnode_pa[n].pa_perim += atoi(*av++);
+	    }
+
+	    /* If this node is identified as substrate, ensure that */
+	    /* the flag is set.					    */
+	    if (isSubsnode == TRUE)
+		newnode->efnode_flags |= EF_SUBS_NODE;
+    
+	    return;
 	}
-	return;
     }
 
-    /* Allocate a new node with 'nodeName' as its single name */
-    newname = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
-    newname->efnn_hier = EFStrToHN((HierName *) NULL, nodeName);
-    newname->efnn_port = -1;	/* No port assignment */
-    newname->efnn_next = NULL;
-    HashSetValue(he, (char *) newname);
+    if (!newname)
+    {
+	/* Allocate a new node with 'nodeName' as its single name */
+	newname = (EFNodeName *) mallocMagic((unsigned)(sizeof (EFNodeName)));
+	newname->efnn_hier = EFStrToHN((HierName *) NULL, nodeName);
+	newname->efnn_port = -1;	/* No port assignment */
+	newname->efnn_next = NULL;
+	HashSetValue(he, (char *) newname);
+    }
 
     /* New node itself */
     size = sizeof (EFNode) + (efNumResistClasses - 1) * sizeof (EFPerimArea);
@@ -177,11 +193,12 @@ efBuildNode(def, isSubsnode, nodeName, nodeCap, x, y, layerName, av, ac)
     newnode->efnode_flags = (isSubsnode == TRUE) ? EF_SUBS_NODE : 0;
     newnode->efnode_cap = nodeCap;
     newnode->efnode_attrs = (EFAttr *) NULL;
-    newnode->efnode_loc.r_xbot = x;
-    newnode->efnode_loc.r_ybot = y;
-    newnode->efnode_loc.r_xtop = x + 1;
-    newnode->efnode_loc.r_ytop = y + 1;
+    newnode->efnode_loc.r_xbot = (int)(0.5 + (float)x * locScale);
+    newnode->efnode_loc.r_ybot = (int)(0.5 + (float)y * locScale);
+    newnode->efnode_loc.r_xtop = newnode->efnode_loc.r_xbot + 1;
+    newnode->efnode_loc.r_ytop = newnode->efnode_loc.r_ybot + 1;
     newnode->efnode_client = (ClientData) NULL;
+    newnode->efnode_num = 1;
     if (layerName) newnode->efnode_type =
 	    efBuildAddStr(EFLayerNames, &EFLayerNumNames, MAXTYPES, layerName);
     else newnode->efnode_type = 0;
@@ -285,7 +302,7 @@ efBuildAttr(def, nodeName, r, layerName, text)
     ap->efa_next = nn->efnn_node->efnode_attrs;
     nn->efnn_node->efnode_attrs = ap;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -368,7 +385,7 @@ efBuildDist(def, driver, receiver, min, max)
 	HashSetValue(he, (ClientData) he->h_key.h_ptr);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -398,7 +415,7 @@ efBuildKill(def, name)
     kill->kill_next = def->def_kills;
     def->def_kills = kill;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -454,15 +471,21 @@ efBuildEquiv(def, nodeName1, nodeName2)
 			EFStrToHN((HierName *) NULL, nodeName2));
 	return;
     }
+    else if (nn2->efnn_node == (EFNode *)NULL)
+	return;		/* Repeated "equiv" statement */
 
     /* If both names exist and are for different nodes, merge them */
     if (nn1)
     {
+	if (nn1->efnn_node == (EFNode *)NULL)
+	    return;		/* Repeated "equiv" statement */
 	if (nn1->efnn_node != nn2->efnn_node)
 	{
 	    if (efWarn)
 		efReadError("Merged nodes %s and %s\n", nodeName1, nodeName2);
-	    efNodeMerge(nn1->efnn_node, nn2->efnn_node);
+	    efNodeMerge(&nn1->efnn_node, &nn2->efnn_node);
+	    if (nn1->efnn_port > 0) nn2->efnn_port = nn1->efnn_port;
+	    else if (nn2->efnn_port > 0) nn1->efnn_port = nn2->efnn_port;
 	}
 	return;
     }
@@ -471,7 +494,7 @@ efBuildEquiv(def, nodeName1, nodeName2)
     efNodeAddName(nn2->efnn_node, he1,
 			EFStrToHN((HierName *) NULL, nodeName1));
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -527,7 +550,7 @@ efBuildDeviceParams(name, argc, argv)
 	pptr = strchr(argv[n], '=');
 	if (pptr == NULL)
 	{
-	    efReadError("Bad parameter assignment \"%s\" for device \"%s\"\n", 
+	    efReadError("Bad parameter assignment \"%s\" for device \"%s\"\n",
 			argv[n], name);
 	    continue;
 	}
@@ -606,7 +629,7 @@ efBuildDevice(def, class, type, r, argc, argv)
     Dev *newdev, devtmp;
     DevParam *newparm, *devp, *sparm;
     char ptype, *pptr, **av;
-    char devhash[24];
+    char devhash[64];
     int argstart = 1;	/* start of terminal list in argv[] */
     bool hasModel = strcmp(type, "None") ? TRUE : FALSE;
 
@@ -686,7 +709,8 @@ efBuildDevice(def, class, type, r, argc, argv)
 		{
 		    pn = *(argv[argstart] + 1) - '0';
 		    if (pn == 0)
-			devtmp.dev_area = atoi(pptr);
+			devtmp.dev_area = (int)(0.5 + (float)atoi(pptr)
+				* locScale * locScale);
 		    /* Otherwise, punt */
 		}
 		break;
@@ -697,15 +721,15 @@ efBuildDevice(def, class, type, r, argc, argv)
 		{
 		    pn = *(argv[argstart] + 1) - '0';
 		    if (pn == 0)
-			devtmp.dev_perim = atoi(pptr);
+			devtmp.dev_perim = (int)(0.5 + (float)atoi(pptr) * locScale);
 		    /* Otherwise, use verbatim */
 		}
 		break;
 	    case 'l':
-		devtmp.dev_length = atoi(pptr);
+		devtmp.dev_length = (int)(0.5 + (float)atoi(pptr) * locScale);
 		break;
 	    case 'w':
-		devtmp.dev_width = atoi(pptr);
+		devtmp.dev_width = (int)(0.5 + (float)atoi(pptr) * locScale);
 		break;
 	    case 'c':
 		devtmp.dev_cap = (float)atof(pptr);
@@ -749,7 +773,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 
     /* Determine if this device has been seen before */
 
-    sprintf(devhash, "%dx%d", r->r_xbot, r->r_ybot);
+    sprintf(devhash, "%dx%d%s", r->r_xbot, r->r_ybot, type);
     he = HashFind(&def->def_devs, devhash);
     newdev = (Dev *)HashGetValue(he);
     if (newdev)
@@ -799,7 +823,7 @@ efBuildDevice(def, class, type, r, argc, argv)
         newdev->dev_rect = *r;
         newdev->dev_type = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, type);
         newdev->dev_class = class;
- 
+
         switch (class)
         {
 	    case DEV_FET:		/* old-style "fet" record */
@@ -961,7 +985,7 @@ efBuildPortNode(def, name, idx, x, y, layername, toplevel)
     if (nn != (EFNodeName *) NULL)
     {
 	nn->efnn_node->efnode_flags |= EF_PORT;
-	if (toplevel) 
+	if (toplevel)
 	    nn->efnn_node->efnode_flags |= EF_TOP_PORT;
 	nn->efnn_port = idx;
     }
@@ -1073,7 +1097,7 @@ efBuildDevNode(def, name, isSubsNode)
     }
     return nn->efnn_node;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1117,7 +1141,7 @@ efBuildAddStr(table, pMax, size, str)
 
     return max;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1206,7 +1230,7 @@ efBuildUse(def, subDefName, subUseId, ta, tb, tc, td, te, tf)
         TxError("Warning: use %s appears more than once in def!\n", newuse->use_id);
     HashSetValue(he, (ClientData)newuse);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1248,15 +1272,16 @@ efBuildConnect(def, nodeName1, nodeName2, deltaC, av, ac)
 	conn->conn_next = def->def_conns;
 	for (n = 0; n < efNumResistClasses && ac > 1; n++, ac -= 2)
 	{
-	    conn->conn_pa[n].pa_area = atoi(*av++);
-	    conn->conn_pa[n].pa_perim = atoi(*av++);
+	    conn->conn_pa[n].pa_area = (int)(0.5 + (float)atoi(*av++)
+		    * locScale * locScale);
+	    conn->conn_pa[n].pa_perim = (int)(0.5 + (float)atoi(*av++) * locScale);
 	}
 	for ( ; n < efNumResistClasses; n++)
 	    conn->conn_pa[n].pa_area = conn->conn_pa[n].pa_perim = 0;
 	def->def_conns = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1293,7 +1318,7 @@ efBuildResistor(def, nodeName1, nodeName2, resistance)
 	def->def_resistors = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1330,7 +1355,7 @@ efBuildCap(def, nodeName1, nodeName2, cap)
 	def->def_caps = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1385,7 +1410,7 @@ bad:
     freeMagic((char *) conn);
     return FALSE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1491,7 +1516,7 @@ again:
     cnp->cn_nsubs = nsubs;
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1551,7 +1576,7 @@ efNodeAddName(node, he, hn)
 	oldnn->efnn_next = newnn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1576,70 +1601,86 @@ efNodeAddName(node, he, hn)
  */
 
 void
-efNodeMerge(node1, node2)
-    EFNode *node1, *node2;	/* Hierarchical nodes */
+efNodeMerge(node1ptr, node2ptr)
+    EFNode **node1ptr, **node2ptr;	/* Pointers to hierarchical nodes */
 {
     EFNodeName *nn, *nnlast;
     EFAttr *ap;
     int n;
+    EFNode *keeping, *removing;
 
     /* Sanity check: ignore if same node */
-    if (node1 == node2)
+    if (*node1ptr == *node2ptr)
 	return;
 
+    /* Keep the node with the greater number of entries, and merge  */
+    /* the node with fewer entries into it.			    */
+
+    if ((*node1ptr)->efnode_num >= (*node2ptr)->efnode_num)
+    {
+	keeping = *node1ptr;
+	removing = *node2ptr;
+    }
+    else
+    {
+	keeping = *node2ptr;
+	removing = *node1ptr;
+    }
+
     if (efWatchNodes)
     {
-	if (HashLookOnly(&efWatchTable, (char *) node1->efnode_name->efnn_hier)
-	    || (node2->efnode_name
+	if (HashLookOnly(&efWatchTable, (char *) keeping->efnode_name->efnn_hier)
+	    || (removing->efnode_name
 		&& HashLookOnly(&efWatchTable,
-				(char *) node2->efnode_name->efnn_hier)))
+				(char *) removing->efnode_name->efnn_hier)))
 	{
 	    printf("\ncombine: %s\n",
-		EFHNToStr(node1->efnode_name->efnn_hier));
-	    printf("  with   %s\n\n", 
-		node2->efnode_name
-		    ? EFHNToStr(node2->efnode_name->efnn_hier)
+		EFHNToStr(keeping->efnode_name->efnn_hier));
+	    printf("  with   %s\n\n",
+		removing->efnode_name
+		    ? EFHNToStr(removing->efnode_name->efnn_hier)
 		    : "(unnamed)");
 	}
     }
 
     /* Sum capacitances, perimeters, areas */
-    node1->efnode_cap += node2->efnode_cap;
+    keeping->efnode_cap += removing->efnode_cap;
     for (n = 0; n < efNumResistClasses; n++)
     {
-	node1->efnode_pa[n].pa_area += node2->efnode_pa[n].pa_area;
-	node1->efnode_pa[n].pa_perim += node2->efnode_pa[n].pa_perim;
+	keeping->efnode_pa[n].pa_area += removing->efnode_pa[n].pa_area;
+	keeping->efnode_pa[n].pa_perim += removing->efnode_pa[n].pa_perim;
     }
 
-    /* Make all EFNodeNames point to node1 */
-    if (node2->efnode_name)
+    /* Make all EFNodeNames point to "keeping" */
+    if (removing->efnode_name)
     {
-	bool topport1, topport2;
+	bool topportk, topportr;
 
-	for (nn = node2->efnode_name; nn; nn = nn->efnn_next)
+	for (nn = removing->efnode_name; nn; nn = nn->efnn_next)
 	{
 	    nnlast = nn;
-	    nn->efnn_node = node1;
+	    nn->efnn_node = keeping;
 	}
 
-	topport1 = (node1->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
-	topport2 = (node2->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
+	topportk = (keeping->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
+	topportr = (removing->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
 
 	/* Concatenate list of EFNodeNames, taking into account precedence */
-	if (!topport1 && (topport2 || EFHNBest(node2->efnode_name->efnn_hier,
-		     node1->efnode_name->efnn_hier)))
+	if ((!keeping->efnode_name) || (!topportk && (topportr
+		    || EFHNBest(removing->efnode_name->efnn_hier,
+		     keeping->efnode_name->efnn_hier))))
 	{
 	    /*
-	     * New official name is that of node2.
+	     * New official name is that of "removing".
 	     * The new list of names is:
-	     *	node2-names, node1-names
+	     *	removing-names, keeping-names
 	     */
-	    nnlast->efnn_next = node1->efnode_name;
-	    node1->efnode_name = node2->efnode_name;
+	    nnlast->efnn_next = keeping->efnode_name;
+	    keeping->efnode_name = removing->efnode_name;
 
 	    /*
-	     * Choose the new location only if node2's location is a valid one,
-	     * i.e, node2 wasn't created before it was mentioned.  This is mainly
+	     * Choose the new location only if "removing"'s location is a valid one,
+	     * i.e, "removing" wasn't created before it was mentioned.  This is mainly
 	     * to deal with new fets, resistors, and capacitors created by resistance
 	     * extraction, which appear with their full hierarchical names in the
 	     * .ext file for the root cell.
@@ -1654,18 +1695,10 @@ efNodeMerge(node1, node2)
 	     *
 	     *					Tim, 6/14/04
 	     */
-	    if (node2->efnode_type > 0)
+	    if (removing->efnode_type > 0)
 	    {
-		node1->efnode_loc = node2->efnode_loc;
-		node1->efnode_type = node2->efnode_type;
-
-		if (node2->efnode_loc.r_ybot < node1->efnode_loc.r_ybot
-		    || (node2->efnode_loc.r_ybot == node1->efnode_loc.r_ybot
-		    && node2->efnode_loc.r_xbot < node1->efnode_loc.r_xbot))
-		{
-//		    node1->efnode_loc = node2->efnode_loc;
-//		    node1->efnode_type = node2->efnode_type;
-		}
+		keeping->efnode_loc = removing->efnode_loc;
+		keeping->efnode_type = removing->efnode_type;
 	    }
 	}
 	else
@@ -1673,54 +1706,61 @@ efNodeMerge(node1, node2)
 	    /*
 	     * Keep old official name.
 	     * The new list of names is:
-	     *	node1-names[0], node2-names, node1-names[1-]
+	     *	keeping-names[0], removing-names, keeping-names[1-]
 	     */
-	    nnlast->efnn_next = node1->efnode_name->efnn_next;
-	    node1->efnode_name->efnn_next = node2->efnode_name;
+	    nnlast->efnn_next = keeping->efnode_name->efnn_next;
+	    keeping->efnode_name->efnn_next = removing->efnode_name;
 	}
     }
 
+    /* Merge list counts */
+    keeping->efnode_num += removing->efnode_num;
+
     /* Merge attribute lists */
-    if (ap = node2->efnode_attrs)
+    if (ap = removing->efnode_attrs)
     {
 	while (ap->efa_next)
 	    ap = ap->efa_next;
-	ap->efa_next = node1->efnode_attrs;
-	node1->efnode_attrs = ap;
-	node2->efnode_attrs = (EFAttr *) NULL;	/* Sanity */
+	ap->efa_next = keeping->efnode_attrs;
+	keeping->efnode_attrs = ap;
+	removing->efnode_attrs = (EFAttr *) NULL;	/* Sanity */
     }
 
-    /* Unlink node2 from list for def */
-    node2->efnode_prev->efnhdr_next = node2->efnode_next;
-    node2->efnode_next->efnhdr_prev = node2->efnode_prev;
+    /* Unlink "removing" from list for def */
+    removing->efnode_prev->efnhdr_next = removing->efnode_next;
+    removing->efnode_next->efnhdr_prev = removing->efnode_prev;
 
     /*
      * Only if both nodes were EF_DEVTERM do we keep EF_DEVTERM set
      * in the resultant node.
      */
-    if ((node2->efnode_flags & EF_DEVTERM) == 0)
-	node1->efnode_flags &= ~EF_DEVTERM;
+    if ((removing->efnode_flags & EF_DEVTERM) == 0)
+	keeping->efnode_flags &= ~EF_DEVTERM;
 
     /*
-     * If node2 has the EF_PORT flag set, then copy the port
+     * If "removing" has the EF_PORT flag set, then copy the port
      * record in the flags to node1.
      */
-    if (node2->efnode_flags & EF_PORT)
-	node1->efnode_flags |= EF_PORT;
-    if (node2->efnode_flags & EF_TOP_PORT)
-	node1->efnode_flags |= EF_TOP_PORT;
+    if (removing->efnode_flags & EF_PORT)
+	keeping->efnode_flags |= EF_PORT;
+    if (removing->efnode_flags & EF_TOP_PORT)
+	keeping->efnode_flags |= EF_TOP_PORT;
 
     /*
-     * If node2 has the EF_SUBS_NODE flag set, then copy the port
-     * record in the flags to node1.
+     * If "removing" has the EF_SUBS_NODE flag set, then copy the port
+     * record in the flags to "keeping".
      */
-    if (node2->efnode_flags & EF_SUBS_NODE)
-	node1->efnode_flags |= EF_SUBS_NODE;
+    if (removing->efnode_flags & EF_SUBS_NODE)
+	keeping->efnode_flags |= EF_SUBS_NODE;
 
-    /* Get rid of node2 */
-    freeMagic((char *) node2);
+    /* Get rid of "removing" */
+    freeMagic((char *) removing);
+
+    /* Make sure that the active node is always node1 */
+    *node1ptr = keeping;
+    *node2ptr = (EFNode *)NULL;	    /* Sanity check */
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1824,7 +1864,7 @@ efFreeNodeTable(table)
 	    freeMagic((char *) nn);
 	}
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1858,7 +1898,7 @@ efFreeNodeList(head)
 	freeMagic((char *) node);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
