@@ -4,16 +4,16 @@
  * Procedures to flatten the hierarchical description built
  * by efReadDef().
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -59,9 +59,15 @@ bool efFlatGlobCmp(HierName *, HierName *);
 char *efFlatGlobCopy(HierName *);
 void efFlatGlobError(EFNodeName *, EFNodeName *);
 int efAddNodes(HierContext *, bool);
-int efAddOneConn(HierContext *, char *, char *, Connection *);
+int efAddConns(HierContext *, bool);
+int efAddOneConn(HierContext *, char *, char *, Connection *, bool);
+
+/* Flags passed to efFlatNode() */
+
+#define FLATNODE_STDCELL    0x01
+#define FLATNODE_DOWARN	    0x02
+
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -126,7 +132,10 @@ EFFlatBuild(name, flags)
 	if (flags & EF_NOFLATSUBCKT)
 	    efFlatNodesStdCell(&efFlatContext);
 	else
-	    efFlatNodes(&efFlatContext);
+	{
+	    int flags = FLATNODE_DOWARN;    /* No FLATNODE_STDCELL flag */
+	    efFlatNodes(&efFlatContext, (ClientData)flags);
+	}
 	efFlatKills(&efFlatContext);
 	if (!(flags & EF_NONAMEMERGE))
 	    efFlatGlob();
@@ -165,6 +174,7 @@ EFFlatBuildOneLevel(def, flags)
     Use *use;
     int efFlatNodesDeviceless();	/* Forward declaration */
     int efFlatCapsDeviceless();		/* Forward declaration */
+    int flatnodeflags;
 
     efFlatRootDef = def;
 
@@ -193,8 +203,9 @@ EFFlatBuildOneLevel(def, flags)
     efFlatContext.hc_x = efFlatContext.hc_y = 0;
     efFlatRootUse.use_def = efFlatRootDef;
 
-    /* Record all nodes of the next level in the hierarchy */
-    efHierSrUses(&efFlatContext, efAddNodes, (ClientData)TRUE);
+    /* Record all nodes down the hierarchy from here */
+    flatnodeflags = FLATNODE_STDCELL;	/* No FLATDNODE_DOWARN flag */
+    efFlatNodes(&efFlatContext, (ClientData)flatnodeflags);
 
     /* Expand all subcells that contain connectivity information but	*/
     /* no active devices (including those in subcells).			*/
@@ -210,7 +221,7 @@ EFFlatBuildOneLevel(def, flags)
 
     /* Record all local nodes */
     efAddNodes(&efFlatContext, FALSE);
-    efAddConns(&efFlatContext);
+    efAddConns(&efFlatContext, TRUE);
 
     efFlatKills(&efFlatContext);
     if (!(flags & EF_NONAMEMERGE))
@@ -222,7 +233,7 @@ EFFlatBuildOneLevel(def, flags)
 
     return &efFlatContext;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -257,7 +268,7 @@ EFFlatDone()
     HashKill(&efHNUseHashTable);
     return;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -291,16 +302,22 @@ EFFlatDone()
  */
 
 int
-efFlatNodes(hc)
+efFlatNodes(hc, clientData)
     HierContext *hc;
+    ClientData clientData;
 {
-    (void) efHierSrUses(hc, efFlatNodes);
+    int flags = (int)clientData;
+
+    bool stdcell = (flags & FLATNODE_STDCELL) ? TRUE : FALSE;
+    bool doWarn = (flags & FLATNODE_DOWARN) ? TRUE : FALSE;
+
+    (void) efHierSrUses(hc, efFlatNodes, clientData);
 
     /* Add all our own nodes to the table */
-    efAddNodes(hc, FALSE);
+    efAddNodes(hc, stdcell);
 
     /* Process our own connections and adjustments */
-    (void) efAddConns(hc);
+    (void) efAddConns(hc, doWarn);
 
     return (0);
 }
@@ -352,7 +369,7 @@ efFlatNodesStdCell(hc)
 
     /* Process our own connections and adjustments */
     if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
-	(void) efAddConns(hc);
+	(void) efAddConns(hc, TRUE);
 
     return (0);
 }
@@ -378,7 +395,7 @@ efFlatNodesDeviceless(hc, cdata)
 	efAddNodes(hc, TRUE);
 
 	/* Process our own connections and adjustments */
-	efAddConns(hc);
+	efAddConns(hc, TRUE);
 
 	/* Mark this definition as having no devices, so it will not be visited */
 	hc->hc_use->use_def->def_flags |= DEF_NODEVICES;
@@ -460,11 +477,12 @@ efAddNodes(hc, stdcell)
 
 	// If called with "hierarchy on", all local node caps and adjustments
 	// have been output and should be ignored.
-	
+
 	newnode->efnode_cap = (!stdcell) ? node->efnode_cap : (EFCapValue)0.0;
 	newnode->efnode_client = (ClientData) NULL;
 	newnode->efnode_flags = node->efnode_flags;
 	newnode->efnode_type = node->efnode_type;
+	newnode->efnode_num = 1;
 	if (!stdcell)
 	    bcopy((char *) node->efnode_pa, (char *) newnode->efnode_pa,
 			efNumResistClasses * sizeof (EFPerimArea));
@@ -513,7 +531,7 @@ efAddNodes(hc, stdcell)
 		if (hierName != nn->efnn_hier)
 		    EFHNFree(hierName, hc->hc_hierName, HN_CONCAT);
 		if (oldname->efnn_node != newnode)
-		    efNodeMerge(oldname->efnn_node, newnode);
+		    efNodeMerge(&oldname->efnn_node, &newnode);
 		newnode = oldname->efnn_node;
 		continue;
 	    }
@@ -562,8 +580,9 @@ efAddNodes(hc, stdcell)
  */
 
 int
-efAddConns(hc)
+efAddConns(hc, doWarn)
     HierContext *hc;
+    bool doWarn;
 {
     Connection *conn;
 
@@ -576,14 +595,14 @@ efAddConns(hc)
     {
 	/* Special case for speed when no array info is present */
 	if (conn->conn_1.cn_nsubs == 0)
-	    efAddOneConn(hc, conn->conn_name1, conn->conn_name2, conn);
+	    efAddOneConn(hc, conn->conn_name1, conn->conn_name2, conn, doWarn);
 	else
-	    efHierSrArray(hc, conn, efAddOneConn, (ClientData) NULL);
+	    efHierSrArray(hc, conn, efAddOneConn, (ClientData)doWarn);
     }
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -605,16 +624,17 @@ efAddConns(hc)
  */
 
 int
-efAddOneConn(hc, name1, name2, conn)
+efAddOneConn(hc, name1, name2, conn, doWarn)
     HierContext *hc;
     char *name1, *name2;	/* These are strings, not HierNames */
     Connection *conn;
+    bool doWarn;
 {
     HashEntry *he1, *he2;
     EFNode *node, *newnode;
     int n;
 
-    he1 = EFHNLook(hc->hc_hierName, name1, "connect(1)");
+    he1 = EFHNLook(hc->hc_hierName, name1, (doWarn) ? "connect(1)" : NULL);
     if (he1 == NULL)
 	return 0;
 
@@ -630,17 +650,17 @@ efAddOneConn(hc, name1, name2, conn)
     /* Merge this node with conn_name2 if one was specified */
     if (name2)
     {
-	he2 = EFHNLook(hc->hc_hierName, name2, "connect(2)");
+	he2 = EFHNLook(hc->hc_hierName, name2, (doWarn) ? "connect(2)" : NULL);
 	if (he2 == NULL)
 	    return 0;
 	newnode = ((EFNodeName *) HashGetValue(he2))->efnn_node;
 	if (node != newnode)
-	    efNodeMerge(node, newnode);
+	    efNodeMerge(&node, &newnode);
     }
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -741,7 +761,7 @@ efFlatGlob()
 	    {
 		efFlatGlobError(nameGlob, nameFlat);
 	    }
-	    efNodeMerge(nodeFlat, nodeGlob);
+	    efNodeMerge(&nodeFlat, &nodeGlob);
 	    nameGlob->efnn_node = nodeFlat;
 	}
     }
@@ -851,7 +871,7 @@ efFlatGlobHash(hierName)
 {
     return hierName->hn_hash;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -895,7 +915,7 @@ efFlatKills(hc)
 
     return (0);
 }
-
+
 
 /*----
  * WIP
@@ -987,7 +1007,7 @@ efFlatCaps(hc)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1022,7 +1042,7 @@ efFlatSingleCap(hc, name1, name2, conn)
     static char msg0[] = "cap(1)";
     static char msg1[] = "cap(2)";
     char *msg;
-    
+
     /* Connections that are below threshold (ext2spice hierarchy only)	*/
     /* will be missing.  Do not generate errors for these.		*/
 
@@ -1060,7 +1080,7 @@ efFlatSingleCap(hc, name1, name2, conn)
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1147,14 +1167,14 @@ HashEntry *he;
  * CapHashSetValue()
  *       if the pointer is null, allocate a EFCapValue and point to it.
  *        Then copy in the new value.
- * 
+ *
  * need to pass doubles regardless of what CapValue is because of
  * argument promotion in ANSI C
  *
  */
 void
 CapHashSetValue(he, c)
-HashEntry *he; 
+HashEntry *he;
 double c;
 {
 	EFCapValue *capp = (EFCapValue *)HashGetValue(he);
