@@ -4,16 +4,16 @@
  * Procedures to read a .ext file and call the procedures
  * in EFbuild.c to build up a description of each def.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -38,6 +38,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
 #include "extract/extract.h"
+#include "extract/extractInt.h"
 #include "utils/paths.h"
 
 #ifndef MAGIC_WRAPPER
@@ -96,6 +97,7 @@ keyTable[] =
 /* Data shared with EFerror.c */
 char *efReadFileName;	/* Name of file currently being read */
 int efReadLineNum;	/* Current line number in above file */
+float locScale;		/* Multiply values in the file by this on read-in */
 
 /* Data local to this file */
 static bool efReadDef();
@@ -103,7 +105,7 @@ static bool efReadDef();
 /* atoCap - convert a string to a EFCapValue */
 #define	atoCap(s)	((EFCapValue)atof(s))
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -139,6 +141,7 @@ EFReadFile(name, dosubckt, resist, noscale)
     if (def == NULL)
 	def = efDefNew(name);
 
+    locScale = 1.0;
     rc = efReadDef(def, dosubckt, resist, noscale, TRUE);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
     if (EFScale == 0.0) EFScale = 1.0;
@@ -174,7 +177,8 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
     int argc, ac, n;
     CellDef *dbdef;
     EFCapValue cap;
-    char line[1024], *argv[128], *name, *attrs;
+    char *line = NULL, *argv[128], *name, *attrs;
+    int size = 0;
     int rscale = 1;	/* Multiply resistances by this */
     int cscale = 1;	/* Multiply capacitances by this */
     float lscale = 1.0;	/* Multiply lambda by this */
@@ -204,7 +208,7 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
 	    if (sptr) {
 		*sptr = '\0';
 		inf = PaOpen(name, "r", ".ext", filepath, EFLibPath, &efReadFileName);
-	    }	
+	    }
 	    freeMagic(filepath);
 	}
     }
@@ -234,7 +238,7 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
 
 readfile:
     efReadLineNum = 0;
-    while ((argc = efReadLine(line, sizeof line, inf, argv)) >= 0)
+    while ((argc = efReadLine(&line, &size, inf, argv)) >= 0)
     {
 	n = LookupStruct(argv[0], (LookupTable *) keyTable, sizeof keyTable[0]);
 	if (n < 0)
@@ -265,6 +269,11 @@ readfile:
 		    cscale = 1;
 		}
 		lscale = (float)atof(argv[3]);
+		if (lscale != ExtCurStyle->exts_unitsPerLambda)
+		{
+		    locScale = lscale / ExtCurStyle->exts_unitsPerLambda;
+		    lscale = ExtCurStyle->exts_unitsPerLambda;
+		}
 		if (lscale == 0.0)
 		{
 		    efReadError("Bad linear scaling = 0; reset to 1.\n");
@@ -283,10 +292,10 @@ readfile:
 
 	    /* attr node xlo ylo xhi yhi type text */
 	    case ATTR:
-		r.r_xbot = atoi(argv[2]);
-		r.r_ybot = atoi(argv[3]);
-		r.r_xtop = atoi(argv[4]);
-		r.r_ytop = atoi(argv[5]),
+		r.r_xbot = (int)(0.5 + (float)atoi(argv[2]) * locScale);
+		r.r_ybot = (int)(0.5 + (float)atoi(argv[3]) * locScale);
+		r.r_xtop = (int)(0.5 + (float)atoi(argv[4]) * locScale);
+		r.r_ytop = (int)(0.5 + (float)atoi(argv[5]) * locScale),
 		efBuildAttr(def, argv[1], &r, argv[6], argv[7]);
 		break;
 
@@ -351,10 +360,10 @@ readfile:
 			break;	/* we will deal with in efBuildDevice().   */
 		}
 
-		r.r_xbot = atoi(argv[3]);
-		r.r_ybot = atoi(argv[4]);
-		r.r_xtop = atoi(argv[5]);
-		r.r_ytop = atoi(argv[6]);
+		r.r_xbot = (int)(0.5 + (float)atoi(argv[3]) * locScale);
+		r.r_ybot = (int)(0.5 + (float)atoi(argv[4]) * locScale);
+		r.r_xtop = (int)(0.5 + (float)atoi(argv[5]) * locScale);
+		r.r_ytop = (int)(0.5 + (float)atoi(argv[6]) * locScale);
 
 		if (efBuildDevice(def, (char)n, argv[2], &r, argc - 7, &argv[7]) != 0)
 		{
@@ -366,10 +375,10 @@ readfile:
 	    /* for backwards compatibility */
 	    /* fet type xlo ylo xhi yhi area perim substrate GATE T1 T2 ... */
 	    case FET:
-		r.r_xbot = atoi(argv[2]);
-		r.r_ybot = atoi(argv[3]);
-		r.r_xtop = atoi(argv[4]);
-		r.r_ytop = atoi(argv[5]);
+		r.r_xbot = (int)(0.5 + (float)atoi(argv[2]) * locScale);
+		r.r_ybot = (int)(0.5 + (float)atoi(argv[3]) * locScale);
+		r.r_xtop = (int)(0.5 + (float)atoi(argv[4]) * locScale);
+		r.r_ytop = (int)(0.5 + (float)atoi(argv[5]) * locScale);
 		if (efBuildDevice(def, DEV_FET, argv[1], &r, argc - 6, &argv[6]) != 0)
 		{
 		    efReadError("Incomplete terminal description for fet\n");
@@ -547,7 +556,7 @@ resistChanged:
 		}
 #endif
 		else
-		    EFStyle = StrDup(&EFStyle, argv[1]);
+		    StrDup(&EFStyle, argv[1]);
 		break;
 
 	    /* version version-number */
@@ -563,8 +572,8 @@ resistChanged:
 	    /* distance driver receiver min max */
 	    case DIST:
 		efBuildDist(def, argv[1], argv[2],
-			(int)(lscale*atoi(argv[3])),
-			(int)(lscale*atoi(argv[4])));
+			(int)(lscale*atoi(argv[3])*locScale),
+			(int)(lscale*atoi(argv[4])*locScale));
 		break;
 
 	    /* killnode nodename */
@@ -573,8 +582,12 @@ resistChanged:
 		break;
 
 	    /* resistor node1 node2 resistance */
+	    /* NOTE:  Value changed to floating-point 12/16/2019;     */
+	    /* (value * rscale) is in milliohms which is integer as   */
+	    /* resolution finer than milliohms is deemed unnecessary. */
 	    case RESISTOR:
-		efBuildResistor(def, argv[1], argv[2], rscale*atoi(argv[3]));
+		efBuildResistor(def, argv[1], argv[2],
+			    (int)(0.5 + (double)rscale * atof(argv[3])));
 		break;
 
 	    /* abstract (no options/arguments) */
@@ -602,6 +615,7 @@ resistChanged:
 	if (inf != NULL)
 	    goto readfile;
     }
+    if (line != NULL) freeMagic(line);
 
     /* If we are considering standard cells, only the first level of	*/
     /* subcircuits is meaningful.					*/
@@ -610,7 +624,7 @@ resistChanged:
 	DoSubCircuit = FALSE;
 
     /* Read in each def that has not yet been read in */
-    
+
     HashStartSearch(&hs);
     while (he = HashNext(&def->def_uses, &hs))
     {
@@ -623,7 +637,7 @@ resistChanged:
 
     return rc;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -648,37 +662,58 @@ resistChanged:
  */
 
 int
-efReadLine(line, size, file, argv)
-    char *line;			/* Character array into which line is read */
-    int size;			/* Size of character array */
-    FILE *file;	/* Open .ext file */
-    char *argv[];		/* Vector of tokens built by efReadLine() */
+efReadLine(lineptr, sizeptr, file, argv)
+    char **lineptr;	/* Pointer to character array into which line is read */
+    int *sizeptr;	/* Pointer to size of character array */
+    FILE *file;		/* Open .ext file */
+    char *argv[];	/* Vector of tokens built by efReadLine() */
 {
     char *get, *put;
     bool inquote;
     int argc = 0;
 
+    if (*sizeptr == 0)
+    {
+	*sizeptr = 1024;
+	*lineptr = (char *)mallocMagic(*sizeptr);
+    }
+    int size = *sizeptr;
+
     /* Read one line into the buffer, joining lines when they end in '\' */
 start:
-     get = line;
-     while (size > 0)
+     get = *lineptr;
+     while (TRUE)
      {
-	efReadLineNum += 1;
+	efReadLineNum++;
 	if (fgets(get, size, file) == NULL) return (-1);
-	for (put = get; *put != '\n'; put++) size -= 1;
-	if ((put != get) && (*(put-1) == '\\'))
+	for (put = get; *put != '\n' && *put != '\0'; put++) size--;
+	if ((put != get) && (*(put - 1) == '\\'))
 	{
-	    get = put-1;
+	    get = put - 1;
 	    continue;
 	}
-	*put= '\0';
-	break;
+	*put = '\0';
+
+	if (size <= 1)
+	{
+	    char *newline;
+	    int i;
+
+	    *sizeptr += 1024;	/* Increase buffer size in 1024 byte increments */
+	    newline = (char *)mallocMagic(*sizeptr);
+	    strcpy(newline, *lineptr);
+	    put = (put - (*lineptr)) + newline;
+	    get = put;
+	    freeMagic(*lineptr);
+	    *lineptr = newline;
+	    size = 1024;
+	    efReadLineNum--;
+	}
+	else break;
     }
-    if (size == 0) efReadError("long line truncated\n");
+    get = put = *lineptr;
 
-    get = put = line;
-
-    if (*line == '#') goto start;	/* Ignore comments */
+    if (*(*lineptr) == '#') goto start;	/* Ignore comments */
 
     while (*get != '\0')
     {
@@ -716,11 +751,15 @@ start:
 		}
 	    }
 
-	    if (*get == '\\')	/* Process quoted characters literally */
-	    {
-		get++;
-		if (*get == '\0') break;
-	    }
+	    /* Process backslash characters literally unless they   */
+	    /* are followed by the end-of-line.			    */
+
+	    if (*get == '\\')
+		if (*(get + 1) == '\0')
+		{
+		    get++;
+		    break;
+		}
 
 	    /* Copy into token receiving area */
 	    *put++ = *get++;
@@ -732,7 +771,7 @@ start:
 	 */
 	if (get == argv[argc])
 	    break;
-	
+
 	/* Terminate the token and advance over the terminating character. */
 
 	if (*get != '\0') get++;	/* Careful!  could be at end of line! */
