@@ -9,16 +9,16 @@
  *
  * The procedures in this file are not re-entrant.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -28,9 +28,11 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/time.h>
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
+#include "tcltk/tclmagic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -41,6 +43,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "debug/debug.h"
 #include "extract/extract.h"
 #include "extract/extractInt.h"
+#include "graphics/graphics.h"
 #include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -93,7 +96,27 @@ void extSubtreeAdjustInit();
 void extSubtreeOutputCoupling();
 void extSubtreeHardSearch();
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extClearUseFlags --
+ *
+ *  Callback function to clear the CU_SUB_EXTRACTED flag from each child
+ *  use of a CellDef.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extClearUseFlags(use, clientData)
+    CellUse *use;
+    ClientData clientData;
+{
+    use->cu_flags &= ~CU_SUB_EXTRACTED;
+    return 0;
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -145,6 +168,11 @@ extSubtree(parentUse, reg, f)
     bool result;
     int cuts, totcuts;
     float pdone, plast;
+    SearchContext scx;
+
+    /* Use the display timer to force a 5-second progress check */
+    GrDisplayStatus = DISPLAY_IN_PROGRESS;
+    SigSetTimer(5);		    /* Print at 5-second intervals */
 
     if ((ExtOptions & (EXT_DOCOUPLING|EXT_DOADJUST))
 		   != (EXT_DOCOUPLING|EXT_DOADJUST))
@@ -175,10 +203,10 @@ extSubtree(parentUse, reg, f)
      * halo has been set above to reflect the maximum distance for
      * sidewall coupling capacitance).
      */
-
     b = &def->cd_bbox;
 
-    /* Monitor progress, for large designs */
+    /* Monitor progress, for large designs, and allow display refresh at intervals */
+
     totcuts = (b->r_ytop - b->r_ybot + ExtCurStyle->exts_stepSize - 1)
 		    / ExtCurStyle->exts_stepSize;
     totcuts *= ((b->r_xtop - b->r_xbot + ExtCurStyle->exts_stepSize - 1)
@@ -214,7 +242,7 @@ extSubtree(parentUse, reg, f)
 			// If result == FALSE then ha.ha_interArea is invalid.
 			ha.ha_interArea = rlab;
 			result = TRUE;
-		    }	
+		    }
 		    else
 		        result |= GeoIncludeAll(&rlab, &ha.ha_interArea);
 		}
@@ -231,9 +259,8 @@ extSubtree(parentUse, reg, f)
 	    {
 		/* Make sure substrate connections have been handled	*/
 		/* even if there were no other interactions found.	*/
-		SearchContext scx;
 
-		GEOCLIP(&ha.ha_clipArea, &r);
+		ha.ha_clipArea = r;
 		scx.scx_trans = GeoIdentityTransform;
 		scx.scx_area = r;
 		scx.scx_use = ha.ha_parentUse;
@@ -243,9 +270,16 @@ extSubtree(parentUse, reg, f)
 	    cuts++;
 	    pdone = 100.0 * ((float)cuts / (float)totcuts);
 	    if ((((pdone - plast) > 5.0) || (cuts == totcuts)) && (cuts > 1)) {
-	        TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
-	        plast = pdone;
-		TxFlushOut();
+		/* Only print something if the 5-second timer has expired */
+		if (GrDisplayStatus == DISPLAY_BREAK_PENDING)
+		{
+		    TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
+		    plast = pdone;
+		    TxFlushOut();
+
+		    GrDisplayStatus = DISPLAY_IN_PROGRESS;
+		    SigSetTimer(5);
+		}
 
 #ifdef MAGIC_WRAPPER
 		/* We need to let Tk paint the console display */
@@ -292,6 +326,11 @@ done:
     /* Output connections and node adjustments */
     extOutputConns(&ha.ha_connHash, f);
     HashKill(&ha.ha_connHash);
+    GrDisplayStatus = DISPLAY_IDLE;
+    SigRemoveTimer();
+
+    /* Clear the CU_SUB_EXTRACTED flag from all children instances */
+    DBCellEnum(def, extClearUseFlags, (ClientData)NULL);
 }
 
 #ifdef	exactinteractions
@@ -337,7 +376,7 @@ extSubtreeInterFunc(tile, ha)
     return (0);
 }
 #endif	/* exactinteractions */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -470,7 +509,7 @@ extSubtreeInteraction(ha)
 	 * hierarchy are part of ha_connHash and have adjustments
 	 * that are printed in the "merge" statement.  Nodes not in
 	 * the current cell are not considered.  Anything left over
-	 * has its adjusted value output. 
+	 * has its adjusted value output.
 	 */
 	for (reg = ha->ha_parentReg; reg; reg = reg->nreg_next)
 	{
@@ -531,7 +570,7 @@ extSubtreeInteraction(ha)
     extHierFreeLabels(cumDef);
     DBCellClearDef(cumDef);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -598,7 +637,7 @@ extSubtreeAdjustInit(ha)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -650,7 +689,7 @@ extSubtreeOutputCoupling(ha)
 	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -742,6 +781,8 @@ extSubtreeFunc(scx, ha)
 
 	    for (lab = cumDef->cd_labels; lab ; lab = lab->lab_next)
 	    {
+		if (!(lab->lab_flags & LABEL_STICKY)) continue;
+		
 		n = sizeof (Label) + strlen(lab->lab_text)
 			- sizeof lab->lab_text + 1;
 
@@ -781,11 +822,11 @@ extSubtreeFunc(scx, ha)
     /* connected together in the cell def, so in the case of an	*/
     /* array, just make sure that the first array entry is	*/
     /* connected.						*/
-    
+
     if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
 	extHierSubstrate(ha, use, -1, -1);
     else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
-    {	
+    {
 	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 	    extHierSubstrate(ha, use, -1, y);
     }
@@ -800,6 +841,8 @@ extSubtreeFunc(scx, ha)
 	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 		extHierSubstrate(ha, use, x, y);
     }
+    /* Mark substrate as having been extracted for this use. */
+    use->cu_flags |= CU_SUB_EXTRACTED;
 
     /* Free the cumulative node list we extracted above */
     if (ha->ha_cumFlat.et_nodes)
@@ -829,7 +872,7 @@ extSubtreeFunc(scx, ha)
 
     return (2);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -864,11 +907,11 @@ extSubstrateFunc(scx, ha)
     /* connected together in the cell def, so in the case of an	*/
     /* array, just make sure that the first array entry is	*/
     /* connected.						*/
-    
+
     if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
 	extHierSubstrate(ha, use, -1, -1);
     else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
-    {	
+    {
 	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 	    extHierSubstrate(ha, use, -1, y);
     }
@@ -883,9 +926,10 @@ extSubstrateFunc(scx, ha)
 	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 		extHierSubstrate(ha, use, x, y);
     }
+    use->cu_flags |= CU_SUB_EXTRACTED;
     return (2);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1053,7 +1097,7 @@ extConnFindFunc(tp, preg)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
