@@ -5,16 +5,16 @@
  * Code to read and process the sections of a technology file
  * that are specific to circuit extraction.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -122,11 +122,11 @@ static keydesc keyTable[] = {
     "height",		HEIGHT,		4,	4,
 "type height-above-subtrate thickness",
 
-    "antenna",		ANTENNA,	4,	4,
-"type antenna-ratio",
+    "antenna",		ANTENNA,	4,	6,
+"type [calc-type] [antenna-ratio-proportional] antenna-ratio-const",
 
-    "model",		MODEL,		3,	3,
-"partial-cumulative area-sidewall",
+    "model",		MODEL,		2,	3,
+"partial-cumulative [area-sidewall]",
 
     "tiedown",		TIEDOWN,	2,	2,
 "types",
@@ -166,8 +166,8 @@ static keydesc keyTable[] = {
     "style",		STYLE,		2,	4,
 "stylename",
 
-    "substrate",	SUBSTRATE,	3,	4,
-"types plane",
+    "substrate",	SUBSTRATE,	3,	5,
+"types plane [subs-node]",
 
     "units",		UNITS,		2,	2,
 "lambda|microns",
@@ -177,7 +177,7 @@ static keydesc keyTable[] = {
 
     0
 };
-
+
 
 /*
  * Table used for parsing the "device" keyword types
@@ -286,21 +286,31 @@ ExtCompareStyle(stylename)
  *	inefficient, but keeps the netlist generator separated from
  *	the extractor.  Some of this code is seriously schizophrenic,
  *	and should not be investigated too closely.
- * 
+ *
  * Results:
  *	Return FALSE if no device corresponds to index "idx".  TRUE
  *	otherwise.
  *
  * Side Effects:
  *	Fills values in the argument list.
+ *
+ * Notes:
+ *	The original sd_rclassptr has been expanded to s_rclassptr and
+ *	d_rclassptr to capture asymmetric devices, bipolars, etc.  Note
+ *	that this is not a general-purpose method extending beyond two
+ *	(non-gate) terminals, and should be updated.
+ *	
  * ----------------------------------------------------------------------------
  */
 
 bool
-ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
+ExtGetDevInfo(idx, devnameptr, devtypeptr, s_rclassptr, d_rclassptr,
+		sub_rclassptr, subnameptr)
     int idx;
-    char **devnameptr;
-    short *sd_rclassptr;	/* First SD type only---needs to be updated! */
+    char **devnameptr;	    /* Name of extracted device model */
+    TileType *devtypeptr;   /* Magic tile type of device */
+    short *s_rclassptr;	    /* Source (1st terminal) type only */
+    short *d_rclassptr;	    /* Drain (2nd terminal) type only */
     short *sub_rclassptr;
     char **subnameptr;
 {
@@ -346,20 +356,42 @@ ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
 
     *devnameptr = locdname;
     *subnameptr = devptr->exts_deviceSubstrateName;
+    *devtypeptr = t;
 
     tmask = &devptr->exts_deviceSDTypes[0];
-    *sd_rclassptr = (short)(-1);	/* NO_RESCLASS */
+    *s_rclassptr = (short)(-1);	/* NO_RESCLASS */
 
     for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
     {
 	rmask = &ExtCurStyle->exts_typesByResistClass[n];
 	if (TTMaskIntersect(rmask, tmask))
 	{
-	    *sd_rclassptr = (short)n;
+	    *s_rclassptr = (short)n;
 	    break;
 	}
     }
 
+    tmask = &devptr->exts_deviceSDTypes[1];
+    if (TTMaskIsZero(tmask))
+    {
+	/* Set source and drain resistance classes to be the same */
+	*d_rclassptr = (short)n;
+    }
+    else
+    {
+	*d_rclassptr = (short)(-1);	/* NO_RESCLASS */
+
+	for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+	{
+	    rmask = &ExtCurStyle->exts_typesByResistClass[n];
+	    if (TTMaskIntersect(rmask, tmask))
+	    {
+		*d_rclassptr = (short)n;
+		break;
+	    }
+	}
+    }
+
     tmask = &devptr->exts_deviceSubstrateTypes;
     *sub_rclassptr = (short)(-1);	/* NO_RESCLASS */
 
@@ -408,13 +440,74 @@ extGetDevType(devname)
     return -1;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * ExtGetGateTypesMask --
+ *
+ *	Put a mask of FET gate types in the mask pointer argument.
+ *	Return 0 on success, 1 on failure (no extraction style set)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+ExtGetGateTypesMask(mask)
+    TileTypeBitMask *mask;
+{
+    TileType ttype;
+
+    if (ExtCurStyle == NULL) return 1;
+
+    TTMaskZero(mask);
+    for (ttype = TT_TECHDEPBASE; ttype < DBNumTypes; ttype++)
+    {
+        if (TTMaskHasType(&ExtCurStyle->exts_deviceMask, ttype))
+        {
+            ExtDevice *devptr = ExtCurStyle->exts_device[ttype];
+
+            for (; devptr; devptr = devptr->exts_next)
+                if ((devptr->exts_deviceClass == DEV_MOSFET) ||
+                        (devptr->exts_deviceClass == DEV_FET) ||
+                        (devptr->exts_deviceClass == DEV_ASYMMETRIC) ||
+                        (devptr->exts_deviceClass == DEV_MSUBCKT))
+                    TTMaskSetType(mask, ttype);
+        }
+    }
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * ExtGetDiffTypesMask --
+ *
+ *	Put a mask of diffusion types in the mask pointer argument.
+ *	Return 0 on success, 1 on failure (no extraction style set)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+ExtGetDiffTypesMask(mask)
+    TileTypeBitMask *mask;
+{
+    TileType ttype;
+
+    if (ExtCurStyle == NULL) return 1;
+
+    TTMaskZero(mask);
+    TTMaskSetMask(mask, &ExtCurStyle->exts_antennaTieTypes);
+
+    return 0;
+}
 
 #ifdef THREE_D
 /*
  * ----------------------------------------------------------------------------
  *
  * ExtGetZAxis --
- * 
+ *
  *	Get the height and thickness parameters for a layer (used by the
  *	graphics module which does not have access to internal variables
  *	in the extract section).
@@ -568,7 +661,7 @@ ExtSetStyle(name)
     TxError("\"%s\" is not one of the extraction styles Magic knows.\n", name);
     ExtPrintStyle(FALSE, TRUE, TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -704,8 +797,10 @@ extTechStyleInit(style)
 
     for (r = 0; r < DBNumTypes; r++)
     {
+	style->exts_antennaRatio[r].areaType = (char)0;
 	style->exts_antennaRatio[r].ratioGate = 0.0;
-	style->exts_antennaRatio[r].ratioDiff = 0.0;
+	style->exts_antennaRatio[r].ratioDiffA = 0.0;
+	style->exts_antennaRatio[r].ratioDiffB = 0.0;
 	style->exts_resistByResistClass[r] = 0;
 	TTMaskZero(&style->exts_typesByResistClass[r]);
 	style->exts_typesResistChanged[r] = DBAllButSpaceAndDRCBits;
@@ -723,9 +818,10 @@ extTechStyleInit(style)
     style->exts_globSubstratePlane = -1;
     TTMaskZero(&style->exts_globSubstrateTypes);
     TTMaskZero(&style->exts_globSubstrateShieldTypes);
+    style->exts_globSubstrateName = (char *)NULL;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -746,8 +842,14 @@ ExtStyle *
 extTechStyleNew()
 {
     ExtStyle *style;
+    TileType r;
 
     style = extTechStyleAlloc();
+
+    /* This entry in ExtStyle is allocated and must be initialized */
+    for (r = 0; r < NT; r++)
+	style->exts_device[r] = NULL;
+
     extTechStyleInit(style);
     return style;
 }
@@ -787,7 +889,7 @@ aToCap(str)
  * Re-read the technology file to load the specified technology extraction
  * style into structure ExtCurStyle.  This is much more memory-efficient than
  * keeping a separate (and huge!) ExtStyle structure for each extraction style.
- * It incurs a complete reading of the tech file on startup and every time the 
+ * It incurs a complete reading of the tech file on startup and every time the
  * extraction style is changed, but we can assume that this does not happen
  * often.  The first style in the technology file is assumed to be default, so
  * that re-reading the tech file is not necessary on startup unless the default
@@ -857,7 +959,7 @@ ExtTechInit()
  *	Parse the techfile line for the "defaultareacap" keyword.
  *	This is equivalent to the "areacap" line but also applies
  *	to "overlap" of types on the second plane (if specified) and
- *	all planes below it, with appropriate intervening types. 
+ *	all planes below it, with appropriate intervening types.
  *
  * Usage:
  *	defaultareacap types plane [[subtypes] subplane] value
@@ -891,7 +993,7 @@ ExtTechSimpleAreaCap(argc, argv)
     int plane1, plane2, plane3, pnum1, pnum2, pnum3;
     PlaneMask pshield;
 
-    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder) 
+    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder)
     {
 	TechError("Cannot parse area cap line without plane ordering!\n");
 	return;
@@ -1020,7 +1122,7 @@ ExtTechSimplePerimCap(argc, argv)
     PlaneMask pshield;
     EdgeCap *cnew;
 
-    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder) 
+    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder)
     {
 	TechError("Cannot parse area cap line without plane ordering!\n");
 	return;
@@ -1246,7 +1348,7 @@ ExtTechSimpleOverlapCap(argv)
     int plane1, plane2, plane3, pnum1, pnum2, pnum3;
     PlaneMask pshield;
 
-    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder) 
+    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder)
     {
 	TechError("Cannot parse area cap line without plane ordering!\n");
 	return;
@@ -1340,8 +1442,9 @@ ExtTechSimpleSideOverlapCap(argv)
     int plane1, plane2, plane3, pnum1, pnum2, pnum3;
     PlaneMask pshield;
     EdgeCap *cnew;
+    bool forward;
 
-    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder) 
+    if (ExtCurStyle->exts_planeOrderStatus != seenPlaneOrder)
     {
 	TechError("Cannot parse area cap line without plane ordering!\n");
 	return;
@@ -1372,6 +1475,19 @@ ExtTechSimpleSideOverlapCap(argv)
     pnum1 = ExtCurStyle->exts_planeOrder[plane1];
     pnum2 = ExtCurStyle->exts_planeOrder[plane2];
 
+    if (pnum1 == pnum2)
+    {
+	TechError("Cannot have fringing capacitance between "
+		"types on the same plane\n");
+	return;
+    }
+
+    /* Fringing cap works both directions.  Consider the case plane1 <	*/
+    /* plane2 as the "forward" case, and plane1 > plane2 as the		*/
+    /* "reverse" case.							*/
+
+    forward = (plane1 < plane2) ? TRUE : FALSE;
+
     /* Find all types in or below plane2 (i.e., ~(space)/plane2)	   */
     /* Shield planes are the ones between plane1 and plane2 */
 
@@ -1380,7 +1496,12 @@ ExtTechSimpleSideOverlapCap(argv)
     for (plane3 = PL_TECHDEPBASE; plane3 < DBNumPlanes; plane3++)
     {
 	pnum3 = ExtCurStyle->exts_planeOrder[plane3];
-	if (pnum3 > pnum2 && pnum3 < pnum1)
+	if ((forward == TRUE) && (pnum3 > pnum2 && pnum3 < pnum1))
+	{
+	    TTMaskSetMask(&shields, &DBPlaneTypes[plane3]);
+	    pshield |= PlaneNumToMaskBit(plane3);
+	}
+	else if ((forward == FALSE) && (pnum3 < pnum2 && pnum3 > pnum1))
 	{
 	    TTMaskSetMask(&shields, &DBPlaneTypes[plane3]);
 	    pshield |= PlaneNumToMaskBit(plane3);
@@ -1430,36 +1551,10 @@ ExtTechSimpleSideOverlapCap(argv)
 	    }
 	}
 
-	/* Reverse case (swap "types" and "ov") */
-
-#if 0
-	if (TTMaskHasType(&ov, s))
-	{
-	    ExtCurStyle->exts_sidePlanes |= PlaneNumToMaskBit(plane2);
-	    TTMaskSetType(&ExtCurStyle->exts_sideTypes[plane2], s);
-	    TTMaskSetMask(&ExtCurStyle->exts_sideEdges[s], &notov);
-	    for (t = TT_SPACE; t < DBNumTypes; t++)
-	    {
-		if (!TTMaskHasType(&notov, t)) continue;
-  		if (DBIsContact(t)) continue;
-
-		TTMaskSetMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t], &types);
-		ExtCurStyle->exts_sideOverlapOtherPlanes[s][t] |=
-				PlaneNumToMaskBit(plane1);
-		cnew = (EdgeCap *) mallocMagic((unsigned) (sizeof (EdgeCap)));
-		cnew->ec_cap = capVal;
-		cnew->ec_far = shields;		/* Types that shield */
-		cnew->ec_near = types;		/* Types we create cap with */
-		cnew->ec_pmask = PlaneNumToMaskBit(plane1);
-		cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
-		ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
-
-		for (r = TT_TECHDEPBASE; r < DBNumTypes; r++)
-		    if (TTMaskHasType(&types, r))
-			ExtCurStyle->exts_sideOverlapShieldPlanes[s][r] |= pshield;
-	    }
-	}
-#endif /* 0 */
+	/* There is no "reverse case".  Overlap from A->B will be different */
+	/* than B->A because it depends on the thickness of A and B.  For   */
+	/* the reverse case, the defaultsideoverlap statement must be	    */
+	/* repeated with the correct capacitance.			    */
     }
 }
 
@@ -1704,7 +1799,7 @@ ExtTechLine(sectionName, argc, argv)
 			for (es = ExtAllStyles; es->exts_next; es = es->exts_next);
 			es->exts_next = newStyle;
 		    }
-		    
+
 		    if (cptr == NULL)
 			break;
 		    else
@@ -1718,7 +1813,7 @@ ExtTechLine(sectionName, argc, argv)
 	/* style encountered.  Otherwise, if we are changing styles, 	*/
 	/* load this style only if the name matches that in ExtCurStyle.*/
 
-	if (ExtCurStyle == NULL) 
+	if (ExtCurStyle == NULL)
 	{
 	    ExtCurStyle = extTechStyleNew();
 	    ExtCurStyle->exts_name = newStyle->exts_name;
@@ -1787,7 +1882,7 @@ ExtTechLine(sectionName, argc, argv)
 	while (*tptr != '\0')
 	{
 	    cptr = strchr(tptr, ',');
-	    if (cptr != NULL) 
+	    if (cptr != NULL)
 	    {
 		*cptr = '\0';
 		for (j = 1; isspace(*(cptr - j)); j++)
@@ -1808,7 +1903,7 @@ ExtTechLine(sectionName, argc, argv)
 		    return TRUE;
 		}
 	    }
-	
+
 	    if (cptr == NULL)
 		break;
 	    else
@@ -1896,8 +1991,8 @@ ExtTechLine(sectionName, argc, argv)
 	    subsName = argv[5];
 
 	    // From magic version 8.1, subs name can be a nonfunctional
-	    // throwaway (e.g., "error"), so don't throw a warning. 
-	
+	    // throwaway (e.g., "error"), so don't throw a warning.
+
 	    cp = strchr(subsName, '!');
 	    if (cp == NULL || cp[1] != '\0')
 	    {
@@ -2123,7 +2218,7 @@ ExtTechLine(sectionName, argc, argv)
 		    else
 		    {
 			/* Normal symmetric device with swappable source/drain */
-			
+
 			DBTechNoisyNameMask(argv[4], &termtypes[0]); /* source/drain */
 			termtypes[1] = DBZeroTypeBits;
 			if (strcmp(argv[5], "None"))
@@ -2214,10 +2309,10 @@ ExtTechLine(sectionName, argc, argv)
 			nterm = argc - 4;
 			iterm = 4;
 		    }
-		    
+
 		    /* terminals */
 		    for (i = iterm; i < iterm + nterm; i++)
-			DBTechNoisyNameMask(argv[iterm], &termtypes[i - iterm]);
+			DBTechNoisyNameMask(argv[i], &termtypes[i - iterm]);
 		    termtypes[nterm] = DBZeroTypeBits;
 
 		    if (nterm == 0) i++;
@@ -2257,7 +2352,7 @@ ExtTechLine(sectionName, argc, argv)
 		    for (i = 0; !TTMaskIsZero(&termtypes[i]); i++);
 		    devptr->exts_deviceSDTypes = (TileTypeBitMask *)
 					mallocMagic((i + 1) * sizeof(TileTypeBitMask));
-			
+
 		    for (i = 0; !TTMaskIsZero(&termtypes[i]); i++)
 			devptr->exts_deviceSDTypes[i] = termtypes[i];
 		    devptr->exts_deviceSDTypes[i] = DBZeroTypeBits;
@@ -2351,38 +2446,96 @@ ExtTechLine(sectionName, argc, argv)
 	    break;
 	case ANTENNA: {
 	    float antennaratio;
+	    char areaType;
+	    bool hasModel = FALSE;
+	    int argidx = 2;
 
 	    if (!StrIsNumeric(argv[2]))
 	    {
-		TechError("Gate layer antenna ratio %s must be numeric\n", argv[2]);
-		break;
+		if (!strcmp(argv[2], "surface") || !strcmp(argv[2], "area"))
+		{
+		    areaType = ANTENNAMODEL_SURFACE;
+		    hasModel = TRUE;
+		}
+		else if (!strcmp(argv[2], "sidewall") || !strcmp(argv[2], "perimeter"))
+		{
+		    areaType = ANTENNAMODEL_SIDEWALL;
+		    hasModel = TRUE;
+		}
+		else
+		{
+		    TechError("Error in layer antenna calculation type \"%s\"; "
+			    " must be \"surface\" or \"sidewall\"\n", argv[2]);
+		    break;
+		}
 	    }
-	    antennaratio = (float)strtod(argv[2], NULL);
+	    if (hasModel == FALSE)
+	    {
+		if (ExtCurStyle->exts_antennaModel & ANTENNAMODEL_SURFACE)
+		    areaType = ANTENNAMODEL_SURFACE;
+		else if (ExtCurStyle->exts_antennaModel & ANTENNAMODEL_SIDEWALL)
+		    areaType = ANTENNAMODEL_SIDEWALL;
+		else
+		    TechError("No antenna calculation type given for layer(s) %s "
+			    " and no default calculation type found.\n", argv[1]);
+	    }
+
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		if (TTMaskHasType(&types1, t))
-		{
-		    ExtCurStyle->exts_antennaRatio[t].ratioGate = antennaratio;
-		}
+		    ExtCurStyle->exts_antennaRatio[t].areaType = areaType;
 
-	    if (!StrIsNumeric(argv[3]))
+	    if (hasModel == TRUE) argidx = 3;
+
+	    if (!StrIsNumeric(argv[argidx]))
 	    {
-		if (!strcasecmp(argv[3], "none"))
+		TechError("Gate layer antenna ratio %s must be numeric\n", argv[argidx]);
+		break;
+	    }
+	    antennaratio = (float)strtod(argv[argidx], NULL);
+
+	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+		if (TTMaskHasType(&types1, t))
+		    ExtCurStyle->exts_antennaRatio[t].ratioGate = antennaratio;
+
+	    argidx++;
+	    if (!StrIsNumeric(argv[argidx]))
+	    {
+		if (!strcasecmp(argv[argidx], "none"))
 		    antennaratio = INFINITY;
 		else
 		{
-		    TechError("Diff layer antenna ratio %s must be numeric\n", argv[3]);
+		    TechError("Diff layer antenna ratio %s must be numeric\n",
+			    argv[argidx]);
 		    break;
 		}
 	    }
 	    else
-		antennaratio = (float)strtod(argv[3], NULL);
+		antennaratio = (float)strtod(argv[argidx], NULL);
+
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		if (TTMaskHasType(&types1, t))
+		    ExtCurStyle->exts_antennaRatio[t].ratioDiffB = antennaratio;
+
+	    argidx++;
+	    if (argidx < argc)
+	    {
+		if (!StrIsNumeric(argv[argidx]))
 		{
-		    ExtCurStyle->exts_antennaRatio[t].ratioDiff = antennaratio;
+		    TechError("Diff layer antenna ratio %s must be numeric\n",
+				argv[argidx]);
+		    break;
 		}
+		antennaratio = (float)strtod(argv[argidx], NULL);
 	    }
+	    else
+		antennaratio = 0;
+
+	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+		if (TTMaskHasType(&types1, t))
+		    ExtCurStyle->exts_antennaRatio[t].ratioDiffA = antennaratio;
+
 	    break;
+	}
 	case MODEL:
 	    if (!strcmp(argv[1], "partial"))
 		ExtCurStyle->exts_antennaModel |= ANTENNAMODEL_PARTIAL;
@@ -2392,13 +2545,16 @@ ExtTechLine(sectionName, argc, argv)
 		TxError("Unknown antenna model \"%s\":  Use \"partial\" or "
 			    "\"cumulative\"");
 
-	    if (!strcmp(argv[2], "area"))
-		ExtCurStyle->exts_antennaModel |= ANTENNAMODEL_AREA;
-	    else if (!strcmp(argv[2], "sidewall"))
-		ExtCurStyle->exts_antennaModel |= ANTENNAMODEL_SIDEWALL;
-	    else
-		TxError("Unknown antenna model \"%s\":  Use \"area\" or "
-			    "\"sidewall\"");
+	    if (argc > 2)
+	    {
+		if (!strcmp(argv[2], "surface") || !strcmp(argv[2], "area"))
+		    ExtCurStyle->exts_antennaModel |= ANTENNAMODEL_SURFACE;
+		else if (!strcmp(argv[2], "sidewall") || !strcmp(argv[2], "perimeter"))
+		    ExtCurStyle->exts_antennaModel |= ANTENNAMODEL_SIDEWALL;
+		else
+		    TxError("Unknown antenna model \"%s\":  Use \"surface\" or "
+				    "\"sidewall\"");
+	    }
 	    break;
 
 	case TIEDOWN:
@@ -2710,11 +2866,16 @@ ExtTechLine(sectionName, argc, argv)
 	    TTMaskSetMask(&ExtCurStyle->exts_globSubstrateTypes, &types1);
 	    ExtCurStyle->exts_globSubstrateShieldTypes = idTypes;
 	    ExtCurStyle->exts_globSubstratePlane = DBTechNoisyNamePlane(argv[2]);
+
+	    /* Handle optional substrate node name */
+	    if (argc == 4)
+		ExtCurStyle->exts_globSubstrateName = StrDup((char **)NULL, argv[3]);
+
 	    break;
 	case NOPLANEORDER: {
-	     if ( ExtCurStyle->exts_planeOrderStatus == seenPlaneOrder ) 
+	     if ( ExtCurStyle->exts_planeOrderStatus == seenPlaneOrder )
 		TechError("\"noplaneordering\" specified after \"planeorder\".\n");
-	     else 
+	     else
 		ExtCurStyle->exts_planeOrderStatus = noPlaneOrder ;
 	    }
 	    break ;
@@ -2726,10 +2887,10 @@ ExtTechLine(sectionName, argc, argv)
 		TechError("\"planeorder\" specified after \"noplaneordering\".\n");
 	    }
 	    ExtCurStyle->exts_planeOrderStatus = seenPlaneOrder ;
-	    if (pnum < 0) 
+	    if (pnum < 0)
 		TechError("Unknown planeorder plane %s\n", argv[1]);
 	    else if (pos < 0 || pos >= DBNumPlanes-PL_TECHDEPBASE)
-		TechError("Planeorder index must be [0..%d]\n", 
+		TechError("Planeorder index must be [0..%d]\n",
 		    DBNumPlanes-PL_TECHDEPBASE-1);
 	    else
 		ExtCurStyle->exts_planeOrder[pnum] = pos;
@@ -2748,7 +2909,7 @@ diffplane:
 		"\tdifferent plane from intypes and outtypes.\n");
     return (TRUE);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -2863,7 +3024,7 @@ extTechFinalStyle(style)
 
 	    for (q = TT_TECHDEPBASE; q < DBNumUserLayers; q++)
 	    {
-		if (TTMaskHasType(&style->exts_overlapOtherTypes[q], r)) 
+		if (TTMaskHasType(&style->exts_overlapOtherTypes[q], r))
 		    TTMaskSetType(&style->exts_overlapOtherTypes[q], s);
 
 	        for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
@@ -2876,7 +3037,7 @@ extTechFinalStyle(style)
 		/* must count from TT_SPACE, not TT_TECHDEPBASE.		*/
 
 	        for (t = TT_SPACE; t < DBNumUserLayers; t++)
-		    if (TTMaskHasType(&style->exts_sideOverlapOtherTypes[q][t], r)) 
+		    if (TTMaskHasType(&style->exts_sideOverlapOtherTypes[q][t], r))
 			TTMaskSetType(&style->exts_sideOverlapOtherTypes[q][t], s);
 	    }
 	}
@@ -2919,9 +3080,9 @@ extTechFinalStyle(style)
 
 	for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	{
-	    TTMaskAndMask(&style->exts_overlapShieldTypes[s][t], 
+	    TTMaskAndMask(&style->exts_overlapShieldTypes[s][t],
 			&style->exts_activeTypes);
-	    TTMaskAndMask(&style->exts_sideOverlapOtherTypes[s][t], 
+	    TTMaskAndMask(&style->exts_sideOverlapOtherTypes[s][t],
 			&style->exts_activeTypes);
 	    TTMaskAndMask(&style->exts_sideCoupleOtherEdges[s][t],
 			&style->exts_activeTypes);
@@ -2934,7 +3095,7 @@ extTechFinalStyle(style)
 	TTMaskAndMask(&style->exts_sideTypes[p], &style->exts_activeTypes);
     }
 
-    if ( style->exts_planeOrderStatus == noPlaneOrder ) 
+    if ( style->exts_planeOrderStatus == noPlaneOrder )
     	return /* no need to check */ ;
     /* Else Check to make sure the plane order is a permutation of the
        numbers 0..DBNumPlanes-DBNumPlanes-1 */
@@ -2953,7 +3114,7 @@ extTechFinalStyle(style)
 	if (indicis[p1] < 1) missing = TRUE ;
     }
     if (!conflict && !missing)		/* Everything was ok */
-	goto zinit; 	
+	goto zinit;
 
     TxError ("\nWarning: Extraction Style %s\n", style -> exts_name);
     if (conflict) {
@@ -2992,14 +3153,14 @@ zinit:
     }
 
     /* If global variable "doConvert" is TRUE, then we convert from	*/
-    /* microns to lambda and microns^2 to lambda^2, based on the 	*/
-    /* current value of DBLambda and output scale.			*/
+    /* microns to lambda and microns^2 to lambda^2.			*/
 
     if (doConvert)
     {
-	/* Convert from micron units in the file to the current		*/
-	/* physical scalefactor as determined by the default CIF	*/
-	/* output scale.						*/
+	/* Use current CIF output scale for determining the scale	*/
+	/* factor between micron units in the extract section and 	*/
+	/* lambda units of the database (conversion from lambda to	*/
+	/* internal units is done separately).				*/
 
 	float dscale = CIFGetOutputScale(1000);
 
@@ -3031,7 +3192,7 @@ zinit:
 		// a specific distance, the value (run / separation)
 		// is unscaled, so the capacitance does not get
 		// modified by the scalefactor.  However, the lambda
-		// reference for sidewall cap is 2 lambda, so if 
+		// reference for sidewall cap is 2 lambda, so if
 		// the reference is to be interpreted as 1 micron,
 		// the value needs to be divided by 2 (the factor of
 		// 2 is made up by the fact that the sidewall is
