@@ -1,5 +1,5 @@
 /*
- * defRead.c --      
+ * defRead.c --
  *
  * This module incorporates the LEF/DEF format for standard-cell place and
  * route.
@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defRead.c,v 1.2 2008/06/01 18:37:43 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defRead.c,v 1.2 2008/06/01 18:37:43 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,6 +26,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "utils/undo.h"
+#include "utils/utils.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -73,11 +74,12 @@ enum def_netspecial_shape_keys {
 	DEF_SPECNET_SHAPE_DRCFILL};
 
 char *
-DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
+DefAddRoutes(rootDef, f, oscale, special, netname, defLayerMap)
     CellDef *rootDef;		/* Cell to paint */
     FILE *f;			/* Input file */
     float oscale;		/* Scale factor between LEF and magic units */
     bool special;		/* True if this section is SPECIALNETS */
+    char *netname;		/* Name of the net, if net is to be labeled */
     LefMapping *defLayerMap;	/* magic-to-lef layer mapping array */
 {
     char *token;
@@ -85,6 +87,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
     Point refp;			/* reference point */
     bool valid = FALSE;		/* is there a valid reference point? */
     bool initial = TRUE;
+    bool labeled = TRUE;
     Rect locarea;
     int extend, lextend, hextend;
     float x, y, z, w;
@@ -124,6 +127,8 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	NULL
     };
 
+    if (netname != NULL) labeled = FALSE;
+
     while (initial || (token = LefNextToken(f, TRUE)) != NULL)
     {
 	/* Get next point, token "NEW", or via name */
@@ -156,7 +161,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 
 	    if (routeLayer < 0)
 	    {
-		LefError(DEF_ERROR, "Unknown layer type \"%s\" for NEW route\n", token); 
+		LefError(DEF_ERROR, "Unknown layer type \"%s\" for NEW route\n", token);
 		continue;
 	    }
 	    paintLayer = routeLayer;
@@ -260,7 +265,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse X coordinate in RECT.\n"); 
+		LefError(DEF_ERROR, "Cannot parse X coordinate in RECT.\n");
 		goto endCoord;
 	    }
 
@@ -271,7 +276,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse Y coordinate in RECT.\n"); 
+		LefError(DEF_ERROR, "Cannot parse Y coordinate in RECT.\n");
 		goto endCoord;
 	    }
 
@@ -282,7 +287,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse X coordinate in RECT.\n"); 
+		LefError(DEF_ERROR, "Cannot parse X coordinate in RECT.\n");
 		goto endCoord;
 	    }
 	    token = LefNextToken(f, TRUE);	/* read ury */
@@ -292,13 +297,13 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse Y coordinate in RECT.\n"); 
+		LefError(DEF_ERROR, "Cannot parse Y coordinate in RECT.\n");
 		goto endCoord;
 	    }
 	    token = LefNextToken(f, TRUE);	/* read closing parens */
 	    if (*token != ')')
 	    {
-		LefError(DEF_ERROR, "Bad coordinates in RECT.\n"); 
+		LefError(DEF_ERROR, "Bad coordinates in RECT.\n");
 		goto endCoord;
 	    }
 	    GeoCanonicalRect(&locarea, &newRoute->r_r);
@@ -359,7 +364,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 			addRoute->r_r.r_ybot >>= 1;
 			addRoute->r_r.r_xtop >>= 1;
 			addRoute->r_r.r_ytop >>= 1;
-	
+
 			if (routeTop)
 			    routeList->r_next = addRoute;
 			else
@@ -367,7 +372,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 
 			routeList = addRoute;
 		    }
-			
+
 		    paintLayer = lefl->type;
 
 		    newRoute->r_r.r_xbot = refp.p_x + lefl->info.via.area.r_xbot;
@@ -449,7 +454,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    {
 		if (valid == FALSE)
 		{
-		    LefError(DEF_ERROR, "No reference point for \"*\" wildcard\n"); 
+		    LefError(DEF_ERROR, "No reference point for \"*\" wildcard\n");
 		    goto endCoord;
 		}
 	    }
@@ -459,7 +464,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse X coordinate.\n"); 
+		LefError(DEF_ERROR, "Cannot parse X coordinate.\n");
 		goto endCoord;
 	    }
 	    token = LefNextToken(f, TRUE);	/* read Y */
@@ -467,7 +472,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    {
 		if (valid == FALSE)
 		{
-		    LefError(DEF_ERROR, "No reference point for \"*\" wildcard\n"); 
+		    LefError(DEF_ERROR, "No reference point for \"*\" wildcard\n");
 		    freeMagic(newRoute);
 		    newRoute = NULL;
 		    goto endCoord;
@@ -479,7 +484,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    }
 	    else
 	    {
-		LefError(DEF_ERROR, "Cannot parse Y coordinate.\n"); 
+		LefError(DEF_ERROR, "Cannot parse Y coordinate.\n");
 		goto endCoord;
 	    }
 
@@ -605,6 +610,16 @@ endCoord:
 	/* paint */
 	DBPaint(rootDef, &routeTop->r_r, routeTop->r_type);
 
+	/* label */
+	if (labeled == FALSE)
+	{
+	    Rect r;
+	    r.r_xbot = r.r_xtop = (routeTop->r_r.r_xbot + routeTop->r_r.r_xtop) / 2;
+	    r.r_ybot = r.r_ytop = (routeTop->r_r.r_ybot + routeTop->r_r.r_ytop) / 2;
+	    DBPutLabel(rootDef, &r, GEO_CENTER, netname, routeTop->r_type, 0);
+	    labeled = TRUE;
+	}
+
 	/* advance to next point and free record (1-delayed) */
 	freeMagic((char *)routeTop);
 	routeTop = routeTop->r_next;
@@ -636,15 +651,17 @@ enum def_netprop_keys {
 	DEF_NETPROP_PROPERTY};
 
 void
-DefReadNets(f, rootDef, sname, oscale, special, total)
+DefReadNets(f, rootDef, sname, oscale, special, dolabels, total)
     FILE *f;
     CellDef *rootDef;
     char *sname;
     float oscale;
     bool special;		/* True if this section is SPECIALNETS */
+    bool dolabels;		/* If true, create a label for each net */
     int total;
 {
     char *token;
+    char *netname = NULL;
     int keyword, subkey;
     int processed = 0;
     LefMapping *defLayerMap;
@@ -666,7 +683,7 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
 	NULL
     };
 
-    defLayerMap = defMakeInverseLayerMap(); 
+    defLayerMap = defMakeInverseLayerMap(LAYER_MAP_VIAS);
 
     while ((token = LefNextToken(f, TRUE)) != NULL)
     {
@@ -684,8 +701,8 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
 	    case DEF_NET_START:
 
 		/* Get net name */
-		/* Presently, we ignore net names completely.	*/
 		token = LefNextToken(f, TRUE);
+		if (dolabels) netname = StrDup((char **)NULL, token);
 
 		/* Update the record of the number of nets processed	*/
 		/* and spit out a message for every 5% finished.	*/
@@ -725,10 +742,11 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
 			case DEF_NETPROP_FIXED:
 			case DEF_NETPROP_COVER:
 			    token = DefAddRoutes(rootDef, f, oscale, special,
-					defLayerMap);
+					netname, defLayerMap);
 			    break;
 		    }
 		}
+		if (dolabels) freeMagic(netname);
 		break;
 
 	    case DEF_NET_END:
@@ -924,6 +942,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
     int keyword, subkey, values, flags;
     int processed = 0;
     int pinDir = PORT_CLASS_DEFAULT;
+    int pinUse = PORT_USE_DEFAULT;
     int pinNum = 0;
     TileType curlayer = -1;
     Rect *currect, topRect;
@@ -960,6 +979,19 @@ DefReadPins(f, rootDef, sname, oscale, total)
 	NULL
     };
 
+    static char *pin_uses[] = {
+	"DEFAULT",
+	"SIGNAL",
+	"POWER",
+	"GROUND",
+	"CLOCK",
+	"RESET",
+	"ANALOG",
+	"SCAN",
+	"TIEOFF",
+	NULL
+    };
+
     static int lef_class_to_bitmask[] = {
 	PORT_CLASS_DEFAULT,
 	PORT_CLASS_INPUT,
@@ -969,6 +1001,20 @@ DefReadPins(f, rootDef, sname, oscale, total)
 	PORT_CLASS_FEEDTHROUGH
     };
 
+    static int lef_use_to_bitmask[] = {
+	PORT_USE_DEFAULT,
+	PORT_USE_SIGNAL,
+	PORT_USE_POWER,
+	PORT_USE_GROUND,
+	PORT_USE_CLOCK,
+	PORT_USE_RESET,
+	PORT_USE_ANALOG,
+	PORT_USE_SCAN,
+	PORT_USE_TIEOFF
+    };
+
+    flags = 0;
+
     while ((token = LefNextToken(f, TRUE)) != NULL)
     {
 	keyword = Lookup(token, pin_keys);
@@ -995,7 +1041,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 
 		/* Update the record of the number of pins		*/
 		/* processed and spit out a message for every 5% done.	*/
- 
+
 		LefEstimate(processed++, total, "pins");
 
 		/* Get pin name */
@@ -1007,6 +1053,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 		    break;
 		}
 		pending = FALSE;
+		curlayer = -1;
 
 		/* Now do a search through the line for "+" entries	*/
 		/* And process each.					*/
@@ -1039,6 +1086,13 @@ DefReadPins(f, rootDef, sname, oscale, total)
 			    hasports = TRUE;
 			    break;
 			case DEF_PINS_PROP_USE:
+			    token = LefNextToken(f, TRUE);
+			    subkey = Lookup(token, pin_uses);
+			    if (subkey < 0)
+				LefError(DEF_ERROR, "Unknown pin use \"%s\"\n", token);
+			    else
+				pinUse = lef_use_to_bitmask[subkey];
+			    break;
 			case DEF_PINS_PROP_NET:
 			    /* Get the net name, but ignore it */
 			    token = LefNextToken(f, TRUE);
@@ -1047,7 +1101,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 			    token = LefNextToken(f, TRUE);
 			    subkey = Lookup(token, pin_classes);
 			    if (subkey < 0)
-				LefError(DEF_ERROR, "Unknown pin class\n");
+				LefError(DEF_ERROR, "Unknown pin class \"%s\"\n", token);
 			    else
 				pinDir = lef_class_to_bitmask[subkey];
 			    break;
@@ -1056,9 +1110,9 @@ DefReadPins(f, rootDef, sname, oscale, total)
 			    currect = LefReadRect(f, curlayer, oscale);
 			    if (pending)
 			    {
-				flags = PORT_DIR_MASK;
 				/* If layer was unknown, set to space and force	*/
 				/* non-sticky.					*/
+				flags = PORT_DIR_MASK;
 				if (curlayer < 0)
 				    curlayer = TT_SPACE;
 				else
@@ -1067,7 +1121,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 				GeoTransRect(&t, currect, &topRect);
 				DBPaint(rootDef, &topRect, curlayer);
 				DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
-					pinNum | pinDir | flags);
+					pinNum | pinDir | pinUse | flags);
 				pending = FALSE;
 				pinNum++;
 			    }
@@ -1079,9 +1133,9 @@ DefReadPins(f, rootDef, sname, oscale, total)
 				pending = TRUE;
 			    else
 			    {
-				flags = PORT_DIR_MASK;
 				/* If layer was unknown, set to space and force	*/
 				/* non-sticky.					*/
+				flags = PORT_DIR_MASK;
 				if (curlayer < 0)
 				    curlayer = TT_SPACE;
 				else
@@ -1090,7 +1144,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 				GeoTransRect(&t, currect, &topRect);
 				DBPaint(rootDef, &topRect, curlayer);
 				DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
-					pinNum | pinDir | flags);
+					pinNum | pinDir | pinUse | flags);
 				pinNum++;
 			    }
 			    break;
@@ -1116,7 +1170,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 	LefError(DEF_WARNING, "Number of pins read (%d) does not match "
 		"the number declared (%d).\n", processed, total);
 }
- 
+
 /*
  *------------------------------------------------------------
  *
@@ -1199,9 +1253,12 @@ DefReadVias(f, sname, oscale, total)
 
 		/* Update the record of the number of vias		*/
 		/* processed and spit out a message for every 5% done.	*/
- 
+
 		LefEstimate(processed++, total, "vias");
 
+		/* If not otherwise specified, rows and columns default to 1 */
+		rows = cols = 1;
+
 		/* Get via name */
 		token = LefNextToken(f, TRUE);
 		if (sscanf(token, "%2047s", vianame) != 1)
@@ -1287,6 +1344,35 @@ DefReadVias(f, sname, oscale, total)
 			    blayer = LefReadLayer(f, FALSE);
 			    clayer = LefReadLayer(f, FALSE);
 			    tlayer = LefReadLayer(f, FALSE);
+
+			    /* Provisional behavior:  A known tool generating	*/
+			    /* DEF uses the order (bottom, top, cut).  This may	*/
+			    /* be a bug in the tool and an issue is being	*/
+			    /* raised.  However, there is no harm in detecting	*/
+			    /* which layer is the cut and swapping as needed.	*/
+
+			    if (!DBIsContact(clayer))
+			    {
+				TileType swaplayer;
+				LefError(DEF_WARNING, "Improper layer order for"
+					" VIARULE.\n");
+				if (DBIsContact(tlayer))
+				{
+				    swaplayer = clayer;
+				    clayer = tlayer;
+				    tlayer = swaplayer;
+				}
+				else if (DBIsContact(blayer))
+				{
+				    swaplayer = clayer;
+				    clayer = blayer;
+				    blayer = swaplayer;
+				}
+				else
+				    LefError(DEF_ERROR, "No cut layer specified in"
+					    " VIARULE.\n");
+			    }
+
 			    generated = TRUE;
 			    break;
 			case DEF_VIAS_PROP_CUTSPACING:
@@ -1305,18 +1391,6 @@ DefReadVias(f, sname, oscale, total)
 			    generated = TRUE;
 			    break;
 			case DEF_VIAS_PROP_ENCLOSURE:
-			    token = LefNextToken(f, TRUE);
-			    if (sscanf(token, "%d", &enctx) != 1)
-			    {
-				LefError(DEF_ERROR, "Invalid syntax for ENCLOSURE.\n");
-				/* To do:  Get cut enclosures from DRC ruleset */
-			    }
-			    token = LefNextToken(f, TRUE);
-			    if (sscanf(token, "%d", &encty) != 1)
-			    {
-				LefError(DEF_ERROR, "Invalid syntax for ENCLOSURE.\n");
-				/* To do:  Get cut enclosures from DRC ruleset */
-			    }
 			    token = LefNextToken(f, TRUE);
 			    if (sscanf(token, "%d", &encbx) != 1)
 			    {
@@ -1329,6 +1403,18 @@ DefReadVias(f, sname, oscale, total)
 				LefError(DEF_ERROR, "Invalid syntax for ENCLOSURE.\n");
 				/* To do:  Get cut enclosures from DRC ruleset */
 			    }
+			    token = LefNextToken(f, TRUE);
+			    if (sscanf(token, "%d", &enctx) != 1)
+			    {
+				LefError(DEF_ERROR, "Invalid syntax for ENCLOSURE.\n");
+				/* To do:  Get cut enclosures from DRC ruleset */
+			    }
+			    token = LefNextToken(f, TRUE);
+			    if (sscanf(token, "%d", &encty) != 1)
+			    {
+				LefError(DEF_ERROR, "Invalid syntax for ENCLOSURE.\n");
+				/* To do:  Get cut enclosures from DRC ruleset */
+			    }
 			    generated = TRUE;
 			    break;
 			case DEF_VIAS_PROP_ROWCOL:
@@ -1367,7 +1453,7 @@ DefReadVias(f, sname, oscale, total)
 	LefError(DEF_WARNING, "Number of vias read (%d) does not match "
 		"the number declared (%d).\n", processed, total);
 }
- 
+
 /*
  *------------------------------------------------------------
  *
@@ -1447,7 +1533,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 
 		/* Update the record of the number of components	*/
 		/* processed and spit out a message for every 5% done.	*/
- 
+
 		LefEstimate(processed++, total, "subcell instances");
 
 		/* Get use and macro names */
@@ -1470,7 +1556,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 		/* specific syntax [xlo:xsep:xhi][ylo:ysep:yhi] and */
 		/* is easy enough to distinguish.		    */
 
-		/* 
+		/*
 		    dptr = strchr(usename, '[');
 		    if (dptr != NULL) {
 			*dptr = '_';
@@ -1487,11 +1573,14 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 
 		if (defMacro == (CellDef *)NULL)
 		{
+		    bool dereference;
+
 		    /* Before giving up, assume that this cell has a	*/
 		    /* magic .mag layout file.				*/
-		    defMacro = DBCellNewDef(token, (char *)NULL);
+		    defMacro = DBCellNewDef(token);
 		    defMacro->cd_flags &= ~CDNOTFOUND;
-		    if (!DBCellRead(defMacro, (char *)NULL, TRUE, NULL))
+		    dereference = (defMacro->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
+		    if (!DBCellRead(defMacro, (char *)NULL, TRUE, dereference, NULL))
 		    {
 		        LefError(DEF_ERROR, "Cell %s is not defined.  Maybe you "
 				"have not read the corresponding LEF file?\n",
@@ -1616,8 +1705,9 @@ enum def_sections {DEF_VERSION = 0, DEF_NAMESCASESENSITIVE,
 	DEF_END};
 
 void
-DefRead(inName)
+DefRead(inName, dolabels)
     char *inName;
+    bool dolabels;
 {
     CellDef *rootDef;
     FILE *f;
@@ -1793,13 +1883,15 @@ DefRead(inName)
 		token = LefNextToken(f, TRUE);
 		if (sscanf(token, "%d", &total) != 1) total = 0;
 		LefEndStatement(f);
-		DefReadNets(f, rootDef, sections[DEF_SPECIALNETS], oscale, TRUE, total);
+		DefReadNets(f, rootDef, sections[DEF_SPECIALNETS], oscale, TRUE,
+			dolabels, total);
 		break;
 	    case DEF_NETS:
 		token = LefNextToken(f, TRUE);
 		if (sscanf(token, "%d", &total) != 1) total = 0;
 		LefEndStatement(f);
-		DefReadNets(f, rootDef, sections[DEF_NETS], oscale, FALSE, total);
+		DefReadNets(f, rootDef, sections[DEF_NETS], oscale, FALSE,
+			dolabels, total);
 		break;
 	    case DEF_IOTIMINGS:
 		LefSkipSection(f, sections[DEF_IOTIMINGS]);
