@@ -1,5 +1,5 @@
 /*
- * lefRead.c --      
+ * lefRead.c --
  *
  * This module incorporates the LEF/DEF format for standard-cell place and
  * route.
@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/lefRead.c,v 1.2 2008/12/17 18:40:04 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/lefRead.c,v 1.2 2008/12/17 18:40:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -152,7 +152,7 @@ LefEstimate(processed, total, item_name)
 	gettimeofday(&tv_start, &tz);
 	last_time = 0.0;
     }
-	
+
     if (processed > check_interval)
     {
 	gettimeofday(&tv, &tz);
@@ -300,7 +300,7 @@ LefNextToken(f, ignore_eol)
 
 void
 LefError(int type, char *fmt, ...)
-{  
+{
     static int errors = 0, warnings = 0, messages = 0;
     va_list args;
 
@@ -572,7 +572,7 @@ lefFindCell(name)
 	def = DBCellLookDef(name);
 	if (def == NULL)
 	{
-	    def = DBCellNewDef(name, (char *)NULL);
+	    def = DBCellNewDef(name);
 	    DBReComputeBbox(def);
 	}
 	HashSetValue(h, def);
@@ -615,7 +615,7 @@ LefLower(token)
  * Results:
  *	Pointer to a lefLayer, which may or may not be the
  *	same one presented to the subroutine.
- *	
+ *
  * Side Effects:
  *	May add an entry to the list of LEF layers.
  *
@@ -802,6 +802,55 @@ LefReadLayer(f, obstruct)
     return LefReadLayers(f, obstruct, (TileType *)NULL, (Rect **)NULL);
 }
 
+/*
+ *------------------------------------------------------------
+ * LefReadLefPoint --
+ *
+ *	Read a LEF point record from the file, and
+ *	return the two coordinates in LEF units (floating-point).
+ *
+ * Results:
+ *	Return 0 on success, 1 on error.
+ *
+ * Side Effects:
+ *	Reads input from file f;
+ *	Returns values for the two point coordinates, as type
+ *	float, in "xp" and "yp".
+ *
+ * Note:
+ *	Most LEF files do not put parentheses around points, e.g.,
+ *	in the ORIGIN statement, although the LEF specification
+ *	definition of a point insists that it must.  Therefore
+ *	we attempt to parse points correctly either way.
+ *
+ *------------------------------------------------------------
+ */
+
+int
+LefReadLefPoint(f, xp, yp)
+    FILE *f;
+    float *xp, *yp;
+{
+    char *token;
+    bool needMatch = FALSE;
+
+    token = LefNextToken(f, TRUE);
+    if (*token == '(')
+    {
+	token = LefNextToken(f, TRUE);
+	needMatch = TRUE;
+    }
+    if (!token || sscanf(token, "%f", xp) != 1) return 1;
+    token = LefNextToken(f, TRUE);
+    if (!token || sscanf(token, "%f", yp) != 1) return 1;
+    if (needMatch)
+    {
+	token = LefNextToken(f, TRUE);
+	if (*token != ')') return 1;
+    }
+    return 0;
+}
+
 /*
  *------------------------------------------------------------
  * LefReadRect --
@@ -875,7 +924,7 @@ LefReadRect(f, curlayer, oscale)
     {
 	/* Scale coordinates (microns to magic internal units)	*/
 	/* Need to scale grid if necessary!			*/
-		
+
 	lefrect.r_xbot = (int)roundf(llx / oscale);
 	lefrect.r_ybot = (int)roundf(lly / oscale);
 	lefrect.r_xtop = (int)roundf(urx / oscale);
@@ -905,7 +954,7 @@ parse_error:
  * LefReadPolygon --
  *
  *	Read a LEF "POLYGON" record from the file, and
- *	return a linked point structure.  
+ *	return a linked point structure.
  *
  * Results:
  *	Returns a pointer to a Rect containing the magic
@@ -985,8 +1034,8 @@ LefReadPolygon(f, curlayer, oscale, ppoints)
 	freeMagic(lr);
 	lpoints++;
 	lr = lr->r_next;
-    }	
-    return plist;	
+    }
+    return plist;
 }
 
 /*
@@ -994,7 +1043,7 @@ LefReadPolygon(f, curlayer, oscale, ppoints)
  * LefPaintPolygon --
  *
  *	Paint a polygon into the CellDef indicated by lefMacro.
- * 
+ *
  *------------------------------------------------------------
  */
 
@@ -1212,18 +1261,21 @@ LefReadGeometry(lefMacro, f, oscale, do_list)
  *
  * Side Effects:
  *	Reads input from file f;
- *	Paints into the CellDef lefMacro.
+ *	Generates a new label entry in the CellDef lefMacro.
+ *	If "lanno" is not NULL, then the label pointed to by
+ *	lanno is modified.
  *
  *------------------------------------------------------------
  */
 
 void
-LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse, oscale)
+LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse, pinShape, oscale, lanno)
     CellDef *lefMacro;
     FILE *f;
     char *pinName;
-    int pinNum, pinDir, pinUse;
+    int pinNum, pinDir, pinUse, pinShape;
     float oscale;
+    Label *lanno;
 {
     Label *newlab;
     LinkedRect *rectList;
@@ -1232,10 +1284,42 @@ LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse, oscale)
 
     while (rectList != NULL)
     {
-	if (pinNum >= 0)
+	if ((pinNum >= 0) || (lanno != NULL))
 	{
 	    /* Label this area */
-	    DBPutLabel(lefMacro, &rectList->r_r, -1, pinName, rectList->r_type, 0);
+	    if (lanno != NULL)
+	    {
+		/* Modify an existing label */
+		lanno->lab_rect = rectList->r_r;
+		lanno->lab_type = rectList->r_type;
+
+		/* Pin number is not meaninful in LEF files, so keep	*/
+		/* any existing pin number.  If original label was not	*/
+		/* a port, then find a non-conflicting port number for	*/
+		/* it.							*/
+
+		if (lanno->lab_flags & PORT_DIR_MASK)
+		    pinNum = lanno->lab_flags & PORT_NUM_MASK;
+		else
+		{
+		    Label *sl;
+		    int idx;
+
+		    pinNum = -1;
+		    for (sl = lefMacro->cd_labels; sl != NULL; sl = sl->lab_next)
+		    {
+			if (sl->lab_flags & PORT_DIR_MASK)
+			{
+			    idx = sl->lab_flags & PORT_NUM_MASK;
+			    if (idx > pinNum) pinNum = idx;
+			}
+		    }
+		    pinNum++;
+		}
+	    }
+	    else
+		/* Create a new label (non-rendered) */
+		DBPutLabel(lefMacro, &rectList->r_r, -1, pinName, rectList->r_type, 0);
 
 	    /* Set this label to be a port */
 
@@ -1243,13 +1327,19 @@ LefReadPort(lefMacro, f, pinName, pinNum, pinDir, pinUse, oscale)
 		LefError(LEF_ERROR, "Internal error: No labels in cell!\n");
 	    else
 	    {
-		newlab = lefMacro->cd_lastLabel;
+		newlab = (lanno != NULL) ? lanno : lefMacro->cd_lastLabel;
 		if (strcmp(newlab->lab_text, pinName))
 		    LefError(LEF_ERROR, "Internal error:  Can't find the label!\n");
-		else /* Make this a port */
-		    newlab->lab_flags = pinNum | pinUse | pinDir | PORT_DIR_MASK;
+		else
+		    /* Make this a port, and make it a sticky label so that */
+		    /* it is guaranteed to be on the layer on which it is defined */
+		    newlab->lab_flags = pinNum | pinUse | pinDir | pinShape |
+			    PORT_DIR_MASK | LABEL_STICKY;
 	    }
-	    /* DBAdjustLabels(lefMacro, &rectList->area); */
+	    /* If lanno is non-NULL then the first rectangle in the LEF	    */
+	    /* port list is used to modify it.  All other LEF port geometry */
+	    /* generates new labels in the CellDef.			    */
+	    if (lanno != NULL) lanno = NULL;
 	}
 
 	freeMagic((char *)rectList);
@@ -1289,9 +1379,13 @@ LefReadPin(lefMacro, f, pinname, pinNum, oscale, is_imported)
    bool is_imported;
 {
     char *token;
+    char *testpin = pinname;
     int keyword, subkey;
     int pinDir = PORT_CLASS_DEFAULT;
     int pinUse = PORT_USE_DEFAULT;
+    int pinShape = PORT_SHAPE_DEFAULT;
+    Label *firstlab;
+    bool firstport = TRUE;
 
     static char *pin_keys[] = {
 	"DIRECTION",
@@ -1350,6 +1444,88 @@ LefReadPin(lefMacro, f, pinname, pinNum, oscale, is_imported)
 	PORT_USE_CLOCK
     };
 
+    static char *pin_shapes[] = {
+	"DEFAULT",
+	"ABUTMENT",
+	"RING",
+	"FEEDTHRU",
+	NULL
+    };
+
+    static int lef_shape_to_bitmask[] = {
+	PORT_SHAPE_DEFAULT,
+	PORT_SHAPE_ABUT,
+	PORT_SHAPE_RING,
+	PORT_SHAPE_THRU
+    };
+
+
+    /* If LEF file is being imported to annotate an existing layout, then
+     * find the first label in the layout that matches the PIN record.
+     * If there are no matches, then test for various issues with
+     * delimiter translation and case sensitivity.
+     */
+
+    if (is_imported)
+    {
+	/* Find the first matching label */
+
+	for (firstlab = lefMacro->cd_labels; firstlab; firstlab = firstlab->lab_next)
+	    if (!strcmp(firstlab->lab_text, testpin))
+		break;
+
+	/* If no match was found, look for common issues    */
+	/* such as bus delimiter translations or case	    */
+	/* sensitivity.					    */
+
+	if (firstlab == NULL)
+	{
+	    char *delim;
+
+	    testpin = (char *)mallocMagic(strlen(pinname) + 1);
+	    strcpy(testpin, pinname);
+
+	    if ((delim = strchr(testpin, '<')) != NULL)
+	    {
+		*delim = '[';
+		if ((delim = strchr(testpin, '>')) != NULL)
+		    *delim = ']';
+	    }
+	    else if ((delim = strchr(testpin, '[')) != NULL)
+	    {
+		char *delim2;
+		*delim = '<';
+		if ((delim2 = strchr(testpin, ']')) != NULL)
+		    *delim2 = '>';
+	    }
+	    for (firstlab = lefMacro->cd_labels; firstlab; firstlab = firstlab->lab_next)
+		if (!strcmp(firstlab->lab_text, testpin))
+		    break;
+
+	    if (firstlab == NULL)    /* No luck, so return to original */
+	    {
+		freeMagic(testpin);
+		testpin = pinname;
+	    }
+	}
+
+	if (firstlab == NULL)
+	{
+	    for (firstlab = lefMacro->cd_labels; firstlab; firstlab = firstlab->lab_next)
+		if (!strcasecmp(firstlab->lab_text, testpin))
+		    break;
+
+	    if (firstlab != NULL)    /* Case matching succeeded */
+	    {
+		if (testpin == pinname)
+		    testpin = (char *)mallocMagic(strlen(pinname) + 1);
+		strcpy(testpin, firstlab->lab_text);
+	    }
+	}
+
+	if (firstlab == NULL) firstlab = lefMacro->cd_labels;
+    }
+
     while ((token = LefNextToken(f, TRUE)) != NULL)
     {
 	keyword = Lookup(token, pin_keys);
@@ -1380,27 +1556,99 @@ LefReadPin(lefMacro, f, pinname, pinNum, oscale, is_imported)
 		    pinUse = lef_use_to_bitmask[subkey];
 		LefEndStatement(f);
 		break;
+	    case LEF_SHAPE:
+		token = LefNextToken(f, TRUE);
+		subkey = Lookup(token, pin_shapes);
+		if (subkey < 0)
+		    LefError(LEF_ERROR, "Improper SHAPE statement\n");
+		else
+		    pinShape = lef_shape_to_bitmask[subkey];
+		LefEndStatement(f);
+		break;
 	    case LEF_PORT:
 		if (is_imported)
 		{
+		    bool needRect = TRUE;
+		    bool hasPort = FALSE;
 		    Label *lab;
 
-		    LefSkipSection(f, NULL);
+		    /* Conflicting interests: One purpose of annotation */
+		    /* is to make ports where none existed.  But if the	*/
+		    /* cell has both port and non-port labels with the	*/
+		    /* same string, then don't mess with the non-port	*/
+		    /* label.						*/
+
+		    for (lab = firstlab; lab; lab = lab->lab_next)
+			if (lab->lab_flags & PORT_DIR_MASK)
+			    if (!strcmp(lab->lab_text, testpin))
+			    {
+				hasPort = TRUE;
+				break;
+			    }
+
 		    /* Skip the port geometry but find the pin name and	*/
 		    /* annotate with the use and direction.  Note that	*/
 		    /* there may be multiple instances of the label.	*/
-		    for (lab = lefMacro->cd_labels; lab; lab = lab->lab_next)
+		    /* However, if the label is a point label, then	*/
+		    /* replace it with the geometry from the LEF file.	*/
+
+		    if (hasPort == FALSE) lab = firstlab;
+		    for (; lab; lab = lab->lab_next)
 		    {
-			if (!strcmp(lab->lab_text, pinname))
+			if (!strcmp(lab->lab_text, testpin))
 			{
-			    lab->lab_flags &= ~(PORT_USE_MASK | PORT_DIR_MASK |
-					PORT_CLASS_MASK);
-			    lab->lab_flags = pinNum | pinUse | pinDir | PORT_DIR_MASK;
+			    /* If there is at least one port label with this	*/
+			    /* name, then ignore all non-port labels with the	*/
+			    /* same name.					*/
+			    if ((hasPort) && (!(lab->lab_flags & PORT_DIR_MASK)))
+				break;
+			    else if (GEO_RECTNULL(&lab->lab_rect))
+				break;
+			    else
+			    {
+				if (lab->lab_flags & PORT_DIR_MASK)
+				    pinNum = lab->lab_flags & PORT_NUM_MASK;
+				else
+				{
+				    Label *sl;
+				    int idx;
+
+				    pinNum = -1;
+				    for (sl = lefMacro->cd_labels; sl != NULL;
+						sl = sl->lab_next)
+				    {
+					if (sl->lab_flags & PORT_DIR_MASK)
+					{
+					    idx = sl->lab_flags & PORT_NUM_MASK;
+					    if (idx > pinNum) pinNum = idx;
+					}
+				    }
+				    pinNum++;
+				}
+				needRect = FALSE;
+				lab->lab_flags &= ~(PORT_USE_MASK | PORT_DIR_MASK |
+					PORT_CLASS_MASK | PORT_SHAPE_MASK);
+				lab->lab_flags |= pinNum | pinUse | pinDir | pinShape |
+					PORT_DIR_MASK;
+			    }
 			}
 		    }
+		    firstlab = (lab == NULL) ? NULL : lab->lab_next;
+
+		    if (needRect)
+		    {
+			if ((lab == NULL) && (firstport == TRUE))
+			    DBEraseLabelsByContent(lefMacro, NULL, -1, testpin);
+			LefReadPort(lefMacro, f, testpin, pinNum, pinDir, pinUse,
+				pinShape, oscale, lab);
+		    }
+		    else
+			LefSkipSection(f, NULL);
+		    firstport = FALSE;
 		}
 		else
-		    LefReadPort(lefMacro, f, pinname, pinNum, pinDir, pinUse, oscale);
+		    LefReadPort(lefMacro, f, testpin, pinNum, pinDir, pinUse,
+			    pinShape, oscale, NULL);
 		break;
 	    case LEF_CAPACITANCE:
 	    case LEF_ANTENNADIFF:
@@ -1411,7 +1659,6 @@ LefReadPin(lefMacro, f, pinname, pinNum, oscale, is_imported)
 	    case LEF_ANTENNAPARCUT:
 	    case LEF_ANTENNAMAX:
 	    case LEF_ANTENNAMAXSIDE:
-	    case LEF_SHAPE:
 	    case LEF_NETEXPR:
 		LefEndStatement(f);	/* Ignore. . . */
 		break;
@@ -1425,6 +1672,7 @@ LefReadPin(lefMacro, f, pinname, pinNum, oscale, is_imported)
 	}
 	if (keyword == LEF_PIN_END) break;
     }
+    if (testpin != pinname) freeMagic(testpin);
 }
 
 /*
@@ -1464,24 +1712,27 @@ LefEndStatement(f)
 
 enum lef_macro_keys {LEF_CLASS = 0, LEF_SIZE, LEF_ORIGIN,
 	LEF_SYMMETRY, LEF_SOURCE, LEF_SITE, LEF_PIN, LEF_OBS,
-	LEF_TIMING, LEF_FOREIGN, LEF_MACRO_END};
+	LEF_TIMING, LEF_FOREIGN, LEF_PROPERTY, LEF_MACRO_END};
 
 void
-LefReadMacro(f, mname, oscale, importForeign)
+LefReadMacro(f, mname, oscale, importForeign, doAnnotate)
     FILE *f;			/* LEF file being read	*/
     char *mname;		/* name of the macro 	*/
     float oscale;		/* scale factor um->magic units */
-    bool importForeign;		/* Whether we should try to read 
+    bool importForeign;		/* Whether we should try to read
 				 * in a cell.
 				 */
+    bool doAnnotate;		/* If true, ignore all macros that are
+				 * not already CellDefs.
+				 */
 {
     CellDef *lefMacro;
     HashEntry *he;
 
     char *token, tsave[128], *propval;
-    int keyword, pinNum;
+    int keyword, pinNum, propsize;
     float x, y;
-    bool has_size, is_imported = FALSE;
+    bool has_size, is_imported = FALSE, propfound;
     Rect lefBBox;
 
     static char *macro_keys[] = {
@@ -1495,6 +1746,7 @@ LefReadMacro(f, mname, oscale, importForeign)
 	"OBS",
 	"TIMING",
 	"FOREIGN",
+	"PROPERTY",
 	"END",
 	NULL
     };
@@ -1517,6 +1769,12 @@ LefReadMacro(f, mname, oscale, importForeign)
         lefMacro = DBCellLookDef(newname);
         if (lefMacro == NULL)
 	{
+	    if (doAnnotate)
+	    {
+		/* Ignore any macro that does not correspond to an existing cell */
+		LefSkipSection(f, "MACRO");
+		return;
+	    }
 	    lefMacro = lefFindCell(newname);
 	    DBCellClearDef(lefMacro);
 	    DBCellSetAvail(lefMacro);
@@ -1560,10 +1818,14 @@ LefReadMacro(f, mname, oscale, importForeign)
 	switch (keyword)
 	{
 	    case LEF_CLASS:
+		strcpy(tsave, "");
 		token = LefNextToken(f, TRUE);
-		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFclass", StrDup((char **)NULL, token));
-		LefEndStatement(f);
+		while (*token != ';')
+		{
+		    sprintf(tsave + strlen(tsave), " %s", token);
+		    token = LefNextToken(f, TRUE);
+		}
+		DBPropPut(lefMacro, "LEFclass", StrDup((char **)NULL, tsave + 1));
 		break;
 	    case LEF_SIZE:
 		token = LefNextToken(f, TRUE);
@@ -1583,11 +1845,7 @@ size_error:
 		LefEndStatement(f);
 		break;
 	    case LEF_ORIGIN:
-		token = LefNextToken(f, TRUE);
-		if (!token || sscanf(token, "%f", &x) != 1) goto origin_error;
-		token = LefNextToken(f, TRUE);
-		if (!token || sscanf(token, "%f", &y) != 1) goto origin_error;
-
+		if (LefReadLefPoint(f, &x, &y) != 0) goto origin_error;
 		lefBBox.r_xbot = -(int)roundf(x / oscale);
 		lefBBox.r_ybot = -(int)roundf(y / oscale);
 		if (has_size)
@@ -1613,8 +1871,8 @@ origin_error:
 		break;
 	    case LEF_SOURCE:
 		token = LefNextToken(f, TRUE);
-		if (*token != '\n')
-		    DBPropPut(lefMacro, "LEFsource", StrDup((char **)NULL, token));
+		/* Ignore "SOURCE" as it is deprecated from LEF 5.6, and    */
+		/* magic will write LEF version 5.7.			    */
 		LefEndStatement(f);
 		break;
 	    case LEF_SITE:
@@ -1623,6 +1881,31 @@ origin_error:
 		    DBPropPut(lefMacro, "LEFsite", StrDup((char **)NULL, token));
 		LefEndStatement(f);
 		break;
+	    case LEF_PROPERTY:
+		/* Append property key:value pairs to the cell property LEFproperties */
+		propval = (char *)DBPropGet(lefMacro, "LEFproperties", &propfound);
+		if (propfound)
+		    propsize = strlen(propval);
+		else
+		    propsize = 0;
+
+		token = LefNextToken(f, TRUE);
+		if (*token != '\n')
+		{
+		    char *propext;
+		    sprintf(tsave, "%.127s", token);
+		    token = LefNextToken(f, TRUE);
+		    propext = (char *)mallocMagic(propsize + strlen(tsave) +
+			    strlen(token) + 4);
+		    if (propsize > 0)
+			sprintf(propext, "%s %s %s", propval, tsave, token);
+		    else
+			sprintf(propext, "%s %s", tsave, token);
+		    
+		    DBPropPut(lefMacro, "LEFproperties", StrDup((char **)NULL, propext));
+		}
+		LefEndStatement(f);
+		break;
 	    case LEF_PIN:
 		token = LefNextToken(f, TRUE);
 		/* Diagnostic */
@@ -1737,10 +2020,18 @@ origin_error:
  *
  * For LEF contact types matching magic contact types, size the
  * LEF contact cut to cover the minimum	rectangle in the other
- * layers that satisfies the CIF/GDS contact generation.  Use
- * the "cifinput" style	to determine how much the via layer
- * needs to grow to make a contact area.  If the "cifinput"
- * style is not	defined, then determine rules from "cifoutput".
+ * layers that satisfies the CIF/GDS contact generation.
+ *
+ * NOTE:  If a "cifinput" style is defined, then the via
+ * remains the size declared in LEF or DEF file, and the
+ * magic view of the via is generated by applying "cifinput"
+ * rules when painting into the magic database.  If no input
+ * style is defined, then the output style rules are used to
+ * modify the cut size to match the way the via is defined in
+ * magic, and the result is painted directly.
+ *
+ * If a "cifinput" style exists, then this routine does
+ * nothing and has no side effects.
  *
  *------------------------------------------------------------
  */
@@ -1750,36 +2041,10 @@ void LefGrowVia(curlayer, currect, lefl)
     Rect *currect;
     lefLayer *lefl;
 {
-    if (DBIsContact(curlayer) && cifCurReadStyle != NULL)
-    {
-	int growSize;
+    /* To be completed:  This should be deprecated by moving the entire	*/
+    /* LEF and DEF read routines to use the cifinput style.		*/
 
-	/* Get the amount (in magic units) that the layer needs to	*/
-	/* expand according to the "cifinput" style rules to convert	*/
-	/* a contact cut to a magic contact layer.			*/
-
-	growSize = CIFReadGetGrowSize(curlayer);
-
-	/* All internal LEF via geometry values are doubled */
-	growSize <<= 1;	
-
-        if (growSize % cifCurReadStyle->crs_scaleFactor == 0)
-	   growSize /= cifCurReadStyle->crs_scaleFactor;
-	else
-	   growSize = growSize / cifCurReadStyle->crs_scaleFactor + 1;
-
-	if (growSize > 0)
-	{
-	    /* cifinput styles expect the cut size to be correct, so	*/
-	    /* there is no check for correctness of the layer.		*/
-
-	    currect->r_xbot = currect->r_xbot - growSize;
-	    currect->r_ybot = currect->r_ybot - growSize;
-	    currect->r_xtop = currect->r_xtop + growSize;
-	    currect->r_ytop = currect->r_ytop + growSize;
-	}
-    }
-    else if (DBIsContact(curlayer) && CIFCurStyle != NULL)
+    if (DBIsContact(curlayer) && CIFCurStyle != NULL)
     {
 	int edgeSize = 0, contSize, halfSize;
 
@@ -1787,8 +2052,8 @@ void LefGrowVia(curlayer, currect, lefl)
 	contSize = CIFGetContactSize(curlayer, &edgeSize, NULL, NULL);
 
 	/* All internal LEF via geometry values are doubled */
-	contSize <<= 1;	
-	edgeSize <<= 1;	
+	contSize <<= 1;
+	edgeSize <<= 1;
 
         if (contSize % CIFCurStyle->cs_scaleFactor == 0)
 	   contSize /= CIFCurStyle->cs_scaleFactor;
@@ -1915,7 +2180,7 @@ LefGenViaGeometry(f, lefl, sizex, sizey, spacex, spacey,
 	    lefl->info.via.lr = viaLR;
 	    viaLR->r_type = clayer;
 	    viaLR->r_r = rect;
-	    
+
 	    y += sizey + spacey;
 	}
 	x += sizex + spacex;
@@ -1965,7 +2230,7 @@ LefAddViaGeometry(f, lefl, curlayer, oscale)
 	lefl->info.via.area = *currect;
 	lefl->type = curlayer;
     }
-    else 
+    else
     {
 	viaLR = (LinkedRect *)mallocMagic(sizeof(LinkedRect));
 	viaLR->r_next = lefl->info.via.lr;
@@ -2249,14 +2514,15 @@ enum lef_sections {LEF_VERSION = 0,
 	LEF_PROPERTYDEFS, LEF_UNITS, LEF_SECTION_LAYER,
 	LEF_SECTION_VIA, LEF_SECTION_VIARULE, LEF_SECTION_NONDEFAULTRULE,
 	LEF_NOWIREEXTENSIONATPIN, LEF_SECTION_SPACING, LEF_SECTION_SITE,
-	LEF_PROPERTY, LEF_NOISETABLE, LEF_CORRECTIONTABLE, LEF_IRDROP,
+	LEF_NOISETABLE, LEF_CORRECTIONTABLE, LEF_IRDROP,
 	LEF_ARRAY, LEF_SECTION_TIMING, LEF_EXTENSION, LEF_MACRO,
 	LEF_END};
 
 void
-LefRead(inName, importForeign)
+LefRead(inName, importForeign, doAnnotate)
     char *inName;
     bool importForeign;
+    bool doAnnotate;
 {
     FILE *f;
     char *filename;
@@ -2284,7 +2550,6 @@ LefRead(inName, importForeign)
 	"NOWIREEXTENSIONATPIN",
 	"SPACING",
 	"SITE",
-	"PROPERTY",
 	"NOISETABLE",
 	"CORRECTIONTABLE",
 	"IRDROP",
@@ -2455,15 +2720,6 @@ LefRead(inName, importForeign)
 	    case LEF_SECTION_SPACING:
 		LefSkipSection(f, sections[LEF_SECTION_SPACING]);
 		break;
-	    case LEF_SECTION_SITE:
-		token = LefNextToken(f, TRUE);
-		LefError(LEF_INFO, "Defines site %s (ignored)\n", token);
-		sprintf(tsave, "%.127s", token);
-		LefSkipSection(f, tsave);
-		break;
-	    case LEF_PROPERTY:
-		LefSkipSection(f, NULL);
-		break;
 	    case LEF_NOISETABLE:
 		LefSkipSection(f, sections[LEF_NOISETABLE]);
 		break;
@@ -2482,6 +2738,7 @@ LefRead(inName, importForeign)
 	    case LEF_EXTENSION:
 		LefSkipSection(f, sections[LEF_EXTENSION]);
 		break;
+	    case LEF_SECTION_SITE:
 	    case LEF_MACRO:
 		token = LefNextToken(f, TRUE);
 		/* Diagnostic */
@@ -2489,7 +2746,7 @@ LefRead(inName, importForeign)
 		TxPrintf("LEF file:  Defines new cell %s\n", token);
 		*/
 		sprintf(tsave, "%.127s", token);
-		LefReadMacro(f, tsave, oscale, importForeign);
+		LefReadMacro(f, tsave, oscale, importForeign, doAnnotate);
 		break;
 	    case LEF_END:
 		if (LefParseEndStatement(f, "LIBRARY") == 0)
