@@ -105,7 +105,7 @@ ResFixPoint		*ResFixList;
 
 extern void ResSimProcessDrivePoints();
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -128,7 +128,7 @@ ResReadSim(simfile,fetproc,capproc,resproc,attrproc,mergeproc)
      char line[MAXLINE][MAXTOKEN];
      int	result,fettype,extfile;
      FILE *fp, *fopen();
-     
+
          fp = PaOpen(simfile,"r",".sim",".",(char *) NULL, (char **) NULL);
          if (fp == NULL)
          {
@@ -143,11 +143,11 @@ ResReadSim(simfile,fetproc,capproc,resproc,attrproc,mergeproc)
 	       fettype = MINFINITY;
 	       switch(line[0][0])
 	       {
-	       	    case '|': 	
+	       	    case '|':
 		    		if (strcmp(line[NODEUNITS],"units:") == 0)
 				{
 				    resscale = (float)atof(line[NODELAMBDA]);
-				    if (resscale == 0.0) resscale = 1.0; 
+				    if (resscale == 0.0) resscale = 1.0;
 				}
 				result=0;
 				break;
@@ -167,7 +167,7 @@ ResReadSim(simfile,fetproc,capproc,resproc,attrproc,mergeproc)
 		    		break;
 		    case '=':   if (mergeproc)result = (*mergeproc)(line);
 		    		break;
-		    case 'A':	if (attrproc) result = 
+		    case 'A':	if (attrproc) result =
 		    			(*attrproc)(line[ATTRIBUTENODENAME],
 						    line[ATTRIBUTEVALUE],
 						    simfile, &extfile);
@@ -203,7 +203,7 @@ ResReadSim(simfile,fetproc,capproc,resproc,attrproc,mergeproc)
      return(result);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -220,7 +220,7 @@ ResReadSim(simfile,fetproc,capproc,resproc,attrproc,mergeproc)
 int
 ResReadNode(nodefile)
 	char	*nodefile;
- 
+
 {
     char line[MAXLINE][MAXTOKEN];
     FILE *fp, *fopen();
@@ -229,13 +229,13 @@ ResReadNode(nodefile)
     char *cp;
     float lambda;
 
-    /* NOTE:  Units from the .sim file or the .nodes file are in centimicrons
-     * when multiplied by resscale (units from the .sim file 1st line).
-     * multiply resscale by the extract scale (exts_unitsPerLambda) used to
-     * generate .ext dimensions originally, to get back to database units.
+    /* NOTE:  Units from the .nodes file are in centimicrons.
+     * Divide by the extract scale (exts_unitsPerLambda) to get back
+     * to database units.  This assumes that exts_unitsPerLambda doesn't
+     * change between output and readback.
      */
-    lambda = resscale * (float)ExtCurStyle->exts_unitsPerLambda;
-     
+    lambda = (float)ExtCurStyle->exts_unitsPerLambda;
+
     fp = PaOpen(nodefile,"r",".nodes",".", (char *) NULL, (char **) NULL);
     if (fp == NULL)
     {
@@ -246,10 +246,10 @@ ResReadNode(nodefile)
     {
 	entry = HashFind(&ResNodeTable,line[NODENODENAME]);
 	node = ResInitializeNode(entry);
-	      
+
 	node->location.p_x = (int)((float)atof(line[NODENODEX]) / lambda);
 	node->location.p_y = (int)((float)atof(line[NODENODEY]) / lambda);
-#ifdef ARIEL	      
+#ifdef ARIEL
 	node->rs_bbox.r_xbot = (int)((float)atof(line[NODE_BBOX_LL_X]) / lambda);
 	node->rs_bbox.r_ybot = (int)((float)atof(line[NODE_BBOX_LL_Y]) / lambda);
 	node->rs_bbox.r_xtop = (int)((float)atof(line[NODE_BBOX_UR_X]) / lambda);
@@ -269,12 +269,12 @@ ResReadNode(nodefile)
     (void)fclose(fp);
     return(0);
 }
-
+
 /*
  *-------------------------------------------------------------------------
  *
  * getline-- Gets a  line from the current input file and breaks it into
- *	tokens. 
+ *	tokens.
  *
  * Results:returns the number of tokens in the current line
  *
@@ -291,7 +291,7 @@ gettokens(line,fp)
 {
      int i=0,j=0;
      int c;
-     
+
      while ((c = getc(fp)) != EOF && c != '\n')
      {
      	  switch(c)
@@ -316,7 +316,7 @@ gettokens(line,fp)
      return(i);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -341,7 +341,7 @@ ResSimDevice(line,rpersquare,ttype)
      char		*newattr,tmpattr[MAXTOKEN];
      static int		nowarning = TRUE;
      float		lambda;
-     
+
      device = (RDev *) mallocMagic((unsigned) (sizeof(RDev)));
      if ((line[RDEV_WIDTH][0] == '\0') || (line[RDEV_LENGTH][0] == '\0'))
      {
@@ -358,11 +358,10 @@ ResSimDevice(line,rpersquare,ttype)
 	   }
 	   device->resistance = MagAtof(line[RDEV_LENGTH]) * rpersquare/MagAtof(line[RDEV_WIDTH]);
      }
-     device->tnumber = ++Maxtnumber;
      device->status = FALSE;
      device->nextDev = ResRDevList;
 
-     lambda = resscale * (float)ExtCurStyle->exts_unitsPerLambda;
+     lambda = (float)ExtCurStyle->exts_unitsPerLambda / resscale;
      device->location.p_x = (int)((float)atof(line[RDEV_DEVX]) / lambda);
      device->location.p_y = (int)((float)atof(line[RDEV_DEVY]) / lambda);
 
@@ -370,7 +369,7 @@ ResSimDevice(line,rpersquare,ttype)
      device->rs_sattr=RDEV_NOATTR;
      device->rs_dattr=RDEV_NOATTR;
      device->rs_ttype = ttype;
-     
+
      /* sim attributes look like g=a1,a2   	 */
      /* ext attributes are "a1","a2"	   	 */
      /* do conversion from one to the other here */
@@ -411,11 +410,11 @@ ResSimDevice(line,rpersquare,ttype)
      rvalue = ResSimNewNode(line[GATE],GATE,device) +
      	      ResSimNewNode(line[SOURCE],SOURCE,device) +
      	      ResSimNewNode(line[DRAIN],DRAIN,device);
-     
+
      return(rvalue);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -438,7 +437,7 @@ ResSimNewNode(line,type,device)
      HashEntry		*entry;
      ResSimNode		*node;
      devPtr		*tptr;
-     
+
      if (line[0] == '\0')
      {
      	  TxError("Missing device connection\n");
@@ -465,12 +464,12 @@ ResSimNewNode(line,type,device)
      return(0);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
  *  ResSimCapacitor-- Adds the capacitance  from a C line to the appropriate
- *	node. Coupling capacitors are added twice, moving the capacitance 
+ *	node. Coupling capacitors are added twice, moving the capacitance
  *	to the substrate.
  *
  *  Results:
@@ -488,7 +487,7 @@ ResSimCapacitor(line)
 {
      HashEntry		*entry1,*entry2;
      ResSimNode		*node1,*node2;
-     
+
      if (line[COUPLETERMINAL1][0] == 0 || line[COUPLETERMINAL2][0] == 0)
      {
      	  TxError("Bad Capacitor\n");
@@ -499,7 +498,7 @@ ResSimCapacitor(line)
      if (ResOptionsFlags & ResOpt_Signal)
      {
           node1->capacitance += MagAtof(line[COUPLEVALUE]);
-          if (strcmp(line[COUPLETERMINAL2],"GND") == 0 || 
+          if (strcmp(line[COUPLETERMINAL2],"GND") == 0 ||
      	  strcmp(line[COUPLETERMINAL2],"Vdd") == 0)
           {
                return(0);
@@ -534,15 +533,15 @@ ResSimCapacitor(line)
      node1->cap_couple += MagAtof(line[COUPLEVALUE]);
      node2->cap_couple += MagAtof(line[COUPLEVALUE]);
      return(0);
-     
+
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
  *  ResSimResistor-- Adds the capacitance  from a R line to the appropriate
- *	node. 
+ *	node.
  *
  *  Results
  *	Return 0 to keep search going, 1 to abort
@@ -559,7 +558,7 @@ ResSimResistor(line)
 {
      HashEntry		*entry;
      ResSimNode		*node;
-     
+
      if (line[RESNODENAME][0] == 0)
      {
      	  TxError("Bad Resistor\n");
@@ -577,11 +576,11 @@ ResSimResistor(line)
 }
 
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
- *  ResSimAttribute--checks to see if a node attribute is a resistance 
+ *  ResSimAttribute--checks to see if a node attribute is a resistance
  *	attribute. If it is, add it to the correct node's status flag.
  *	Only works with 5.0 1/line attributes
  *
@@ -604,7 +603,7 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
      char		digit[MAXDIGIT];
      int		i;
      static int		notwarned=TRUE;
-     
+
      if (aname[0] == 0)
      {
      	  TxError("Bad Resistor\n");
@@ -620,7 +619,7 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
 	  }
 	  else
 	  {
-	       node->status |= SKIP;	
+	       node->status |= SKIP;
 	  }
      }
      else if (strncmp(avalue,"res:force",9) == 0)
@@ -631,10 +630,10 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
 	  }
 	  else
 	  {
-	       node->status |= FORCE;	
+	       node->status |= FORCE;
 	  }
      }
-     else if (strncmp(avalue,"res:min=",8) == 0)     	  
+     else if (strncmp(avalue,"res:min=",8) == 0)
      {
 	 node->status |= MINSIZE;
 	 for(i=0,avalue += 8; *avalue != '\0' && *avalue != ','; avalue++)
@@ -653,7 +652,7 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
 	       *readextfile = 1;
 	  }
 	  /* is the attribute in root.ext?    */
-	  if (node->drivepoint.p_x != INFINITY) 
+	  if (node->drivepoint.p_x != INFINITY)
 	  {
 	       node->status |= DRIVELOC;
 	  }
@@ -665,7 +664,7 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
 	  }
      }
 #ifdef ARIEL
-     else if (strncmp(avalue,"res:fix",7) == 0 && 
+     else if (strncmp(avalue,"res:fix",7) == 0 &&
      	      (ResOptionsFlags & ResOpt_Power))
      {
 	  if (*readextfile == 0)
@@ -675,14 +674,14 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
 	  }
      }
 #endif
-     if (avalue = strchr(avalue,',')) 
+     if (avalue = strchr(avalue,','))
      {
           (void) ResSimAttribute(aname,avalue+1,rootname,readextfile);
      }
      return(0);
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -690,7 +689,7 @@ ResSimAttribute(aname,avalue,rootname,readextfile)
  *	and we are doing a signal extraction,
  *	we need to search through the .ext file looking for attr labels that
  *	contain this text. For efficiency, the .ext file is only parsed when
- *	the first res:drive is encountered.  res:drive labels only work if 
+ *	the first res:drive is encountered.  res:drive labels only work if
  *	they are in the root cell.
  *
  * Results:
@@ -710,7 +709,7 @@ ResSimProcessDrivePoints(filename)
      FILE	*fp;
      HashEntry	*entry;
      ResSimNode	*node;
-     
+
      fp = PaOpen(filename,"r",".ext",".",(char *) NULL,(char **) NULL);
      if (fp == NULL)
      {
@@ -721,7 +720,7 @@ ResSimProcessDrivePoints(filename)
      {
      	  if (strncmp(line[RES_EXT_ATTR],"attr",4) != 0 ||
 	      strncmp(line[RES_EXT_ATTR_TEXT],"\"res:drive\"",11) != 0) continue;
-	  
+
 	 entry = HashFind(&ResNodeTable,line[RES_EXT_ATTR_NAME]);
 	 node = ResInitializeNode(entry);
 	 node->drivepoint.p_x = atoi(line[RES_EXT_ATTR_X]);
@@ -730,12 +729,12 @@ ResSimProcessDrivePoints(filename)
      }
 }
 
-
+
 /*
  *-------------------------------------------------------------------------
  *
- * ResSimProcessFixPoints -- if the sim file contains a "res:fix:name" label 
- *	and we are checking for power supply noise, then we have to 
+ * ResSimProcessFixPoints -- if the sim file contains a "res:fix:name" label
+ *	and we are checking for power supply noise, then we have to
  *	parse the .ext file looking for the fix label locations.  This
  *	is only done after the first res:fix label is encountered.
  *
@@ -756,7 +755,7 @@ ResSimProcessFixPoints(filename)
      char	line[MAXLINE][MAXTOKEN],*label,*c;
      FILE	*fp;
      ResFixPoint	*thisfix;
-     
+
      fp = PaOpen(filename,"r",".ext",".",(char *) NULL,(char **) NULL);
      if (fp == NULL)
      {
@@ -772,7 +771,7 @@ ResSimProcessFixPoints(filename)
 	  if (*label == ':') label++;
 	  if ((c=strrchr(label,'"')) != NULL) *c='\0';
 	  else if (*label == '\0');
-	  else 
+	  else
 	  {
 	       TxError("Bad res:fix attribute label %s\n",
 	       			line[RES_EXT_ATTR_TEXT]);
@@ -789,7 +788,7 @@ ResSimProcessFixPoints(filename)
      }
 }
 
-
+
  /*
  *-------------------------------------------------------------------------
  *
@@ -811,7 +810,7 @@ ResSimMerge(line)
 {
      ResSimNode		*node;
      devPtr		*ptr;
-     
+
      if ((line[ALIASNAME][0] == '\0') || (line[REALNAME][0] == '\0'))
      {
      	  TxError("Bad node alias line\n");
@@ -831,7 +830,7 @@ ResSimMerge(line)
      }
      return(0);
 }
-
+
 /*
  *-------------------------------------------------------------------------
  *
