@@ -6,6 +6,8 @@
 # Revision 0
 # December 15, 2016
 # Revision 1
+# October 29, 2020
+# Revision 2	(names are hashed from properties)
 #--------------------------------------------------------------
 # Sets up the environment for a toolkit.  The toolkit must
 # supply a namespace that is the "library name".  For each
@@ -150,7 +152,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
     }
     set argpar [dict create {*}$args]
 
-    if {$gencell_name == {}} { 
+    if {$gencell_name == {}} {
 	# Find selected item  (to-do:  handle multiple selections)
 
 	set wlist [what -list]
@@ -204,7 +206,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
 		if {[dict exists $parameters nocell]} {
 		    set arcount [array -list count]
 		    set arpitch [array -list pitch]
-	
+
 		    dict set parameters nx [lindex $arcount 1]
 		    dict set parameters ny [lindex $arcount 3]
 		    dict set parameters pitchx $delx
@@ -237,6 +239,36 @@ proc magic::gencell {gencell_name {instname {}} args} {
     }
 }
 
+#-------------------------------------------------------------
+# gencell_makecell
+#
+# This is a variation of magic::gencell and is used to generate
+# a cell and return the cell name without creating or placing
+# an instance.
+#-------------------------------------------------------------
+
+proc magic::gencell_makecell {gencell_fullname args} {
+
+    set argpar [dict create {*}$args]
+    set gencell_basename [namespace tail $gencell_fullname]
+    set library [namespace qualifiers $gencell_fullname]
+    set parameters [magic::gencell_defaults $gencell_basename $library $argpar]
+    set gsuffix [magic::get_gencell_hash ${parameters}]
+    set gname ${gencell_basename}_${gsuffix}
+    suspendall
+    cellname create $gname
+    pushstack $gname
+    if {[catch {${library}::${gencell_basename}_draw $parameters} drawerr]} {
+        puts stderr $drawerr
+    }
+    property library $library
+    property gencell $gencell_basename
+    property parameters $parameters
+    popstack
+    resumeall
+    return $gname
+}
+
 #-------------------------------------------------------------
 # gencell_getparams
 #
@@ -271,6 +303,9 @@ proc magic::gencell_setparams {parameters} {
    if {[catch {set state [wm state .params]}]} {return}
    set slist [grid slaves .params.edits]
    foreach s $slist {
+      # ignore .params.edits.gencell_sel, as that does not exist in the
+      # parameters dictionary
+      if {$s == ".params.edits.gencell_sel"} {continue}
       if {[regexp {^.params.edits.(.*)_ent$} $s valid pname] != 0} {
 	 set value [dict get $parameters $pname]
          set magic::${pname}_val $value
@@ -280,6 +315,7 @@ proc magic::gencell_setparams {parameters} {
       } elseif {[regexp {^.params.edits.(.*)_sel$} $s valid pname] != 0} {
 	 set value [dict get $parameters $pname]
          set magic::${pname}_val $value
+	 .params.edits.${pname}_sel configure -text $value
       } elseif {[regexp {^.params.edits.(.*)_txt$} $s valid pname] != 0} {
 	 if {[dict exists $parameters $pname]} {
 	    set value [dict get $parameters $pname]
@@ -292,7 +328,16 @@ proc magic::gencell_setparams {parameters} {
 #-------------------------------------------------------------
 # gencell_change
 #
-#   Redraw a gencell with new parameters.
+#   Recreate a gencell with new parameters.  Note that because
+#   each cellname is uniquely identified by the (hashed) set
+#   of parameters, changing parameters effectively means
+#   creating a new cell.  If the original cell has parents
+#   other than the parent of the instance being changed, then
+#   it is retained;  otherwise, it is deleted.  The instance
+#   being edited gets replaced by an instance of the new cell.
+#   If the instance name was the cellname + suffix, then the
+#   instance name is regenerated.  Otherwise, the instance
+#   name is retained.
 #-------------------------------------------------------------
 
 proc magic::gencell_change {instname gencell_type library parameters} {
@@ -310,11 +355,140 @@ proc magic::gencell_change {instname gencell_type library parameters} {
 	if {$newinstname == $instname} {set newinstname $instname}
 	if {[instance list exists $newinstname] != ""} {set newinstname $instname}
     }
+    if {[dict exists $parameters gencell]} {
+        # Setting special parameter "gencell" forces the gencell to change type
+	set gencell_type [dict get $parameters gencell]
+    }
     if {[catch {set parameters [${library}::${gencell_type}_check $parameters]} \
 		checkerr]} {
 	puts stderr $checkerr
     }
     magic::gencell_setparams $parameters
+    if {[dict exists $parameters gencell]} {
+	set parameters [dict remove $parameters gencell]
+    }
+
+    set old_gname [instance list celldef $instname]
+    set gsuffix [magic::get_gencell_hash ${parameters}]
+    set gname ${gencell_type}_${gsuffix}
+
+    # Guard against instance having been deleted.  Also, if parameters have not
+    # changed as evidenced by the cell suffix not changing, then nothing further
+    # needs to be done.
+    if {$gname == "" || $gname == $old_gname} {
+	resumeall
+        return
+    }
+
+    set snaptype [snap list]
+    snap internal
+    set savebox [box values]
+
+    catch {setpoint 0 0 $Opts(focus)}
+    if [dict exists $parameters nocell] {
+        select cell $instname
+	set abox [instance list abutment]
+	delete
+	if {$abox != ""} {box values {*}$abox}
+	if {[catch {set newinst [${library}::${gencell_type}_draw $parameters]} \
+		drawerr]} {
+	    puts stderr $drawerr
+	}
+        select cell $newinst
+    } elseif {[cellname list exists $gname] != 0} {
+	# If there is already a cell of this type then it is only required to
+	# remove the instance and replace it with an instance of the cell
+        select cell $instname
+	# check rotate/flip before replacing and replace with same
+	set orient [instance list orientation]
+	set abox [instance list abutment]
+	delete
+
+	if {$abox != ""} {box values {*}$abox}
+	set newinstname [getcell $gname $orient]
+        select cell $newinstname
+	expand
+
+	# If the old instance name was not formed from the old cell name,
+	# then keep the old instance name.
+	if {[string first $old_gname $instname] != 0} {
+	    set newinstname $instname
+	}
+
+	if {[cellname list parents $old_gname] == []} {
+	    # If the original cell has no intances left, delete it.  It can
+	    # be regenerated if and when necessary.
+	    cellname delete $old_gname
+	}
+
+    } else {
+        select cell $instname
+	set orient [instance list orientation]
+	set abox [instance list abutment]
+	delete
+
+	# There is no cell of this name, so generate one and instantiate it.
+	if {$abox != ""} {box values {*}$abox}
+	set newinstname [magic::gencell_create $gencell_type $library $parameters $orient]
+	select cell $newinstname
+
+	# If the old instance name was not formed from the old cell name,
+	# then keep the old instance name.
+	if {[string first $old_gname $instname] != 0} {
+	    set newinstname $instname
+	}
+    }
+    identify $newinstname
+    eval "box values $savebox"
+    snap $snaptype
+
+    # Update window
+    if {$gname != $old_gname} {
+        catch {.params.title.glab configure -text "$gname"}
+    }
+    if {$instname != $newinstname} {
+        catch {.params.title.ient delete 0 end}
+        catch {.params.title.ient insert 0 "$newinstname"}
+    }
+
+    resumeall
+    redraw
+}
+
+#-------------------------------------------------------------
+# gencell_change_orig
+#
+#   Original version:  Redraw a gencell with new parameters,
+#   without changing the cell itself.
+#-------------------------------------------------------------
+
+proc magic::gencell_change_orig {instname gencell_type library parameters} {
+    global Opts
+    suspendall
+
+    set newinstname $instname
+    if {$parameters == {}} {
+        # Get device defaults
+	set pdefaults [${library}::${gencell_type}_defaults]
+        # Pull user-entered values from dialog
+        set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+	set newinstname [.params.title.ient get]
+	if {$newinstname == "(default)"} {set newinstname $instname}
+	if {$newinstname == $instname} {set newinstname $instname}
+	if {[instance list exists $newinstname] != ""} {set newinstname $instname}
+    }
+    if {[dict exists $parameters gencell]} {
+        # Setting special parameter "gencell" forces the gencell to change type
+	set gencell_type [dict get $parameters gencell]
+    }
+    if {[catch {set parameters [${library}::${gencell_type}_check $parameters]} \
+		checkerr]} {
+	puts stderr $checkerr
+    }
+    magic::gencell_setparams $parameters
+    if {[dict exists $parameters gencell]} {
+	set parameters [dict remove $parameters gencell]
+    }
 
     set gname [instance list celldef $instname]
 
@@ -346,6 +520,7 @@ proc magic::gencell_change {instname gencell_type library parameters} {
 	    puts stderr $drawerr
 	}
 	property parameters $parameters
+	property gencell ${gencell_type}
 	tech revert
 	popstack
         select cell $instname
@@ -376,12 +551,57 @@ proc magic::get_gencell_name {gencell_type} {
 	    set pint [expr 48 + int(rand() * 36)]
 	    if {$pint > 57} {set pint [expr $pint + 39]}
 	    append postfix [format %c $pint]
-	}   
+	}
 	if {[cellname list exists ${gencell_type}_$postfix] == 0} {break}
     }
     return ${gencell_type}_$postfix
 }
 
+#----------------------------------------------------------------
+# get_gencell_hash
+#
+#   A better approach to the above.  Take the parameter
+#   dictionary, and run all the values through a hash function
+#   to generate a 30-bit value, then convert to base32.  This
+#   gives a result that is repeatable for the same set of
+#   parameter values with a very low probability of a collision.
+#
+#   The hash function is similar to elfhash but reduced from 32
+#   to 30 bits so that the result can form a 6-character value
+#   in base32 with all characters being valid for a SPICE subcell
+#   name (e.g., alphanumeric only and case-insensitive).
+#----------------------------------------------------------------
+
+proc magic::get_gencell_hash {parameters} {
+    set hash 0
+    # Apply hash
+    dict for {key value} $parameters {
+	foreach s [split $value {}] {
+	    set hash [expr {($hash << 4) + [scan $s %c]}]
+	    set high [expr {$hash & 0x03c0000000}]
+	    set hash [expr {$hash ^ ($high >> 30)}]
+	    set hash [expr {$hash & (~$high)}]
+	}
+    }
+    # Divide hash up into 5 bit values and convert to base32
+    # using letters A-Z less I and O, and digits 2-9.
+    set cvals ""
+    for {set i 0} {$i < 6} {incr i} {
+	set oval [expr {($hash >> ($i * 5)) & 0x1f}]
+        if {$oval < 8} {
+	    set bval [expr {$oval + 50}]
+	} elseif {$oval < 16} {
+	    set bval [expr {$oval + 57}]
+	} elseif {$oval < 21} {
+	    set bval [expr {$oval + 58}]
+	} else {
+	    set bval [expr {$oval + 59}]
+	}
+	append cvals [binary format c* $bval]
+    }
+    return $cvals
+}
+
 #-------------------------------------------------------------
 # gencell_create
 #
@@ -393,21 +613,31 @@ proc magic::get_gencell_name {gencell_type} {
 #   the drawing routine is going to do to the stack!
 #-------------------------------------------------------------
 
-proc magic::gencell_create {gencell_type library parameters} {
+proc magic::gencell_create {gencell_type library parameters {orient 0}} {
     global Opts
     suspendall
 
     set newinstname ""
 
     # Get device defaults
-    set pdefaults [${library}::${gencell_type}_defaults]
     if {$parameters == {}} {
         # Pull user-entered values from dialog
-        set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+        set dialogparams [magic::gencell_getparams]
+	if {[dict exists $dialogparams gencell]} {
+	    # Setting special parameter "gencell" forces the gencell to change type
+	    set gencell_type [dict get $dialogparams gencell]
+	}
+	set pdefaults [${library}::${gencell_type}_defaults]
+        set parameters [dict merge $pdefaults $dialogparams]
 	set newinstname [.params.title.ient get]
 	if {$newinstname == "(default)"} {set newinstname ""}
 	if {[instance list exists $newinstname] != ""} {set newinstname ""}
     } else {
+	if {[dict exists $parameters gencell]} {
+	    # Setting special parameter "gencell" forces the gencell to change type
+	    set gencell_type [dict get $parameters gencell]
+	}
+	set pdefaults [${library}::${gencell_type}_defaults]
         set parameters [dict merge $pdefaults $parameters]
     }
 
@@ -416,6 +646,9 @@ proc magic::gencell_create {gencell_type library parameters} {
 	puts stderr $checkerr
     }
     magic::gencell_setparams $parameters
+    if {[dict exists $parameters gencell]} {
+	set parameters [dict remove $parameters gencell]
+    }
 
     set snaptype [snap list]
     snap internal
@@ -429,18 +662,19 @@ proc magic::gencell_create {gencell_type library parameters} {
 	set gname [instance list celldef $instname]
 	eval "box values $savebox"
     } else {
-        set gname [magic::get_gencell_name ${gencell_type}]
+        set gsuffix [magic::get_gencell_hash ${parameters}]
+        set gname ${gencell_type}_${gsuffix}
 	cellname create $gname
 	pushstack $gname
 	if {[catch {${library}::${gencell_type}_draw $parameters} drawerr]} {
 	    puts stderr $drawerr
 	}
-	property library $library 
+	property library $library
 	property gencell $gencell_type
 	property parameters $parameters
 	popstack
 	eval "box values $savebox"
-	set instname [getcell $gname]
+	set instname [getcell $gname $orient]
 	expand
     }
     if {$newinstname != ""} {
@@ -454,7 +688,7 @@ proc magic::gencell_create {gencell_type library parameters} {
 }
 
 #-----------------------------------------------------
-#  Add a standard entry parameter to the gencell window 
+#  Add a standard entry parameter to the gencell window
 #-----------------------------------------------------
 
 proc magic::add_entry {pname ptext parameters} {
@@ -464,7 +698,7 @@ proc magic::add_entry {pname ptext parameters} {
    } else {
        set value ""
    }
-   
+
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    entry .params.edits.${pname}_ent -background white -textvariable magic::${pname}_val
@@ -477,8 +711,8 @@ proc magic::add_entry {pname ptext parameters} {
 }
 
 #----------------------------------------------------------
-# Default entry callback, without any dependencies.  Each 
-# parameter changed 
+# Default entry callback, without any dependencies.  Each
+# parameter changed
 #----------------------------------------------------------
 
 proc magic::add_check_callbacks {gencell_type library} {
@@ -548,6 +782,14 @@ proc magic::add_dependency {callback gencell_type library args} {
 proc magic::update_dialog {callback pname gencell_type library} {
     set pdefaults [${library}::${gencell_type}_defaults]
     set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+
+    if {[dict exists $parameters gencell]} {
+        # Setting special parameter "gencell" forces the gencell to change type
+	set gencell_type [dict get $parameters gencell]
+	set pdefaults [${library}::${gencell_type}_defaults]
+	set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+    }
+
     if {$callback != {}} {
        set parameters [$callback $pname $parameters]
     }
@@ -559,7 +801,7 @@ proc magic::update_dialog {callback pname gencell_type library} {
 }
 
 #----------------------------------------------------------
-#  Add a standard checkbox parameter to the gencell window 
+#  Add a standard checkbox parameter to the gencell window
 #----------------------------------------------------------
 
 proc magic::add_checkbox {pname ptext parameters} {
@@ -569,7 +811,7 @@ proc magic::add_checkbox {pname ptext parameters} {
    } else {
        set value ""
    }
-   
+
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    checkbutton .params.edits.${pname}_chk -variable magic::${pname}_val
@@ -592,7 +834,7 @@ proc magic::add_message {pname ptext parameters {color blue}} {
    } else {
       set value ""
    }
-   
+
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    label .params.edits.${pname}_txt -text $value \
@@ -602,21 +844,21 @@ proc magic::add_message {pname ptext parameters {color blue}} {
 }
 
 #----------------------------------------------------------
-#  Add a selectable-list parameter to the gencell window 
+#  Add a selectable-list parameter to the gencell window
 #----------------------------------------------------------
 
-proc magic::add_selectlist {pname ptext all_values parameters} {
+proc magic::add_selectlist {pname ptext all_values parameters {itext ""}} {
 
    if [dict exists $parameters $pname] {
         set value [dict get $parameters $pname]
    } else {
-       set value ""
+       set value $itext
    }
 
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    menubutton .params.edits.${pname}_sel -menu .params.edits.${pname}_sel.menu \
-		-relief groove -text ${value} 
+		-relief groove -text ${value}
    grid .params.edits.${pname}_lab -row $numrows -column 0 -sticky ens
    grid .params.edits.${pname}_sel -row $numrows -column 1 -sticky wns
    menu .params.edits.${pname}_sel.menu -tearoff 0
@@ -628,6 +870,38 @@ proc magic::add_selectlist {pname ptext all_values parameters} {
    set magic::${pname}_val $value
 }
 
+#----------------------------------------------------------
+#  Add a selectable-list parameter to the gencell window
+#  Unlike the routine above, it returns the index of the
+#  selection, not the selection itself.  This is useful for
+#  keying the selection to other parameter value lists.
+#----------------------------------------------------------
+
+proc magic::add_selectindex {pname ptext all_values parameters {ival 0}} {
+
+   if [dict exists $parameters $pname] {
+        set value [dict get $parameters $pname]
+   } else {
+       set value $ival
+   }
+
+   set numrows [lindex [grid size .params.edits] 1]
+   label .params.edits.${pname}_lab -text $ptext
+   menubutton .params.edits.${pname}_sel -menu .params.edits.${pname}_sel.menu \
+		-relief groove -text [lindex ${all_values} ${value}]
+   grid .params.edits.${pname}_lab -row $numrows -column 0 -sticky ens
+   grid .params.edits.${pname}_sel -row $numrows -column 1 -sticky wns
+   menu .params.edits.${pname}_sel.menu -tearoff 0
+   set idx 0
+   foreach item ${all_values} {
+       .params.edits.${pname}_sel.menu add radio -label $item \
+	-variable magic::${pname}_val -value $idx \
+	-command ".params.edits.${pname}_sel configure -text $item"
+       incr idx
+   }
+   set magic::${pname}_val $value
+}
+
 #-------------------------------------------------------------
 # gencell_defaults ---
 #
@@ -713,9 +987,7 @@ proc magic::gencell_dialog {instname gencell_type library parameters} {
 	 set instname $instroot
       }
       set gname [instance list celldef [subst $instname]]
-      if {$gencell_type == {}} {
-	 set gencell_type [cellname list property $gname gencell]
-      }
+      set gencell_type [cellname list property $gname gencell]
       if {$library == {}} {
 	 set library [cellname list property $gname library]
       }
